-- Designed by Toshio Iwata at DIGITALFILTER.COM 2013/03/19

LIBRARY ieee;
USE ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY LINEOUT_CTRL IS
	PORT	(
		CLK	: IN std_logic;
		CLK100M	: IN std_logic;
		RST_N	: IN std_logic;
--		SoftReset_N	: IN std_logic;
		ViewMode : in std_logic_vector(3 downto 0);
--		BigLineN : in std_logic_vector(8 downto 0);
--		RD_ADDR_LOAD : in std_logic;
--		RD_ADDR_ENB : in std_logic;
--		WR_ADDR_CLR : in std_logic;
		PIXSTB: in std_logic;
		PIXWR : in std_logic;
		PIXRD : in std_logic;
  RD_BUF_SEL : out std_logic_vector(1 downto 0);
  WR_BUF_SEL : out std_logic_vector(1 downto 0);
  WR_BUF_SEL2 : out std_logic_vector(1 downto 0);
  WR_BUF_N : out std_logic;
  RD_BUF_ADDR : out std_logic_vector(7 downto 0);
  SsdLineCount : out std_logic_vector(8 downto 0);
  SsdLineCount2 : out std_logic_vector(8 downto 0);
  SsdPixCount : out std_logic_vector(8 downto 0);
  SsdPixCount2 : out std_logic_vector(8 downto 0);
  OutChSel : out std_logic_vector(3 downto 0);
  CO9 : out std_logic_vector(3 downto 0);
  CO512 : out std_logic_vector(8 downto 0);
  CO512_dly2 : out std_logic_vector(8 downto 0);
  PIX2LOGIC : in std_logic_vector(15 downto 0);
LBRDY : out std_logic;
CamHsync_edge : in std_logic;
VgaHsync_edge : in std_logic;
DISPJUDGE : in std_logic;
		latch0	: IN std_logic_vector(15 downto 0);
		latch1	: IN std_logic_vector(15 downto 0);
		latch2	: IN std_logic_vector(15 downto 0);
		latch3	: IN std_logic_vector(15 downto 0);
		CamLineCount : in std_logic_vector(8 downto 0);
		VgaLineCount	: in std_logic_vector(8 downto 0);
		VgaPixCount : IN std_logic_vector(9 downto 0);
		winp0: in std_logic;
		winp1: in std_logic;
		winp2: in std_logic;
		winp3: in std_logic );
END LINEOUT_CTRL;

ARCHITECTURE structure OF LINEOUT_CTRL IS

component SRAM_512x16
   Port (
          WRCLK : In std_logic;
          RDCLK : In std_logic;
          WR_N : In std_logic;
          WRADDR : In std_logic_vector(8 downto 0);
          RDADDR : In std_logic_vector(8 downto 0);
          WRDATA : In std_logic_vector(15 downto 0);
          RDDATA : Out std_logic_vector(15 downto 0) );
end component;

signal ODDLINE, oddline_dly1, oddline_dly2, oddline_dly3, oddline_edge : std_logic;
 signal ls2b : std_logic_vector(1 downto 0);
 signal LB_ADDR : std_logic_vector(7 downto 0);
 signal baseaddr : std_logic_vector(7 downto 0);
signal PIXRD_edge, PIXRD_dly1, PIXRD_dly2 : std_logic;
signal PIXWR_edge, PIXWR_dly1, PIXWR_dly2 : std_logic;
signal WaitLine : std_logic_vector(2 downto 0);
signal  pixstb_count : std_logic_vector(9 downto 0);
signal  pixstb_count_plus : std_logic_vector(9 downto 0);
signal LBRDY_sig : std_logic;
	signal ViewMode_dly1, ViewMode_dly2 : std_logic_vector(3 downto 0);
  signal ViewMode5_edge : std_logic;
  signal ViewMode6_edge : std_logic;
 signal count4wraddr : std_logic_vector(9 downto 0);
signal SsdStart : std_logic;
signal count512 : std_logic_vector(8 downto 0); -- counts input channels(+1)
signal count512_clr : std_logic;
signal count512_dly1, count512_dly2 : std_logic_vector(8 downto 0);
signal count3 : std_logic_vector(7 downto 0); -- counts horizontal 3 pixs
signal count9 : std_logic_vector(3 downto 0); -- counts 3x3 pixs
signal count9_clr : std_logic;
signal count9_dly1, count9_dly2 : std_logic_vector(3 downto 0);
signal count600 : std_logic_vector(10 downto 0); -- counts 300 areas
signal count600_clr : std_logic;
--signal count600_dly1, count600_dly2 : std_logic_vector(8 downto 0);
signal count19_64_dly1, count19_64_dly2, count19_64_dly3, count19_64_dly4 : std_logic_vector(6 downto 0);
signal count19_64_dly5, count19_64_dly6, count19_64_dly7, count19_64_dly8 : std_logic_vector(6 downto 0);
signal count19_64_dly9, count19_64_dly10, count19_64_dly11, count19_64_dly12 : std_logic_vector(6 downto 0);
signal count19 : std_logic_vector(6 downto 0); -- counts 300 areas
signal countV : std_logic_vector(8 downto 0);
signal countV_clr : std_logic;
signal rdbufsel : std_logic_vector(1 downto 0);
signal rdbufsel_dly1, rdbufsel_dly2 : std_logic_vector(1 downto 0);
signal loadval : std_logic_vector(1 downto 0);
signal WR_ADDR_CLR_dly1, WR_ADDR_CLR_dly2, WR_ADDR_CLR_edge : std_logic;
signal RD_ADDR_ENB_dly1, RD_ADDR_ENB_dly2, RD_ADDR_ENB_edge : std_logic;
signal WRN2DL_sig, DISPJUDGE_latch : std_logic;
signal countV_dly1, countV_dly2, countV_dly3, countV_dly4 : std_logic_vector(8 downto 0);
signal countV_dly5, countV_dly6, countV_dly7, countV_dly8 : std_logic_vector(8 downto 0);
signal countV_dly9, countV_dly10, countV_dly11, countV_dly12 : std_logic_vector(8 downto 0);
signal WR_N_X : std_logic_vector(1 downto 0);
signal SsdLineCount_tmp, SsdLineCount_dly1 : std_logic_vector(8 downto 0);

BEGIN
-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     oddline_dly3 <= '0';
     oddline_dly2 <= '0';
     oddline_dly1 <= '0';
   elsif( CLK'event and CLK = '1' ) then
     oddline_dly3 <= oddline_dly2;
     oddline_dly2 <= oddline_dly1;
     oddline_dly1 <= ODDLINE;
   end if;  
end process;

  ODDLINE <= VgaLineCount(1);
  oddline_edge <= '1' when (oddline_dly1 = '0' and oddline_dly2 = '1') else '0';

  WR_BUF_N <= '0' when count512_clr = '1' and count600(0) = '1' else '1';
  --WR_BUF_SEL <= countV_dly8(1 downto 0); -- for lay5 2022/05/25
  --WR_BUF_SEL2 <= countV_dly8(2 downto 1); -- because of pooling -- for lay4, needs more dly
  WR_BUF_SEL <= countV_dly4(1 downto 0); -- for lay8
  WR_BUF_SEL2 <= countV_dly12(2 downto 1); -- for lay7 more dly for addrs38

-------------------------------------------------------------- 
  WR_N_X <= countV(1 downto 0);

  WRN2DL_sig <= '0' when (winp0 = '1' or winp1 = '1' or winp2 = '1' or winp3 = '1') 
                     else '1';
                       
-------------------------------------------------------------------
  ls2b <= "00" when winp0 = '1' else
          "01" when winp1 = '1' else
          "10" when winp2 = '1' else
          "11";

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
	if( RST_N = '0' ) then
		LB_ADDR <= (others => '0');
	elsif( CLK100M'event and CLK100M = '1' ) then
		--LB_ADDR <= count600(9 downto 2) + count3; -- for lay9
		LB_ADDR <= baseaddr + count3; -- for lay11
	end if;  
end process;
          
baseaddr <= ("0" & count600(10 downto 4)); -- for lay11

OutChSel <= count600(3 downto 0); -- for lay11 

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then --or SoftReset_N = '0' ) then
     countV <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
       if(SsdStart = '1' ) then
         countV <= (others => '0');
       elsif(count600_clr = '1' and countV /= 63) then 
         countV <= countV + 1;
       end if;
   end if;
end process;

--countV_clr <= '1' when countV = 299 and count600 = 299 and count512 = 63 and count9 = 8 else '0';

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then --or SoftReset_N = '0' ) then
     countV_dly12 <= (others => '0');
     countV_dly11 <= (others => '0');
     countV_dly10 <= (others => '0');
     countV_dly9 <= (others => '0');
     countV_dly8 <= (others => '0');
     countV_dly7 <= (others => '0');
     countV_dly6 <= (others => '0');
     countV_dly5 <= (others => '0');
     countV_dly4 <= (others => '0');
     countV_dly3 <= (others => '0');
     countV_dly2 <= (others => '0');
     countV_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
       if(count512_clr = '1') then
         countV_dly12 <= countV_dly11; -- for writing Relu-out
         countV_dly11 <= countV_dly10; -- for writing Relu-out
         countV_dly10 <= countV_dly9; -- for writing Relu-out
         countV_dly9 <= countV_dly8; -- for writing Relu-out
         countV_dly8 <= countV_dly7; -- for writing Relu-out
         countV_dly7 <= countV_dly6; -- for writing Relu-out
         countV_dly6 <= countV_dly5; -- for writing Relu-out
         countV_dly5 <= countV_dly4; -- for writing Relu-out
         countV_dly4 <= countV_dly3; -- for writing Relu-out
         countV_dly3 <= countV_dly2; -- for writing Relu-out
         countV_dly2 <= countV_dly1; -- for writing Relu-out
         countV_dly1 <= countV; -- for writing Relu-out
       end if;
   end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then --or SoftReset_N = '0' ) then
     count600 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
       if(count600_clr = '1' or SsdStart = '1' ) then
         count600 <= (others => '0');
       elsif(count512_clr = '1') then
         count600 <= count600 + 1;
       end if;
   end if;
end process;

--count300_clr <= '1' when count600 = 299 and count512 = 511 and count9 = 8 else '0'; -- 46.080us x 4 x 75 = 13.824ms, 4x75=300, 300-1=299
count600_clr <= '1' when count600 = 599 and count512 = 511 and count9 = 8 else '0'; -- 46.080us x 16 x 37.5 = 27.648ms, 16x37.5=600, 600-1=599

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then -- or SoftReset_N = '0' ) then
     count512 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     if(count512_clr = '1' or SsdStart = '1' ) then
       count512 <= (others => '0');
     elsif(count9_clr = '1') then
       count512 <= count512 + 1;
     end if;
  end if;
end process;

count512_clr <= '1' when count512 = 511 and count9 = 8 else '0';

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then -- or SoftReset_N = '0' ) then
     count9 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     if(count9_clr = '1' or  SsdStart = '1' ) then 
       count9 <= (others => '0');
     else
       count9 <= count9 + 1;
     end if;
  end if;
end process;

  count9_clr <= '1' when count9 = 8 else '0';

  count3 <= "00000000" when count9 = 0 or count9 = 3 or  count9 = 6 else 
		"00000001" when count9 = 1 or count9 = 4 or  count9 = 7 else 
		"00000010";

  rdbufsel <= WR_N_X - 3 when count9 = 0 or count9 = 1 or count9 = 2 else
		WR_N_X - 2 when count9 = 3 or count9 = 4 or count9 = 5 else
		WR_N_X - 1;

  RD_BUF_SEL <= rdbufsel_dly2;

-------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    DISPJUDGE_latch <= '0';
  elsif( CLK'event and CLK = '1' ) then
    if(VgaHsync_edge = '1') then
      DISPJUDGE_latch <= DISPJUDGE;
    end if;
  end if;
end process;

  SsdStart <= '1' when VgaHsync_edge = '1' and DISPJUDGE_latch = '0' else '0';

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     rdbufsel_dly2 <= (others => '0');
     rdbufsel_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     rdbufsel_dly2 <= rdbufsel_dly1;
     rdbufsel_dly1 <= rdbufsel;
   end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     count9_dly2 <= (others => '0');
     count9_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     count9_dly2 <= count9_dly1;
     count9_dly1 <= count9;
   end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     count512_dly2 <= (others => '0');
     count512_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     count512_dly2 <= count512_dly1;
     count512_dly1 <= count512;
   end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     SsdLineCount <= (others => '0');
     SsdLineCount_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     SsdLineCount <= SsdLineCount_dly1; -- delay 20ns to make it the same timing with ram_x1_rd_sel 2022/04/17
     SsdLineCount_dly1 <= SsdLineCount_tmp;
   end if;
end process;

  --count19 <= '0' & count600(9 downto 2); -- for lay9; 
  count19 <= count600(10 downto 4); -- for lay11; 

  SsdLineCount_tmp <= countV;
  SsdLineCount2 <= countV_dly4; -- for lay9, needs more dly
  --SsdLineCount2 <= countV_dly8; -- for lay8, needs more dly
  --SsdLineCount2 <= countV_dly12; -- for lay6, needs more dly for pooling
  SsdPixCount <= "00" & count19; 
  SsdPixCount2 <= "00" & count19_64_dly12; -- for lay6 from (9 downto 1); -- for layer2 this would be better (not tricky)
  CO9 <= count9;
  CO512 <= count512;
  CO512_dly2 <= count512_dly2;

process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     count19_64_dly12 <= (others => '0');
     count19_64_dly11 <= (others => '0');
     count19_64_dly10 <= (others => '0');
     count19_64_dly9 <= (others => '0');
     count19_64_dly8 <= (others => '0');
     count19_64_dly7 <= (others => '0');
     count19_64_dly6 <= (others => '0');
     count19_64_dly5 <= (others => '0');
     count19_64_dly4 <= (others => '0');
     count19_64_dly3 <= (others => '0');
     count19_64_dly2 <= (others => '0');
     count19_64_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
      if(count512_clr = '1') then
         count19_64_dly12 <= count19_64_dly11;
         count19_64_dly11 <= count19_64_dly10;
         count19_64_dly10 <= count19_64_dly9;
         count19_64_dly9 <= count19_64_dly8;
         count19_64_dly8 <= count19_64_dly7;
         count19_64_dly7 <= count19_64_dly6;
         count19_64_dly6 <= count19_64_dly5;
         count19_64_dly5 <= count19_64_dly4;
         count19_64_dly4 <= count19_64_dly3;
         count19_64_dly3 <= count19_64_dly2;
         count19_64_dly2 <= count19_64_dly1;
         count19_64_dly1 <= count19;
      end if;
   end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     count4wraddr <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     if(VgaHsync_edge = '1') then 
       count4wraddr <= (others => '0');
     elsif(WRN2DL_sig = '0' and count4wraddr /= 607) then -- 1023) then
       count4wraddr <= count4wraddr + 1;
     end if;
  end if;
end process;

-------------------------------------------------------------- 
  RD_BUF_ADDR <= LB_ADDR;

-------------------------------------------------------------- 
process( RST_N, CLK )
begin
  if( RST_N = '0') then
      PIXRD_dly2 <= '0';
      PIXRD_dly1 <= '0';
  elsif( CLK'event and CLK = '1' ) then
    PIXRD_dly2 <= PIXRD_dly1;
    PIXRD_dly1 <= PIXRD;
  end if;
end process;

  PIXRD_edge <= '1' when PIXRD_dly1 = '0' and PIXRD_dly2 = '1' else '0';
  
-------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0') then
      PIXWR_dly2 <= '0';
      PIXWR_dly1 <= '0';
  elsif( CLK'event and CLK = '1' ) then
    PIXWR_dly2 <= PIXWR_dly1;
    PIXWR_dly1 <= PIXWR;
  end if;
end process;

  PIXWR_edge <= '1' when PIXWR_dly1 = '0' and PIXWR_dly2 = '1' else '0';

-------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    WaitLine <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if(PIXWR_edge = '1' or PIXRD_edge = '1') then
      WaitLine <= (others => '0');
    elsif(CamHsync_edge = '1') then
      if(WaitLine /= 3) then -- 2012/11/15
        WaitLine <= WaitLine + 1;
      end if;
    end if;
  end if;
end process;

  LBRDY_sig <= '1' when not(WaitLine = 1 or WaitLine = 2) and (ViewMode = "0101" or ViewMode = "0110") else '0'; -- 2012/11/15
  LBRDY <= LBRDY_sig;
  
-------------------------------------------------------------- 
process( RST_N, PIXSTB, PIXRD_edge, PIXWR_edge )  
begin 
   if( RST_N = '0' or PIXRD_edge = '1'  or PIXWR_edge = '1' ) then
     pixstb_count <= (others => '0');
   elsif( PIXSTB'event and PIXSTB = '1' ) then
     pixstb_count <= pixstb_count + 1;
  end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     pixstb_count_plus <= (others => '0');
   elsif( CLK'event and CLK = '1' ) then
     pixstb_count_plus <= pixstb_count + 96;
  end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     ViewMode_dly2 <= (others => '0');
     ViewMode_dly1 <= (others => '0');
   elsif( CLK'event and CLK = '1' ) then
     ViewMode_dly2 <= ViewMode_dly1;
     ViewMode_dly1 <= ViewMode;
   end if;  
end process;
   
-------------------------------------------------------------- 
  ViewMode5_edge <= '1' when ViewMode_dly1 = "0101" and ViewMode_dly2 /= "0101" else '0';
  ViewMode6_edge <= '1' when ViewMode_dly1 = "0110" and ViewMode_dly2 /= "0110" else '0';

END structure;
