-- A module of Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2019/02/13

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity SHIFTMUL_3B2 is
port(
  COEFF : in std_logic_vector(2 downto 0); 
  PIXDATA : in std_logic_vector(4 downto 0);
  MULOUT : Out std_logic_vector(9 downto 0) );  
end SHIFTMUL_3B2;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of SHIFTMUL_3B2 is

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal data_sft0, data_sft1, data_sft2, data_sft3 : std_logic_vector(9 downto 0);
signal data_inv, data_inv_p1, data_mul_pm1 : std_logic_vector(5 downto 0);

begin

---------------------------------------------------------
  data_inv <= '1' & (not PIXDATA(4)) & (not PIXDATA(3)) & (not PIXDATA(2)) & (not PIXDATA(1)) & (not PIXDATA(0)) ;

  data_inv_p1 <= data_inv + 1;

  data_mul_pm1 <= data_inv_p1 when COEFF(2) = '1' else ('0' & PIXDATA);

  data_sft0 <= data_mul_pm1(5) & data_mul_pm1(5) & data_mul_pm1(5) & data_mul_pm1(5) & data_mul_pm1(5)
                & data_mul_pm1(4) & data_mul_pm1(3) & data_mul_pm1(2) 
		& data_mul_pm1(1) & data_mul_pm1(0);

  data_sft1 <= data_mul_pm1(5) & data_mul_pm1(5) & data_mul_pm1(5) & data_mul_pm1(5) 
		& data_mul_pm1(4) & data_mul_pm1(3) & data_mul_pm1(2) 
		& data_mul_pm1(1) & data_mul_pm1(0) & "0";

  data_sft2 <= data_mul_pm1(5) & data_mul_pm1(5) & data_mul_pm1(5) 
		& data_mul_pm1(4) & data_mul_pm1(3) & data_mul_pm1(2) 
		& data_mul_pm1(1) & data_mul_pm1(0) & "00";

  data_sft3 <= data_mul_pm1(5) 
		& data_mul_pm1(4) & data_mul_pm1(3) 
		& data_mul_pm1(2) & data_mul_pm1(1) & data_mul_pm1(0) & "0000";

  MULOUT <= data_sft0 when COEFF = "000" or COEFF = "111" else 
		data_sft1 when COEFF = "001" or COEFF = "110" else 
		data_sft2 when COEFF = "010" or COEFF = "101" else data_sft3;

end RTL;

