-- Designed by Toshio Iwata at DIGITALFILTER.COM 2022/01/11

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;

entity SRAM_32x2560 is
   Port (
          WRCLK : In std_logic;
          RDCLK : In std_logic;
          WR_N : In std_logic;
          WRADDR : In std_logic_vector(4 downto 0);
          RDADDR : In std_logic_vector(4 downto 0);
          WRDATA : In std_logic_vector(2559 downto 0);
          RDDATA : Out std_logic_vector(2559 downto 0) );
end SRAM_32x2560;

architecture RTL of SRAM_32x2560 is

  subtype RAMWORD is std_logic_vector(2559 downto 0);
  type RAMARRAY is array (0 to 2**5 - 1) of RAMWORD;
  signal RAMDATA : RAMARRAY;
  signal wraddr_int : integer range 0 to 2**5 - 1;
  signal rdaddr_int : integer range 0 to 2**5 - 1;
  signal wraddr_sig : unsigned(4 downto 0);
  signal rdaddr_sig : unsigned(4 downto 0);

begin 

-------------------------------------------------------------------
gen_addr_sig : process( WRADDR, RDADDR )
begin 
  for i in 4 downto 0 loop
    wraddr_sig(i) <= WRADDR(i);
    rdaddr_sig(i) <= RDADDR(i);
  end loop;
end process gen_addr_sig;
 
--------------------------------------------------------------
  wraddr_int <= conv_integer(wraddr_sig);
  rdaddr_int <= conv_integer(rdaddr_sig);

--------------------------------------------------------------
wr_data : process(WRCLK)
begin 
  if(WRCLK'event and WRCLK = '1') then
    if(WR_N = '0') then
      RAMDATA(wraddr_int) <= WRDATA;
    end if;
  end if;
end process wr_data;

--------------------------------------------------------------
rd_data : process(RDCLK) 
begin 
  if(RDCLK'event and RDCLK = '1') then
    if(rdaddr_int = 0 or rdaddr_int = 20) then -- for horizontal padding
      RDDATA <= (others => '0');
    else
      RDDATA <= RAMDATA(rdaddr_int);
    end if;
  end if;
end process rd_data;

end; 
