-- VGA Logic Analyzer & Pattern Generator Top Module
-- Designed by T.Iwata at DIGITALFILTER.COM 2012/11/09

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity VideoProcCore is
	PORT	(
	CLK	: IN std_logic;
	CLK100M	: IN std_logic;
	RST_N	: IN std_logic;
	DISPJUDGE	: IN std_logic;
	mdet_enb	: IN std_logic;
  ViewMode : in std_logic_vector(3 downto 0);
	CamHsync	: IN std_logic;
	CamVsync	: IN std_logic;
	PCLK	: IN std_logic;
  CamData	: in std_logic_vector(7 downto 0);
	MDET_L : out std_logic;
	  PIXSTB : in std_logic;
  PIXRD : in std_logic;
  PIXWR : in std_logic;
  PIX2NIOS : out std_logic_vector(15 downto 0);
  PIX2LOGIC : in std_logic_vector(15 downto 0);
  LBRDY : out std_logic;
  CurrentFrame : out std_logic_vector(2 downto 0);
  NeededFrame : in std_logic_vector(2 downto 0);
	SDRAM_DQENB_N : out std_logic;
  SDRAM_DQMH	: out std_logic;
  SDRAM_DQML	: out std_logic;
  SDRAM_CS_N	: out std_logic;
  SDRAM_WE_N	: out std_logic;
  SDRAM_RAS_N	: out std_logic;
  SDRAM_CAS_N	: out std_logic;
  SDRAM_CKE	: out std_logic;
  SDRAM_DQI : in std_logic_vector(15 downto 0);
  SDRAM_DQO : out std_logic_vector(15 downto 0);
  SDRAM_A : out std_logic_vector(11 downto 0);
  SDRAM_BA : out std_logic_vector(1 downto 0);
  SDRAM_CLK : out std_logic;
  ReluWrN : out std_logic;
  ReluWrData : out std_logic_vector(2559 downto 0);
  ReluWrAddr : out std_logic_vector(4 downto 0);
  ReluWrSel : out std_logic_vector(1 downto 0);
  LayNextWrN : in std_logic;
  LayNextWrData : in std_logic_vector(2559 downto 0);
  LayNextWrAddr : in std_logic_vector(4 downto 0);
  LayNextWrSel : in std_logic_vector(1 downto 0);
	VgaVsync : OUT std_logic;
	VgaHsync : OUT std_logic;
   VgaDataR	: OUT std_logic_vector(7 downto 0);
   VgaDataG	: OUT std_logic_vector(7 downto 0);
   VgaDataB	: OUT std_logic_vector(7 downto 0) );
end VideoProcCore ;

architecture structure of VideoProcCore is

--component CAM_CTRL
--	PORT	(
--		CLK	: IN std_logic;
--		RST_N	: IN std_logic;
--		PCLK	: IN std_logic;
--		CamHsync	: IN std_logic;
--		CamVsync	: IN std_logic;
--		CamData	: IN std_logic_vector(7 downto 0);
--		LB_WR_ADDR	: out std_logic_vector(9 downto 0);
--		LB_WR_DATA	: out std_logic_vector(15 downto 0);
--		LB_WR_N	: out std_logic;
--		CamLineCount28 : out std_logic_vector(4 downto 0);
--		CamHsync_EDGE	: out std_logic;
--		CamVsync_EDGE	: out std_logic;
--		CamPixEnb : out std_logic;
--		CamPixCount : out std_logic_vector(10 downto 0); 
--		CamLineCount : out std_logic_vector(8 downto 0)); 
--end component;
--
--component VGA_CTRL
--	PORT	(
--		CLK	: IN std_logic;
--		RST_N	: IN std_logic;
----		SoftReset_N	: IN std_logic;
----		BigLineN : out std_logic_vector(8 downto 0);
----		BigPixN : out std_logic_vector(9 downto 0);
----		DISPJUDGE	: out std_logic;
--		CamHsync_EDGE	: IN std_logic;
--		CamVsync_EDGE	: IN std_logic;
----		RD_ADDR_LOAD : out std_logic;
----		RD_ADDR_ENB : out std_logic;
----		WR_ADDR_CLR : out std_logic;
--		VgaPixCount : out std_logic_vector(9 downto 0);
--		VgaLineCount	: out std_logic_vector(8 downto 0);
--		VgaPixGate : out std_logic;
--		VgaVisible : OUT std_logic;
--		VgaVsync : OUT std_logic;
--		VgaVsyncDisp : OUT std_logic;
--		VgaHsync : OUT std_logic;
--		VgaHsyncDisp : OUT std_logic;
--		VgaVsync_edge : out std_logic;
--		VgaHsync_edge : out std_logic );
--end component;
--
--component LineIn_CTRL
--	Port (
--		PCLK	: IN std_logic;
--		CLK	: IN std_logic;
--		RST_N	: IN std_logic;
--		LB_WR_ADDR	: IN std_logic_vector(9 downto 0);
--		LB_WR_DATA	: IN std_logic_vector(15 downto 0);
--		LB_WR_N	: IN std_logic;
--		CamLineCount : in std_logic_vector(8 downto 0); 
--		VgaLineCount : in std_logic_vector(8 downto 0);
--		VgaPixCount : in std_logic_vector(9 downto 0);
--		VgaHsync_edge : in std_logic;
--		buf_RGB : out std_logic_vector(15 downto 0)
--);
--end component;

component LineOut_CTRL
	Port (
		CLK	: IN std_logic;
		CLK100M	: IN std_logic;
		RST_N	: IN std_logic;
--		SoftReset_N	: IN std_logic;
		ViewMode : in std_logic_vector(3 downto 0);
--		BigLineN : out std_logic_vector(8 downto 0);
--		RD_ADDR_LOAD : in std_logic;
--		RD_ADDR_ENB : in std_logic;
--		WR_ADDR_CLR : in std_logic;
  PIXSTB : in std_logic;
  PIXWR : in std_logic;
  PIXRD : in std_logic;
  RD_BUF_SEL : out std_logic_vector(1 downto 0);
  WR_BUF_SEL : out std_logic_vector(1 downto 0);
  WR_BUF_SEL2 : out std_logic_vector(1 downto 0);
  WR_BUF_N : out std_logic;
  RD_BUF_ADDR : out std_logic_vector(7 downto 0);
  SsdLineCount : out std_logic_vector(8 downto 0);
  SsdLineCount2 : out std_logic_vector(8 downto 0);
  SsdPixCount : out std_logic_vector(8 downto 0);
  SsdPixCount2 : out std_logic_vector(8 downto 0);
  OutChSel : out std_logic_vector(3 downto 0);
  CO9 : out std_logic_vector(3 downto 0);
  CO512 : out std_logic_vector(8 downto 0);
  CO512_dly2 : out std_logic_vector(8 downto 0);
  PIX2LOGIC : in std_logic_vector(15 downto 0);
LBRDY : out std_logic;
CamHsync_edge : in std_logic;
VgaHsync_edge : in std_logic;
DISPJUDGE : in std_logic;
		latch0	: IN std_logic_vector(15 downto 0);
		latch1	: IN std_logic_vector(15 downto 0);
		latch2	: IN std_logic_vector(15 downto 0);
		latch3	: IN std_logic_vector(15 downto 0);
		CamLineCount : in std_logic_vector(8 downto 0);
		VgaLineCount	: in std_logic_vector(8 downto 0);
		VgaPixCount : IN std_logic_vector(9 downto 0);
		winp0: in std_logic;
		winp1: in std_logic;
		winp2: in std_logic;
		winp3: in std_logic );
end component;

       
component ssd_hw
	Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  WR_BUF_SEL : in std_logic_vector(1 downto 0);
  WR_BUF_SEL2 : in std_logic_vector(1 downto 0);
  WR_BUF_N : in std_logic;
  RD_BUF_SEL : in std_logic_vector(1 downto 0);
  RD_BUF_ADDR : in std_logic_vector(7 downto 0);
  SsdLineCount : in std_logic_vector(8 downto 0);
  SsdLineCount2 : in std_logic_vector(8 downto 0);
  SsdPixCount : in std_logic_vector(8 downto 0);
  SsdPixCount2 : in std_logic_vector(8 downto 0);
  OutChSel : in std_logic_vector(3 downto 0);
  CO9 : in std_logic_vector(3 downto 0);
  CO512 : in std_logic_vector(8 downto 0);
  CO512_dly2 : in std_logic_vector(8 downto 0);
--		BigLineN : in std_logic_vector(8 downto 0);
		DISPJUDGE : in std_logic;
  ReluWrN : out std_logic;
  ReluWrData : out std_logic_vector(2559 downto 0);
  ReluWrAddr : out std_logic_vector(4 downto 0);
  ReluWrSel : out std_logic_vector(1 downto 0);
  LayNextWrN : in std_logic;
  LayNextWrData : in std_logic_vector(2559 downto 0);
  LayNextWrAddr : in std_logic_vector(4 downto 0);
  LayNextWrSel : in std_logic_vector(1 downto 0);
		ReluSum : out std_logic_vector(11 downto 0) );
end component;

signal VgaHsync_sig, VgaHsyncDisp : std_logic;
signal 	LB_WR_ADDR	: std_logic_vector(9 downto 0);
signal 	LB_WR_DATA	: std_logic_vector(15 downto 0);
signal 	LB_WR_N	: std_logic;
signal CamHsync_EDGE, CamVsync_EDGE : std_logic;
signal VgaVisible : std_logic;
 signal pclk_sig : std_logic;
 signal buf_RGB : std_logic_vector(15 downto 0);
signal VgaHsync_edge : std_logic;
signal VgaVsync_edge : std_logic;
signal CamLineCount : std_logic_vector(8 downto 0);
signal latch0, latch1, latch2, latch3 : std_logic_vector(15 downto 0);
signal winp0,winp1,winp2,winp3 : std_logic;
signal  vga_RGB_latch0 : std_logic_vector(15 downto 0);
signal  vga_RGB_latch1 : std_logic_vector(15 downto 0);
signal  vga_RGB_latch2 : std_logic_vector(15 downto 0);
signal  vga_RGB_latch3 : std_logic_vector(15 downto 0);
signal  counter0 : std_logic_vector(23 downto 0);
signal  counter0_clr, counter0_dec : std_logic;
signal VgaPixCount : std_logic_vector(9 downto 0);
signal VgaLineCount	: std_logic_vector(8 downto 0);
 signal buf_r, buf_g, buf_b : std_logic_vector(7 downto 0);
signal LBRDY_sig : std_logic;
signal VgaPixGate : std_logic;
signal SsdLineCount : std_logic_vector(8 downto 0);
signal SsdPixCount : std_logic_vector(8 downto 0);
signal SsdLineCount2 : std_logic_vector(8 downto 0);
signal SsdPixCount2 : std_logic_vector(8 downto 0);
--signal BigPixN : std_logic_vector(9 downto 0);
signal CAM_RD_ADDR : std_logic_vector(15 downto 0);
signal CAM_RD_DATA : std_logic_vector(7 downto 0);
signal CamPixCount : std_logic_vector(10 downto 0); 
signal CAM_CS_N, CamPixEnb : std_logic;
signal CamLineCount28 : std_logic_vector(4 downto 0);
signal RD_ADDR_LOAD, RD_ADDR_ENB, WR_ADDR_CLR : std_logic;
signal CO300 : std_logic_vector(8 downto 0);
signal CO9 : std_logic_vector(3 downto 0);
signal CO512, CO512_dly2 : std_logic_vector(8 downto 0);
signal ReluSum : std_logic_vector(11 downto 0);
signal RD_BUF_SEL, WR_BUF_SEL, WR_BUF_SEL2 : std_logic_vector(1 downto 0);
signal RD_BUF_ADDR : std_logic_vector(7 downto 0);
signal WR_BUF_N : std_logic;
signal OutChSel : std_logic_vector(3 downto 0);

begin

  VgaHsync <= VgaHsyncDisp;

--  VgaDataR <= "00000000" when VgaVisible = '0' else 
--		ReluSum(11 downto 8) & "0000" when DISPJUDGE = '1' else 
--		vga_r;
--
--  VgaDataG <= "00000000" when VgaVisible = '0' else 
--		ReluSum(7 downto 4) & "0000" when DISPJUDGE = '1' else 
--		vga_g;
--
--  VgaDataB <= "00000000" when VgaVisible = '0' else 
--		ReluSum(3 downto 0) & "0000" when DISPJUDGE = '1' else 
--		vga_b;

  pclk_sig <= PCLK;
  
---------------------------------------------------------
--CAM_CTRL_1 : CAM_CTRL port map (
--		CLK	=> CLK,
--		RST_N	=> RST_N,
--		PCLK	=> PCLK_sig,
--		CamHsync	=> CamHsync,
--		CamVsync	=> CamVsync,
--		CamData	=> CAM_RD_DATA,
--		LB_WR_ADDR	=> LB_WR_ADDR,
--		LB_WR_DATA	=> LB_WR_DATA,
--		LB_WR_N	=> LB_WR_N,
--		CamLineCount28 => CamLineCount28,
--		CamHsync_EDGE => CamHsync_EDGE,
--		CamVsync_EDGE => CamVsync_EDGE,
--		CamPixEnb => CamPixEnb, 
--		CamPixCount => CamPixCount, 
--		CamLineCount => CamLineCount ); 
--
--VGA_CTRL_1 : VGA_CTRL port map (
--		CLK	=> CLK,
--		RST_N	=> RST_N,
----		SoftReset_N	=> SoftReset_N,
----		BigLineN => BigLineN,
----		BigPixN	=> BigPixN,
----		DISPJUDGE	=> DISPJUDGE,
--		CamHsync_EDGE	=> CamHsync_EDGE, 
--		CamVsync_EDGE	=> CamVsync_EDGE, 
----		RD_ADDR_LOAD => RD_ADDR_LOAD,
----		RD_ADDR_ENB => RD_ADDR_ENB,
----		WR_ADDR_CLR => WR_ADDR_CLR,
--		VgaLineCount	=> VgaLineCount, 
--		VgaPixCount	=> VgaPixCount, 
--		VgaPixGate => VgaPixGate, 
--		VgaVisible => VgaVisible,
--		VgaVsync => open,
--		VgaVsyncDisp => VgaVsync,
--		VgaHsync => VgaHsync_sig,
--		VgaHsyncDisp => VgaHsyncDisp,
--		VgaVsync_edge => VgaVsync_edge,
--		VgaHsync_edge => VgaHsync_edge );
--
--LineIn_CTRL_1 : LineIn_CTRL port map (
--   PCLK		=> PCLK,
--   CLK		=> CLK,
--		RST_N	=> RST_N,
--		LB_WR_ADDR	=> LB_WR_ADDR,
--		LB_WR_DATA	=> LB_WR_DATA,
--		LB_WR_N	=> LB_WR_N,
--		CamLineCount => CamLineCount,
--		VgaLineCount => VgaLineCount,
--		VgaPixCount	=> VgaPixCount, 
--		VgaHsync_edge => VgaHsync_edge,
--		buf_RGB => buf_RGB );

LineOut_CTRL_1 : LineOut_CTRL port map (
   CLK		=> CLK,
   CLK100M		=> CLK100M,
		RST_N	=> RST_N,
--		SoftReset_N	=> SoftReset_N,
		ViewMode => ViewMode,
--		BigLineN => BigLineN ,
--		RD_ADDR_LOAD => RD_ADDR_LOAD,
--		RD_ADDR_ENB => RD_ADDR_ENB,
--		WR_ADDR_CLR => WR_ADDR_CLR,
  PIXSTB => PIXSTB,
  PIXWR => PIXWR,
  PIXRD => PIXRD,
		RD_BUF_SEL => RD_BUF_SEL,
		WR_BUF_SEL => WR_BUF_SEL,
		WR_BUF_SEL2 => WR_BUF_SEL2,
		WR_BUF_N => WR_BUF_N,
		RD_BUF_ADDR => RD_BUF_ADDR,
  SsdLineCount => SsdLineCount, -- 2022/01/02
  SsdLineCount2 => SsdLineCount2, -- 2022/01/02
  SsdPixCount => SsdPixCount, -- 2022/01/02
  SsdPixCount2 => SsdPixCount2, -- 2022/01/02
  OutChSel => OutChSel, -- for layer3
  CO9 => CO9, --2022/01/02
  CO512 => CO512, --2022/01/02
  CO512_dly2 => CO512_dly2, --2022/01/02
  PIX2LOGIC => PIX2LOGIC,
LBRDY => LBRDY_sig,
		CamHsync_edge => CamHsync_edge,
		VgaHsync_edge => VgaHsync_edge,
		DISPJUDGE => DISPJUDGE,
		latch0	=> latch0,
		latch1	=> latch1,
		latch2	=> latch2,
		latch3	=> latch3,
		CamLineCount => CamLineCount,
		VgaLineCount	=> VgaLineCount, 
		VgaPixCount	=> VgaPixCount,
		winp0	=> winp0,
		winp1	=> winp1,
		winp2	=> winp2,
		winp3	=> winp3 );

ssd_hw_1 : ssd_hw port map (
		CLK		=> CLK100M,
		RST_N	=> RST_N,
		RD_BUF_SEL => RD_BUF_SEL,
		WR_BUF_SEL => WR_BUF_SEL,
		WR_BUF_SEL2 => WR_BUF_SEL2,
		WR_BUF_N => WR_BUF_N,
		RD_BUF_ADDR => RD_BUF_ADDR,
  SsdLineCount => SsdLineCount, --2022/01/02
  SsdLineCount2 => SsdLineCount2, --2022/01/02
  SsdPixCount => SsdPixCount, -- 2022/01/02
  SsdPixCount2 => SsdPixCount2, -- 2022/01/02
  OutChSel => OutChSel, -- for layer3
  CO9 => CO9, --2022/01/02
  CO512 => CO512, --2022/01/02
  CO512_dly2 => CO512_dly2, --2022/01/02
--		BigLineN => BigLineN,
		DISPJUDGE => DISPJUDGE,
  ReluWrN => ReluWrN, 
  ReluWrData => ReluWrData, 
  ReluWrAddr => ReluWrAddr, 
  ReluWrSel => ReluWrSel, 
  LayNextWrN => LayNextWrN, 
  LayNextWrData => LayNextWrData, 
  LayNextWrAddr => LayNextWrAddr, 
  LayNextWrSel => LayNextWrSel, 
		ReluSum => ReluSum );

end structure ;
