-- The layer11 of Convolutional Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2022/01/02

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity conv_layer11 is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  OutChSel : in std_logic_vector(3 downto 0);
  CO300 : in std_logic_vector(8 downto 0);
  CO9 : in std_logic_vector(3 downto 0);
  CO512 : in std_logic_vector(8 downto 0);
  CO512_dly2 : in std_logic_vector(8 downto 0);
  X_RD_DATA : in std_logic_vector(2559 downto 0); 
  ReluAll : out std_logic_vector(2559 downto 0); 
  ReluAllWrN : out std_logic; 
  ReluSum : out std_logic_vector(11 downto 0) ); 
end conv_layer11;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of conv_layer11 is

component dotpro_w11
Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  PixRomData : in std_logic_vector(4 downto 0); 
  MatRomData : in std_logic_vector(2 downto 0);
  CO9is1 : in std_logic;
  CO512is0 : in std_logic;
  MATMULOUT : Out std_logic_vector(13 downto 0) );  
end component;

component ROM_W11
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(12 downto 0);
          DATA : Out std_logic_vector(1535 downto 0) );
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal CO9_clr : std_logic;
signal W3BUSY, W3BUSY_dly1, W3BUSY_dly2, W3BUSY_dly3, W3BUSY_dly4 : std_logic;

signal mac_000, mac_001, mac_002, mac_003, mac_004, mac_005, mac_006, mac_007, mac_008, mac_009 : std_logic_vector(13 downto 0);
signal mac_010, mac_011, mac_012, mac_013, mac_014, mac_015, mac_016, mac_017, mac_018, mac_019 : std_logic_vector(13 downto 0);
signal mac_020, mac_021, mac_022, mac_023, mac_024, mac_025, mac_026, mac_027, mac_028, mac_029 : std_logic_vector(13 downto 0);
signal mac_030, mac_031 : std_logic_vector(13 downto 0);

signal truncate_000, truncate_001, truncate_002, truncate_003, truncate_004, truncate_005, truncate_006, truncate_007, truncate_008, truncate_009 : std_logic_vector(6 downto 0);
signal truncate_010, truncate_011, truncate_012, truncate_013, truncate_014, truncate_015, truncate_016, truncate_017, truncate_018, truncate_019 : std_logic_vector(6 downto 0);
signal truncate_020, truncate_021, truncate_022, truncate_023, truncate_024, truncate_025, truncate_026, truncate_027, truncate_028, truncate_029 : std_logic_vector(6 downto 0);
signal truncate_030, truncate_031 : std_logic_vector(6 downto 0);

signal Relu_000, Relu_001, Relu_002, Relu_003, Relu_004, Relu_005, Relu_006, Relu_007, Relu_008, Relu_009 : std_logic_vector(4 downto 0);
signal Relu_010, Relu_011, Relu_012, Relu_013, Relu_014, Relu_015, Relu_016, Relu_017, Relu_018, Relu_019 : std_logic_vector(4 downto 0);
signal Relu_020, Relu_021, Relu_022, Relu_023, Relu_024, Relu_025, Relu_026, Relu_027, Relu_028, Relu_029 : std_logic_vector(4 downto 0);
signal Relu_030, Relu_031 : std_logic_vector(4 downto 0);

signal PixRomData : std_logic_vector(4 downto 0);
signal W11Addr, W11Addr_prev : std_logic_vector(12 downto 0);
signal W11Addr_L : std_logic_vector(3 downto 0);
signal W11Addr_H : std_logic_vector(12 downto 0);
signal W11Sel : std_logic_vector(3 downto 0);
signal OutChSel_dly1, OutChSel_dly2 : std_logic_vector(3 downto 0);
signal reg_000_00, reg_000_01, reg_000_02, reg_000_03: std_logic_vector(4 downto 0);
signal reg_000_04, reg_000_05, reg_000_06, reg_000_07: std_logic_vector(4 downto 0);
signal reg_000_08, reg_000_09, reg_000_10, reg_000_11: std_logic_vector(4 downto 0);
signal reg_000_12, reg_000_13, reg_000_14, reg_000_15: std_logic_vector(4 downto 0);
signal reg_001_00, reg_001_01, reg_001_02, reg_001_03: std_logic_vector(4 downto 0);
signal reg_001_04, reg_001_05, reg_001_06, reg_001_07: std_logic_vector(4 downto 0);
signal reg_001_08, reg_001_09, reg_001_10, reg_001_11: std_logic_vector(4 downto 0);
signal reg_001_12, reg_001_13, reg_001_14, reg_001_15: std_logic_vector(4 downto 0);
signal reg_002_00, reg_002_01, reg_002_02, reg_002_03: std_logic_vector(4 downto 0);
signal reg_002_04, reg_002_05, reg_002_06, reg_002_07: std_logic_vector(4 downto 0);
signal reg_002_08, reg_002_09, reg_002_10, reg_002_11: std_logic_vector(4 downto 0);
signal reg_002_12, reg_002_13, reg_002_14, reg_002_15: std_logic_vector(4 downto 0);
signal reg_003_00, reg_003_01, reg_003_02, reg_003_03: std_logic_vector(4 downto 0);
signal reg_003_04, reg_003_05, reg_003_06, reg_003_07: std_logic_vector(4 downto 0);
signal reg_003_08, reg_003_09, reg_003_10, reg_003_11: std_logic_vector(4 downto 0);
signal reg_003_12, reg_003_13, reg_003_14, reg_003_15: std_logic_vector(4 downto 0);
signal reg_004_00, reg_004_01, reg_004_02, reg_004_03: std_logic_vector(4 downto 0);
signal reg_004_04, reg_004_05, reg_004_06, reg_004_07: std_logic_vector(4 downto 0);
signal reg_004_08, reg_004_09, reg_004_10, reg_004_11: std_logic_vector(4 downto 0);
signal reg_004_12, reg_004_13, reg_004_14, reg_004_15: std_logic_vector(4 downto 0);
signal reg_005_00, reg_005_01, reg_005_02, reg_005_03: std_logic_vector(4 downto 0);
signal reg_005_04, reg_005_05, reg_005_06, reg_005_07: std_logic_vector(4 downto 0);
signal reg_005_08, reg_005_09, reg_005_10, reg_005_11: std_logic_vector(4 downto 0);
signal reg_005_12, reg_005_13, reg_005_14, reg_005_15: std_logic_vector(4 downto 0);
signal reg_006_00, reg_006_01, reg_006_02, reg_006_03: std_logic_vector(4 downto 0);
signal reg_006_04, reg_006_05, reg_006_06, reg_006_07: std_logic_vector(4 downto 0);
signal reg_006_08, reg_006_09, reg_006_10, reg_006_11: std_logic_vector(4 downto 0);
signal reg_006_12, reg_006_13, reg_006_14, reg_006_15: std_logic_vector(4 downto 0);
signal reg_007_00, reg_007_01, reg_007_02, reg_007_03: std_logic_vector(4 downto 0);
signal reg_007_04, reg_007_05, reg_007_06, reg_007_07: std_logic_vector(4 downto 0);
signal reg_007_08, reg_007_09, reg_007_10, reg_007_11: std_logic_vector(4 downto 0);
signal reg_007_12, reg_007_13, reg_007_14, reg_007_15: std_logic_vector(4 downto 0);
signal reg_008_00, reg_008_01, reg_008_02, reg_008_03: std_logic_vector(4 downto 0);
signal reg_008_04, reg_008_05, reg_008_06, reg_008_07: std_logic_vector(4 downto 0);
signal reg_008_08, reg_008_09, reg_008_10, reg_008_11: std_logic_vector(4 downto 0);
signal reg_008_12, reg_008_13, reg_008_14, reg_008_15: std_logic_vector(4 downto 0);
signal reg_009_00, reg_009_01, reg_009_02, reg_009_03: std_logic_vector(4 downto 0);
signal reg_009_04, reg_009_05, reg_009_06, reg_009_07: std_logic_vector(4 downto 0);
signal reg_009_08, reg_009_09, reg_009_10, reg_009_11: std_logic_vector(4 downto 0);
signal reg_009_12, reg_009_13, reg_009_14, reg_009_15: std_logic_vector(4 downto 0);
signal reg_010_00, reg_010_01, reg_010_02, reg_010_03: std_logic_vector(4 downto 0);
signal reg_010_04, reg_010_05, reg_010_06, reg_010_07: std_logic_vector(4 downto 0);
signal reg_010_08, reg_010_09, reg_010_10, reg_010_11: std_logic_vector(4 downto 0);
signal reg_010_12, reg_010_13, reg_010_14, reg_010_15: std_logic_vector(4 downto 0);
signal reg_011_00, reg_011_01, reg_011_02, reg_011_03: std_logic_vector(4 downto 0);
signal reg_011_04, reg_011_05, reg_011_06, reg_011_07: std_logic_vector(4 downto 0);
signal reg_011_08, reg_011_09, reg_011_10, reg_011_11: std_logic_vector(4 downto 0);
signal reg_011_12, reg_011_13, reg_011_14, reg_011_15: std_logic_vector(4 downto 0);
signal reg_012_00, reg_012_01, reg_012_02, reg_012_03: std_logic_vector(4 downto 0);
signal reg_012_04, reg_012_05, reg_012_06, reg_012_07: std_logic_vector(4 downto 0);
signal reg_012_08, reg_012_09, reg_012_10, reg_012_11: std_logic_vector(4 downto 0);
signal reg_012_12, reg_012_13, reg_012_14, reg_012_15: std_logic_vector(4 downto 0);
signal reg_013_00, reg_013_01, reg_013_02, reg_013_03: std_logic_vector(4 downto 0);
signal reg_013_04, reg_013_05, reg_013_06, reg_013_07: std_logic_vector(4 downto 0);
signal reg_013_08, reg_013_09, reg_013_10, reg_013_11: std_logic_vector(4 downto 0);
signal reg_013_12, reg_013_13, reg_013_14, reg_013_15: std_logic_vector(4 downto 0);
signal reg_014_00, reg_014_01, reg_014_02, reg_014_03: std_logic_vector(4 downto 0);
signal reg_014_04, reg_014_05, reg_014_06, reg_014_07: std_logic_vector(4 downto 0);
signal reg_014_08, reg_014_09, reg_014_10, reg_014_11: std_logic_vector(4 downto 0);
signal reg_014_12, reg_014_13, reg_014_14, reg_014_15: std_logic_vector(4 downto 0);
signal reg_015_00, reg_015_01, reg_015_02, reg_015_03: std_logic_vector(4 downto 0);
signal reg_015_04, reg_015_05, reg_015_06, reg_015_07: std_logic_vector(4 downto 0);
signal reg_015_08, reg_015_09, reg_015_10, reg_015_11: std_logic_vector(4 downto 0);
signal reg_015_12, reg_015_13, reg_015_14, reg_015_15: std_logic_vector(4 downto 0);
signal reg_016_00, reg_016_01, reg_016_02, reg_016_03: std_logic_vector(4 downto 0);
signal reg_016_04, reg_016_05, reg_016_06, reg_016_07: std_logic_vector(4 downto 0);
signal reg_016_08, reg_016_09, reg_016_10, reg_016_11: std_logic_vector(4 downto 0);
signal reg_016_12, reg_016_13, reg_016_14, reg_016_15: std_logic_vector(4 downto 0);
signal reg_017_00, reg_017_01, reg_017_02, reg_017_03: std_logic_vector(4 downto 0);
signal reg_017_04, reg_017_05, reg_017_06, reg_017_07: std_logic_vector(4 downto 0);
signal reg_017_08, reg_017_09, reg_017_10, reg_017_11: std_logic_vector(4 downto 0);
signal reg_017_12, reg_017_13, reg_017_14, reg_017_15: std_logic_vector(4 downto 0);
signal reg_018_00, reg_018_01, reg_018_02, reg_018_03: std_logic_vector(4 downto 0);
signal reg_018_04, reg_018_05, reg_018_06, reg_018_07: std_logic_vector(4 downto 0);
signal reg_018_08, reg_018_09, reg_018_10, reg_018_11: std_logic_vector(4 downto 0);
signal reg_018_12, reg_018_13, reg_018_14, reg_018_15: std_logic_vector(4 downto 0);
signal reg_019_00, reg_019_01, reg_019_02, reg_019_03: std_logic_vector(4 downto 0);
signal reg_019_04, reg_019_05, reg_019_06, reg_019_07: std_logic_vector(4 downto 0);
signal reg_019_08, reg_019_09, reg_019_10, reg_019_11: std_logic_vector(4 downto 0);
signal reg_019_12, reg_019_13, reg_019_14, reg_019_15: std_logic_vector(4 downto 0);
signal reg_020_00, reg_020_01, reg_020_02, reg_020_03: std_logic_vector(4 downto 0);
signal reg_020_04, reg_020_05, reg_020_06, reg_020_07: std_logic_vector(4 downto 0);
signal reg_020_08, reg_020_09, reg_020_10, reg_020_11: std_logic_vector(4 downto 0);
signal reg_020_12, reg_020_13, reg_020_14, reg_020_15: std_logic_vector(4 downto 0);
signal reg_021_00, reg_021_01, reg_021_02, reg_021_03: std_logic_vector(4 downto 0);
signal reg_021_04, reg_021_05, reg_021_06, reg_021_07: std_logic_vector(4 downto 0);
signal reg_021_08, reg_021_09, reg_021_10, reg_021_11: std_logic_vector(4 downto 0);
signal reg_021_12, reg_021_13, reg_021_14, reg_021_15: std_logic_vector(4 downto 0);
signal reg_022_00, reg_022_01, reg_022_02, reg_022_03: std_logic_vector(4 downto 0);
signal reg_022_04, reg_022_05, reg_022_06, reg_022_07: std_logic_vector(4 downto 0);
signal reg_022_08, reg_022_09, reg_022_10, reg_022_11: std_logic_vector(4 downto 0);
signal reg_022_12, reg_022_13, reg_022_14, reg_022_15: std_logic_vector(4 downto 0);
signal reg_023_00, reg_023_01, reg_023_02, reg_023_03: std_logic_vector(4 downto 0);
signal reg_023_04, reg_023_05, reg_023_06, reg_023_07: std_logic_vector(4 downto 0);
signal reg_023_08, reg_023_09, reg_023_10, reg_023_11: std_logic_vector(4 downto 0);
signal reg_023_12, reg_023_13, reg_023_14, reg_023_15: std_logic_vector(4 downto 0);
signal reg_024_00, reg_024_01, reg_024_02, reg_024_03: std_logic_vector(4 downto 0);
signal reg_024_04, reg_024_05, reg_024_06, reg_024_07: std_logic_vector(4 downto 0);
signal reg_024_08, reg_024_09, reg_024_10, reg_024_11: std_logic_vector(4 downto 0);
signal reg_024_12, reg_024_13, reg_024_14, reg_024_15: std_logic_vector(4 downto 0);
signal reg_025_00, reg_025_01, reg_025_02, reg_025_03: std_logic_vector(4 downto 0);
signal reg_025_04, reg_025_05, reg_025_06, reg_025_07: std_logic_vector(4 downto 0);
signal reg_025_08, reg_025_09, reg_025_10, reg_025_11: std_logic_vector(4 downto 0);
signal reg_025_12, reg_025_13, reg_025_14, reg_025_15: std_logic_vector(4 downto 0);
signal reg_026_00, reg_026_01, reg_026_02, reg_026_03: std_logic_vector(4 downto 0);
signal reg_026_04, reg_026_05, reg_026_06, reg_026_07: std_logic_vector(4 downto 0);
signal reg_026_08, reg_026_09, reg_026_10, reg_026_11: std_logic_vector(4 downto 0);
signal reg_026_12, reg_026_13, reg_026_14, reg_026_15: std_logic_vector(4 downto 0);
signal reg_027_00, reg_027_01, reg_027_02, reg_027_03: std_logic_vector(4 downto 0);
signal reg_027_04, reg_027_05, reg_027_06, reg_027_07: std_logic_vector(4 downto 0);
signal reg_027_08, reg_027_09, reg_027_10, reg_027_11: std_logic_vector(4 downto 0);
signal reg_027_12, reg_027_13, reg_027_14, reg_027_15: std_logic_vector(4 downto 0);
signal reg_028_00, reg_028_01, reg_028_02, reg_028_03: std_logic_vector(4 downto 0);
signal reg_028_04, reg_028_05, reg_028_06, reg_028_07: std_logic_vector(4 downto 0);
signal reg_028_08, reg_028_09, reg_028_10, reg_028_11: std_logic_vector(4 downto 0);
signal reg_028_12, reg_028_13, reg_028_14, reg_028_15: std_logic_vector(4 downto 0);
signal reg_029_00, reg_029_01, reg_029_02, reg_029_03: std_logic_vector(4 downto 0);
signal reg_029_04, reg_029_05, reg_029_06, reg_029_07: std_logic_vector(4 downto 0);
signal reg_029_08, reg_029_09, reg_029_10, reg_029_11: std_logic_vector(4 downto 0);
signal reg_029_12, reg_029_13, reg_029_14, reg_029_15: std_logic_vector(4 downto 0);
signal reg_030_00, reg_030_01, reg_030_02, reg_030_03: std_logic_vector(4 downto 0);
signal reg_030_04, reg_030_05, reg_030_06, reg_030_07: std_logic_vector(4 downto 0);
signal reg_030_08, reg_030_09, reg_030_10, reg_030_11: std_logic_vector(4 downto 0);
signal reg_030_12, reg_030_13, reg_030_14, reg_030_15: std_logic_vector(4 downto 0);
signal reg_031_00, reg_031_01, reg_031_02, reg_031_03: std_logic_vector(4 downto 0);
signal reg_031_04, reg_031_05, reg_031_06, reg_031_07: std_logic_vector(4 downto 0);
signal reg_031_08, reg_031_09, reg_031_10, reg_031_11: std_logic_vector(4 downto 0);
signal reg_031_12, reg_031_13, reg_031_14, reg_031_15: std_logic_vector(4 downto 0);
signal enb_a, enb_b, enb_c, enb_d : std_logic;
signal enb_e, enb_f, enb_g, enb_h : std_logic;
signal enb_i, enb_j, enb_k, enb_l : std_logic;
signal enb_m, enb_n, enb_o, enb_p : std_logic;
signal CO9is1, CO512is0 : std_logic;
signal W11Data : std_logic_vector(1535 downto 0);
signal coeff_000, coeff_001, coeff_002, coeff_003, coeff_004, coeff_005, coeff_006, coeff_007, coeff_008, coeff_009: std_logic_vector(2 downto 0);
signal coeff_010, coeff_011, coeff_012, coeff_013, coeff_014, coeff_015, coeff_016, coeff_017, coeff_018, coeff_019: std_logic_vector(2 downto 0);
signal coeff_020, coeff_021, coeff_022, coeff_023, coeff_024, coeff_025, coeff_026, coeff_027, coeff_028, coeff_029: std_logic_vector(2 downto 0);
signal coeff_030, coeff_031: std_logic_vector(2 downto 0);

begin

CO9is1 <= '1' when CO9 = 1 else '0'; 
CO512is0 <= '1' when CO512 = 0 else '0';

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		OutChSel_dly2 <= (others => '0');
		OutChSel_dly1 <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		OutChSel_dly2 <= OutChSel_dly1;
		OutChSel_dly1 <= OutChSel;
	end if;  
end process;

W11Sel <= OutChSel_dly2(3 downto 0);

---------------------------------------------------------
  PixRomData <= 
X_RD_DATA( 2559 downto 2555 ) when CO512_dly2 = "000000000" else 
X_RD_DATA( 2554 downto 2550 ) when CO512_dly2 = "000000001" else 
X_RD_DATA( 2549 downto 2545 ) when CO512_dly2 = "000000010" else 
X_RD_DATA( 2544 downto 2540 ) when CO512_dly2 = "000000011" else 
X_RD_DATA( 2539 downto 2535 ) when CO512_dly2 = "000000100" else 
X_RD_DATA( 2534 downto 2530 ) when CO512_dly2 = "000000101" else 
X_RD_DATA( 2529 downto 2525 ) when CO512_dly2 = "000000110" else 
X_RD_DATA( 2524 downto 2520 ) when CO512_dly2 = "000000111" else 
X_RD_DATA( 2519 downto 2515 ) when CO512_dly2 = "000001000" else 
X_RD_DATA( 2514 downto 2510 ) when CO512_dly2 = "000001001" else 
X_RD_DATA( 2509 downto 2505 ) when CO512_dly2 = "000001010" else 
X_RD_DATA( 2504 downto 2500 ) when CO512_dly2 = "000001011" else 
X_RD_DATA( 2499 downto 2495 ) when CO512_dly2 = "000001100" else 
X_RD_DATA( 2494 downto 2490 ) when CO512_dly2 = "000001101" else 
X_RD_DATA( 2489 downto 2485 ) when CO512_dly2 = "000001110" else 
X_RD_DATA( 2484 downto 2480 ) when CO512_dly2 = "000001111" else 
X_RD_DATA( 2479 downto 2475 ) when CO512_dly2 = "000010000" else 
X_RD_DATA( 2474 downto 2470 ) when CO512_dly2 = "000010001" else 
X_RD_DATA( 2469 downto 2465 ) when CO512_dly2 = "000010010" else 
X_RD_DATA( 2464 downto 2460 ) when CO512_dly2 = "000010011" else 
X_RD_DATA( 2459 downto 2455 ) when CO512_dly2 = "000010100" else 
X_RD_DATA( 2454 downto 2450 ) when CO512_dly2 = "000010101" else 
X_RD_DATA( 2449 downto 2445 ) when CO512_dly2 = "000010110" else 
X_RD_DATA( 2444 downto 2440 ) when CO512_dly2 = "000010111" else 
X_RD_DATA( 2439 downto 2435 ) when CO512_dly2 = "000011000" else 
X_RD_DATA( 2434 downto 2430 ) when CO512_dly2 = "000011001" else 
X_RD_DATA( 2429 downto 2425 ) when CO512_dly2 = "000011010" else 
X_RD_DATA( 2424 downto 2420 ) when CO512_dly2 = "000011011" else 
X_RD_DATA( 2419 downto 2415 ) when CO512_dly2 = "000011100" else 
X_RD_DATA( 2414 downto 2410 ) when CO512_dly2 = "000011101" else 
X_RD_DATA( 2409 downto 2405 ) when CO512_dly2 = "000011110" else 
X_RD_DATA( 2404 downto 2400 ) when CO512_dly2 = "000011111" else 
X_RD_DATA( 2399 downto 2395 ) when CO512_dly2 = "000100000" else 
X_RD_DATA( 2394 downto 2390 ) when CO512_dly2 = "000100001" else 
X_RD_DATA( 2389 downto 2385 ) when CO512_dly2 = "000100010" else 
X_RD_DATA( 2384 downto 2380 ) when CO512_dly2 = "000100011" else 
X_RD_DATA( 2379 downto 2375 ) when CO512_dly2 = "000100100" else 
X_RD_DATA( 2374 downto 2370 ) when CO512_dly2 = "000100101" else 
X_RD_DATA( 2369 downto 2365 ) when CO512_dly2 = "000100110" else 
X_RD_DATA( 2364 downto 2360 ) when CO512_dly2 = "000100111" else 
X_RD_DATA( 2359 downto 2355 ) when CO512_dly2 = "000101000" else 
X_RD_DATA( 2354 downto 2350 ) when CO512_dly2 = "000101001" else 
X_RD_DATA( 2349 downto 2345 ) when CO512_dly2 = "000101010" else 
X_RD_DATA( 2344 downto 2340 ) when CO512_dly2 = "000101011" else 
X_RD_DATA( 2339 downto 2335 ) when CO512_dly2 = "000101100" else 
X_RD_DATA( 2334 downto 2330 ) when CO512_dly2 = "000101101" else 
X_RD_DATA( 2329 downto 2325 ) when CO512_dly2 = "000101110" else 
X_RD_DATA( 2324 downto 2320 ) when CO512_dly2 = "000101111" else 
X_RD_DATA( 2319 downto 2315 ) when CO512_dly2 = "000110000" else 
X_RD_DATA( 2314 downto 2310 ) when CO512_dly2 = "000110001" else 
X_RD_DATA( 2309 downto 2305 ) when CO512_dly2 = "000110010" else 
X_RD_DATA( 2304 downto 2300 ) when CO512_dly2 = "000110011" else 
X_RD_DATA( 2299 downto 2295 ) when CO512_dly2 = "000110100" else 
X_RD_DATA( 2294 downto 2290 ) when CO512_dly2 = "000110101" else 
X_RD_DATA( 2289 downto 2285 ) when CO512_dly2 = "000110110" else 
X_RD_DATA( 2284 downto 2280 ) when CO512_dly2 = "000110111" else 
X_RD_DATA( 2279 downto 2275 ) when CO512_dly2 = "000111000" else 
X_RD_DATA( 2274 downto 2270 ) when CO512_dly2 = "000111001" else 
X_RD_DATA( 2269 downto 2265 ) when CO512_dly2 = "000111010" else 
X_RD_DATA( 2264 downto 2260 ) when CO512_dly2 = "000111011" else 
X_RD_DATA( 2259 downto 2255 ) when CO512_dly2 = "000111100" else 
X_RD_DATA( 2254 downto 2250 ) when CO512_dly2 = "000111101" else 
X_RD_DATA( 2249 downto 2245 ) when CO512_dly2 = "000111110" else 
X_RD_DATA( 2244 downto 2240 ) when CO512_dly2 = "000111111" else 
X_RD_DATA( 2239 downto 2235 ) when CO512_dly2 = "001000000" else 
X_RD_DATA( 2234 downto 2230 ) when CO512_dly2 = "001000001" else 
X_RD_DATA( 2229 downto 2225 ) when CO512_dly2 = "001000010" else 
X_RD_DATA( 2224 downto 2220 ) when CO512_dly2 = "001000011" else 
X_RD_DATA( 2219 downto 2215 ) when CO512_dly2 = "001000100" else 
X_RD_DATA( 2214 downto 2210 ) when CO512_dly2 = "001000101" else 
X_RD_DATA( 2209 downto 2205 ) when CO512_dly2 = "001000110" else 
X_RD_DATA( 2204 downto 2200 ) when CO512_dly2 = "001000111" else 
X_RD_DATA( 2199 downto 2195 ) when CO512_dly2 = "001001000" else 
X_RD_DATA( 2194 downto 2190 ) when CO512_dly2 = "001001001" else 
X_RD_DATA( 2189 downto 2185 ) when CO512_dly2 = "001001010" else 
X_RD_DATA( 2184 downto 2180 ) when CO512_dly2 = "001001011" else 
X_RD_DATA( 2179 downto 2175 ) when CO512_dly2 = "001001100" else 
X_RD_DATA( 2174 downto 2170 ) when CO512_dly2 = "001001101" else 
X_RD_DATA( 2169 downto 2165 ) when CO512_dly2 = "001001110" else 
X_RD_DATA( 2164 downto 2160 ) when CO512_dly2 = "001001111" else 
X_RD_DATA( 2159 downto 2155 ) when CO512_dly2 = "001010000" else 
X_RD_DATA( 2154 downto 2150 ) when CO512_dly2 = "001010001" else 
X_RD_DATA( 2149 downto 2145 ) when CO512_dly2 = "001010010" else 
X_RD_DATA( 2144 downto 2140 ) when CO512_dly2 = "001010011" else 
X_RD_DATA( 2139 downto 2135 ) when CO512_dly2 = "001010100" else 
X_RD_DATA( 2134 downto 2130 ) when CO512_dly2 = "001010101" else 
X_RD_DATA( 2129 downto 2125 ) when CO512_dly2 = "001010110" else 
X_RD_DATA( 2124 downto 2120 ) when CO512_dly2 = "001010111" else 
X_RD_DATA( 2119 downto 2115 ) when CO512_dly2 = "001011000" else 
X_RD_DATA( 2114 downto 2110 ) when CO512_dly2 = "001011001" else 
X_RD_DATA( 2109 downto 2105 ) when CO512_dly2 = "001011010" else 
X_RD_DATA( 2104 downto 2100 ) when CO512_dly2 = "001011011" else 
X_RD_DATA( 2099 downto 2095 ) when CO512_dly2 = "001011100" else 
X_RD_DATA( 2094 downto 2090 ) when CO512_dly2 = "001011101" else 
X_RD_DATA( 2089 downto 2085 ) when CO512_dly2 = "001011110" else 
X_RD_DATA( 2084 downto 2080 ) when CO512_dly2 = "001011111" else 
X_RD_DATA( 2079 downto 2075 ) when CO512_dly2 = "001100000" else 
X_RD_DATA( 2074 downto 2070 ) when CO512_dly2 = "001100001" else 
X_RD_DATA( 2069 downto 2065 ) when CO512_dly2 = "001100010" else 
X_RD_DATA( 2064 downto 2060 ) when CO512_dly2 = "001100011" else 
X_RD_DATA( 2059 downto 2055 ) when CO512_dly2 = "001100100" else 
X_RD_DATA( 2054 downto 2050 ) when CO512_dly2 = "001100101" else 
X_RD_DATA( 2049 downto 2045 ) when CO512_dly2 = "001100110" else 
X_RD_DATA( 2044 downto 2040 ) when CO512_dly2 = "001100111" else 
X_RD_DATA( 2039 downto 2035 ) when CO512_dly2 = "001101000" else 
X_RD_DATA( 2034 downto 2030 ) when CO512_dly2 = "001101001" else 
X_RD_DATA( 2029 downto 2025 ) when CO512_dly2 = "001101010" else 
X_RD_DATA( 2024 downto 2020 ) when CO512_dly2 = "001101011" else 
X_RD_DATA( 2019 downto 2015 ) when CO512_dly2 = "001101100" else 
X_RD_DATA( 2014 downto 2010 ) when CO512_dly2 = "001101101" else 
X_RD_DATA( 2009 downto 2005 ) when CO512_dly2 = "001101110" else 
X_RD_DATA( 2004 downto 2000 ) when CO512_dly2 = "001101111" else 
X_RD_DATA( 1999 downto 1995 ) when CO512_dly2 = "001110000" else 
X_RD_DATA( 1994 downto 1990 ) when CO512_dly2 = "001110001" else 
X_RD_DATA( 1989 downto 1985 ) when CO512_dly2 = "001110010" else 
X_RD_DATA( 1984 downto 1980 ) when CO512_dly2 = "001110011" else 
X_RD_DATA( 1979 downto 1975 ) when CO512_dly2 = "001110100" else 
X_RD_DATA( 1974 downto 1970 ) when CO512_dly2 = "001110101" else 
X_RD_DATA( 1969 downto 1965 ) when CO512_dly2 = "001110110" else 
X_RD_DATA( 1964 downto 1960 ) when CO512_dly2 = "001110111" else 
X_RD_DATA( 1959 downto 1955 ) when CO512_dly2 = "001111000" else 
X_RD_DATA( 1954 downto 1950 ) when CO512_dly2 = "001111001" else 
X_RD_DATA( 1949 downto 1945 ) when CO512_dly2 = "001111010" else 
X_RD_DATA( 1944 downto 1940 ) when CO512_dly2 = "001111011" else 
X_RD_DATA( 1939 downto 1935 ) when CO512_dly2 = "001111100" else 
X_RD_DATA( 1934 downto 1930 ) when CO512_dly2 = "001111101" else 
X_RD_DATA( 1929 downto 1925 ) when CO512_dly2 = "001111110" else 
X_RD_DATA( 1924 downto 1920 ) when CO512_dly2 = "001111111" else 
X_RD_DATA( 1919 downto 1915 ) when CO512_dly2 = "010000000" else 
X_RD_DATA( 1914 downto 1910 ) when CO512_dly2 = "010000001" else 
X_RD_DATA( 1909 downto 1905 ) when CO512_dly2 = "010000010" else 
X_RD_DATA( 1904 downto 1900 ) when CO512_dly2 = "010000011" else 
X_RD_DATA( 1899 downto 1895 ) when CO512_dly2 = "010000100" else 
X_RD_DATA( 1894 downto 1890 ) when CO512_dly2 = "010000101" else 
X_RD_DATA( 1889 downto 1885 ) when CO512_dly2 = "010000110" else 
X_RD_DATA( 1884 downto 1880 ) when CO512_dly2 = "010000111" else 
X_RD_DATA( 1879 downto 1875 ) when CO512_dly2 = "010001000" else 
X_RD_DATA( 1874 downto 1870 ) when CO512_dly2 = "010001001" else 
X_RD_DATA( 1869 downto 1865 ) when CO512_dly2 = "010001010" else 
X_RD_DATA( 1864 downto 1860 ) when CO512_dly2 = "010001011" else 
X_RD_DATA( 1859 downto 1855 ) when CO512_dly2 = "010001100" else 
X_RD_DATA( 1854 downto 1850 ) when CO512_dly2 = "010001101" else 
X_RD_DATA( 1849 downto 1845 ) when CO512_dly2 = "010001110" else 
X_RD_DATA( 1844 downto 1840 ) when CO512_dly2 = "010001111" else 
X_RD_DATA( 1839 downto 1835 ) when CO512_dly2 = "010010000" else 
X_RD_DATA( 1834 downto 1830 ) when CO512_dly2 = "010010001" else 
X_RD_DATA( 1829 downto 1825 ) when CO512_dly2 = "010010010" else 
X_RD_DATA( 1824 downto 1820 ) when CO512_dly2 = "010010011" else 
X_RD_DATA( 1819 downto 1815 ) when CO512_dly2 = "010010100" else 
X_RD_DATA( 1814 downto 1810 ) when CO512_dly2 = "010010101" else 
X_RD_DATA( 1809 downto 1805 ) when CO512_dly2 = "010010110" else 
X_RD_DATA( 1804 downto 1800 ) when CO512_dly2 = "010010111" else 
X_RD_DATA( 1799 downto 1795 ) when CO512_dly2 = "010011000" else 
X_RD_DATA( 1794 downto 1790 ) when CO512_dly2 = "010011001" else 
X_RD_DATA( 1789 downto 1785 ) when CO512_dly2 = "010011010" else 
X_RD_DATA( 1784 downto 1780 ) when CO512_dly2 = "010011011" else 
X_RD_DATA( 1779 downto 1775 ) when CO512_dly2 = "010011100" else 
X_RD_DATA( 1774 downto 1770 ) when CO512_dly2 = "010011101" else 
X_RD_DATA( 1769 downto 1765 ) when CO512_dly2 = "010011110" else 
X_RD_DATA( 1764 downto 1760 ) when CO512_dly2 = "010011111" else 
X_RD_DATA( 1759 downto 1755 ) when CO512_dly2 = "010100000" else 
X_RD_DATA( 1754 downto 1750 ) when CO512_dly2 = "010100001" else 
X_RD_DATA( 1749 downto 1745 ) when CO512_dly2 = "010100010" else 
X_RD_DATA( 1744 downto 1740 ) when CO512_dly2 = "010100011" else 
X_RD_DATA( 1739 downto 1735 ) when CO512_dly2 = "010100100" else 
X_RD_DATA( 1734 downto 1730 ) when CO512_dly2 = "010100101" else 
X_RD_DATA( 1729 downto 1725 ) when CO512_dly2 = "010100110" else 
X_RD_DATA( 1724 downto 1720 ) when CO512_dly2 = "010100111" else 
X_RD_DATA( 1719 downto 1715 ) when CO512_dly2 = "010101000" else 
X_RD_DATA( 1714 downto 1710 ) when CO512_dly2 = "010101001" else 
X_RD_DATA( 1709 downto 1705 ) when CO512_dly2 = "010101010" else 
X_RD_DATA( 1704 downto 1700 ) when CO512_dly2 = "010101011" else 
X_RD_DATA( 1699 downto 1695 ) when CO512_dly2 = "010101100" else 
X_RD_DATA( 1694 downto 1690 ) when CO512_dly2 = "010101101" else 
X_RD_DATA( 1689 downto 1685 ) when CO512_dly2 = "010101110" else 
X_RD_DATA( 1684 downto 1680 ) when CO512_dly2 = "010101111" else 
X_RD_DATA( 1679 downto 1675 ) when CO512_dly2 = "010110000" else 
X_RD_DATA( 1674 downto 1670 ) when CO512_dly2 = "010110001" else 
X_RD_DATA( 1669 downto 1665 ) when CO512_dly2 = "010110010" else 
X_RD_DATA( 1664 downto 1660 ) when CO512_dly2 = "010110011" else 
X_RD_DATA( 1659 downto 1655 ) when CO512_dly2 = "010110100" else 
X_RD_DATA( 1654 downto 1650 ) when CO512_dly2 = "010110101" else 
X_RD_DATA( 1649 downto 1645 ) when CO512_dly2 = "010110110" else 
X_RD_DATA( 1644 downto 1640 ) when CO512_dly2 = "010110111" else 
X_RD_DATA( 1639 downto 1635 ) when CO512_dly2 = "010111000" else 
X_RD_DATA( 1634 downto 1630 ) when CO512_dly2 = "010111001" else 
X_RD_DATA( 1629 downto 1625 ) when CO512_dly2 = "010111010" else 
X_RD_DATA( 1624 downto 1620 ) when CO512_dly2 = "010111011" else 
X_RD_DATA( 1619 downto 1615 ) when CO512_dly2 = "010111100" else 
X_RD_DATA( 1614 downto 1610 ) when CO512_dly2 = "010111101" else 
X_RD_DATA( 1609 downto 1605 ) when CO512_dly2 = "010111110" else 
X_RD_DATA( 1604 downto 1600 ) when CO512_dly2 = "010111111" else 
X_RD_DATA( 1599 downto 1595 ) when CO512_dly2 = "011000000" else 
X_RD_DATA( 1594 downto 1590 ) when CO512_dly2 = "011000001" else 
X_RD_DATA( 1589 downto 1585 ) when CO512_dly2 = "011000010" else 
X_RD_DATA( 1584 downto 1580 ) when CO512_dly2 = "011000011" else 
X_RD_DATA( 1579 downto 1575 ) when CO512_dly2 = "011000100" else 
X_RD_DATA( 1574 downto 1570 ) when CO512_dly2 = "011000101" else 
X_RD_DATA( 1569 downto 1565 ) when CO512_dly2 = "011000110" else 
X_RD_DATA( 1564 downto 1560 ) when CO512_dly2 = "011000111" else 
X_RD_DATA( 1559 downto 1555 ) when CO512_dly2 = "011001000" else 
X_RD_DATA( 1554 downto 1550 ) when CO512_dly2 = "011001001" else 
X_RD_DATA( 1549 downto 1545 ) when CO512_dly2 = "011001010" else 
X_RD_DATA( 1544 downto 1540 ) when CO512_dly2 = "011001011" else 
X_RD_DATA( 1539 downto 1535 ) when CO512_dly2 = "011001100" else 
X_RD_DATA( 1534 downto 1530 ) when CO512_dly2 = "011001101" else 
X_RD_DATA( 1529 downto 1525 ) when CO512_dly2 = "011001110" else 
X_RD_DATA( 1524 downto 1520 ) when CO512_dly2 = "011001111" else 
X_RD_DATA( 1519 downto 1515 ) when CO512_dly2 = "011010000" else 
X_RD_DATA( 1514 downto 1510 ) when CO512_dly2 = "011010001" else 
X_RD_DATA( 1509 downto 1505 ) when CO512_dly2 = "011010010" else 
X_RD_DATA( 1504 downto 1500 ) when CO512_dly2 = "011010011" else 
X_RD_DATA( 1499 downto 1495 ) when CO512_dly2 = "011010100" else 
X_RD_DATA( 1494 downto 1490 ) when CO512_dly2 = "011010101" else 
X_RD_DATA( 1489 downto 1485 ) when CO512_dly2 = "011010110" else 
X_RD_DATA( 1484 downto 1480 ) when CO512_dly2 = "011010111" else 
X_RD_DATA( 1479 downto 1475 ) when CO512_dly2 = "011011000" else 
X_RD_DATA( 1474 downto 1470 ) when CO512_dly2 = "011011001" else 
X_RD_DATA( 1469 downto 1465 ) when CO512_dly2 = "011011010" else 
X_RD_DATA( 1464 downto 1460 ) when CO512_dly2 = "011011011" else 
X_RD_DATA( 1459 downto 1455 ) when CO512_dly2 = "011011100" else 
X_RD_DATA( 1454 downto 1450 ) when CO512_dly2 = "011011101" else 
X_RD_DATA( 1449 downto 1445 ) when CO512_dly2 = "011011110" else 
X_RD_DATA( 1444 downto 1440 ) when CO512_dly2 = "011011111" else 
X_RD_DATA( 1439 downto 1435 ) when CO512_dly2 = "011100000" else 
X_RD_DATA( 1434 downto 1430 ) when CO512_dly2 = "011100001" else 
X_RD_DATA( 1429 downto 1425 ) when CO512_dly2 = "011100010" else 
X_RD_DATA( 1424 downto 1420 ) when CO512_dly2 = "011100011" else 
X_RD_DATA( 1419 downto 1415 ) when CO512_dly2 = "011100100" else 
X_RD_DATA( 1414 downto 1410 ) when CO512_dly2 = "011100101" else 
X_RD_DATA( 1409 downto 1405 ) when CO512_dly2 = "011100110" else 
X_RD_DATA( 1404 downto 1400 ) when CO512_dly2 = "011100111" else 
X_RD_DATA( 1399 downto 1395 ) when CO512_dly2 = "011101000" else 
X_RD_DATA( 1394 downto 1390 ) when CO512_dly2 = "011101001" else 
X_RD_DATA( 1389 downto 1385 ) when CO512_dly2 = "011101010" else 
X_RD_DATA( 1384 downto 1380 ) when CO512_dly2 = "011101011" else 
X_RD_DATA( 1379 downto 1375 ) when CO512_dly2 = "011101100" else 
X_RD_DATA( 1374 downto 1370 ) when CO512_dly2 = "011101101" else 
X_RD_DATA( 1369 downto 1365 ) when CO512_dly2 = "011101110" else 
X_RD_DATA( 1364 downto 1360 ) when CO512_dly2 = "011101111" else 
X_RD_DATA( 1359 downto 1355 ) when CO512_dly2 = "011110000" else 
X_RD_DATA( 1354 downto 1350 ) when CO512_dly2 = "011110001" else 
X_RD_DATA( 1349 downto 1345 ) when CO512_dly2 = "011110010" else 
X_RD_DATA( 1344 downto 1340 ) when CO512_dly2 = "011110011" else 
X_RD_DATA( 1339 downto 1335 ) when CO512_dly2 = "011110100" else 
X_RD_DATA( 1334 downto 1330 ) when CO512_dly2 = "011110101" else 
X_RD_DATA( 1329 downto 1325 ) when CO512_dly2 = "011110110" else 
X_RD_DATA( 1324 downto 1320 ) when CO512_dly2 = "011110111" else 
X_RD_DATA( 1319 downto 1315 ) when CO512_dly2 = "011111000" else 
X_RD_DATA( 1314 downto 1310 ) when CO512_dly2 = "011111001" else 
X_RD_DATA( 1309 downto 1305 ) when CO512_dly2 = "011111010" else 
X_RD_DATA( 1304 downto 1300 ) when CO512_dly2 = "011111011" else 
X_RD_DATA( 1299 downto 1295 ) when CO512_dly2 = "011111100" else 
X_RD_DATA( 1294 downto 1290 ) when CO512_dly2 = "011111101" else 
X_RD_DATA( 1289 downto 1285 ) when CO512_dly2 = "011111110" else 
X_RD_DATA( 1284 downto 1280 ) when CO512_dly2 = "011111111" else 
X_RD_DATA( 1279 downto 1275 ) when CO512_dly2 = "100000000" else 
X_RD_DATA( 1274 downto 1270 ) when CO512_dly2 = "100000001" else 
X_RD_DATA( 1269 downto 1265 ) when CO512_dly2 = "100000010" else 
X_RD_DATA( 1264 downto 1260 ) when CO512_dly2 = "100000011" else 
X_RD_DATA( 1259 downto 1255 ) when CO512_dly2 = "100000100" else 
X_RD_DATA( 1254 downto 1250 ) when CO512_dly2 = "100000101" else 
X_RD_DATA( 1249 downto 1245 ) when CO512_dly2 = "100000110" else 
X_RD_DATA( 1244 downto 1240 ) when CO512_dly2 = "100000111" else 
X_RD_DATA( 1239 downto 1235 ) when CO512_dly2 = "100001000" else 
X_RD_DATA( 1234 downto 1230 ) when CO512_dly2 = "100001001" else 
X_RD_DATA( 1229 downto 1225 ) when CO512_dly2 = "100001010" else 
X_RD_DATA( 1224 downto 1220 ) when CO512_dly2 = "100001011" else 
X_RD_DATA( 1219 downto 1215 ) when CO512_dly2 = "100001100" else 
X_RD_DATA( 1214 downto 1210 ) when CO512_dly2 = "100001101" else 
X_RD_DATA( 1209 downto 1205 ) when CO512_dly2 = "100001110" else 
X_RD_DATA( 1204 downto 1200 ) when CO512_dly2 = "100001111" else 
X_RD_DATA( 1199 downto 1195 ) when CO512_dly2 = "100010000" else 
X_RD_DATA( 1194 downto 1190 ) when CO512_dly2 = "100010001" else 
X_RD_DATA( 1189 downto 1185 ) when CO512_dly2 = "100010010" else 
X_RD_DATA( 1184 downto 1180 ) when CO512_dly2 = "100010011" else 
X_RD_DATA( 1179 downto 1175 ) when CO512_dly2 = "100010100" else 
X_RD_DATA( 1174 downto 1170 ) when CO512_dly2 = "100010101" else 
X_RD_DATA( 1169 downto 1165 ) when CO512_dly2 = "100010110" else 
X_RD_DATA( 1164 downto 1160 ) when CO512_dly2 = "100010111" else 
X_RD_DATA( 1159 downto 1155 ) when CO512_dly2 = "100011000" else 
X_RD_DATA( 1154 downto 1150 ) when CO512_dly2 = "100011001" else 
X_RD_DATA( 1149 downto 1145 ) when CO512_dly2 = "100011010" else 
X_RD_DATA( 1144 downto 1140 ) when CO512_dly2 = "100011011" else 
X_RD_DATA( 1139 downto 1135 ) when CO512_dly2 = "100011100" else 
X_RD_DATA( 1134 downto 1130 ) when CO512_dly2 = "100011101" else 
X_RD_DATA( 1129 downto 1125 ) when CO512_dly2 = "100011110" else 
X_RD_DATA( 1124 downto 1120 ) when CO512_dly2 = "100011111" else 
X_RD_DATA( 1119 downto 1115 ) when CO512_dly2 = "100100000" else 
X_RD_DATA( 1114 downto 1110 ) when CO512_dly2 = "100100001" else 
X_RD_DATA( 1109 downto 1105 ) when CO512_dly2 = "100100010" else 
X_RD_DATA( 1104 downto 1100 ) when CO512_dly2 = "100100011" else 
X_RD_DATA( 1099 downto 1095 ) when CO512_dly2 = "100100100" else 
X_RD_DATA( 1094 downto 1090 ) when CO512_dly2 = "100100101" else 
X_RD_DATA( 1089 downto 1085 ) when CO512_dly2 = "100100110" else 
X_RD_DATA( 1084 downto 1080 ) when CO512_dly2 = "100100111" else 
X_RD_DATA( 1079 downto 1075 ) when CO512_dly2 = "100101000" else 
X_RD_DATA( 1074 downto 1070 ) when CO512_dly2 = "100101001" else 
X_RD_DATA( 1069 downto 1065 ) when CO512_dly2 = "100101010" else 
X_RD_DATA( 1064 downto 1060 ) when CO512_dly2 = "100101011" else 
X_RD_DATA( 1059 downto 1055 ) when CO512_dly2 = "100101100" else 
X_RD_DATA( 1054 downto 1050 ) when CO512_dly2 = "100101101" else 
X_RD_DATA( 1049 downto 1045 ) when CO512_dly2 = "100101110" else 
X_RD_DATA( 1044 downto 1040 ) when CO512_dly2 = "100101111" else 
X_RD_DATA( 1039 downto 1035 ) when CO512_dly2 = "100110000" else 
X_RD_DATA( 1034 downto 1030 ) when CO512_dly2 = "100110001" else 
X_RD_DATA( 1029 downto 1025 ) when CO512_dly2 = "100110010" else 
X_RD_DATA( 1024 downto 1020 ) when CO512_dly2 = "100110011" else 
X_RD_DATA( 1019 downto 1015 ) when CO512_dly2 = "100110100" else 
X_RD_DATA( 1014 downto 1010 ) when CO512_dly2 = "100110101" else 
X_RD_DATA( 1009 downto 1005 ) when CO512_dly2 = "100110110" else 
X_RD_DATA( 1004 downto 1000 ) when CO512_dly2 = "100110111" else 
X_RD_DATA( 999 downto 995 ) when CO512_dly2 = "100111000" else 
X_RD_DATA( 994 downto 990 ) when CO512_dly2 = "100111001" else 
X_RD_DATA( 989 downto 985 ) when CO512_dly2 = "100111010" else 
X_RD_DATA( 984 downto 980 ) when CO512_dly2 = "100111011" else 
X_RD_DATA( 979 downto 975 ) when CO512_dly2 = "100111100" else 
X_RD_DATA( 974 downto 970 ) when CO512_dly2 = "100111101" else 
X_RD_DATA( 969 downto 965 ) when CO512_dly2 = "100111110" else 
X_RD_DATA( 964 downto 960 ) when CO512_dly2 = "100111111" else 
X_RD_DATA( 959 downto 955 ) when CO512_dly2 = "101000000" else 
X_RD_DATA( 954 downto 950 ) when CO512_dly2 = "101000001" else 
X_RD_DATA( 949 downto 945 ) when CO512_dly2 = "101000010" else 
X_RD_DATA( 944 downto 940 ) when CO512_dly2 = "101000011" else 
X_RD_DATA( 939 downto 935 ) when CO512_dly2 = "101000100" else 
X_RD_DATA( 934 downto 930 ) when CO512_dly2 = "101000101" else 
X_RD_DATA( 929 downto 925 ) when CO512_dly2 = "101000110" else 
X_RD_DATA( 924 downto 920 ) when CO512_dly2 = "101000111" else 
X_RD_DATA( 919 downto 915 ) when CO512_dly2 = "101001000" else 
X_RD_DATA( 914 downto 910 ) when CO512_dly2 = "101001001" else 
X_RD_DATA( 909 downto 905 ) when CO512_dly2 = "101001010" else 
X_RD_DATA( 904 downto 900 ) when CO512_dly2 = "101001011" else 
X_RD_DATA( 899 downto 895 ) when CO512_dly2 = "101001100" else 
X_RD_DATA( 894 downto 890 ) when CO512_dly2 = "101001101" else 
X_RD_DATA( 889 downto 885 ) when CO512_dly2 = "101001110" else 
X_RD_DATA( 884 downto 880 ) when CO512_dly2 = "101001111" else 
X_RD_DATA( 879 downto 875 ) when CO512_dly2 = "101010000" else 
X_RD_DATA( 874 downto 870 ) when CO512_dly2 = "101010001" else 
X_RD_DATA( 869 downto 865 ) when CO512_dly2 = "101010010" else 
X_RD_DATA( 864 downto 860 ) when CO512_dly2 = "101010011" else 
X_RD_DATA( 859 downto 855 ) when CO512_dly2 = "101010100" else 
X_RD_DATA( 854 downto 850 ) when CO512_dly2 = "101010101" else 
X_RD_DATA( 849 downto 845 ) when CO512_dly2 = "101010110" else 
X_RD_DATA( 844 downto 840 ) when CO512_dly2 = "101010111" else 
X_RD_DATA( 839 downto 835 ) when CO512_dly2 = "101011000" else 
X_RD_DATA( 834 downto 830 ) when CO512_dly2 = "101011001" else 
X_RD_DATA( 829 downto 825 ) when CO512_dly2 = "101011010" else 
X_RD_DATA( 824 downto 820 ) when CO512_dly2 = "101011011" else 
X_RD_DATA( 819 downto 815 ) when CO512_dly2 = "101011100" else 
X_RD_DATA( 814 downto 810 ) when CO512_dly2 = "101011101" else 
X_RD_DATA( 809 downto 805 ) when CO512_dly2 = "101011110" else 
X_RD_DATA( 804 downto 800 ) when CO512_dly2 = "101011111" else 
X_RD_DATA( 799 downto 795 ) when CO512_dly2 = "101100000" else 
X_RD_DATA( 794 downto 790 ) when CO512_dly2 = "101100001" else 
X_RD_DATA( 789 downto 785 ) when CO512_dly2 = "101100010" else 
X_RD_DATA( 784 downto 780 ) when CO512_dly2 = "101100011" else 
X_RD_DATA( 779 downto 775 ) when CO512_dly2 = "101100100" else 
X_RD_DATA( 774 downto 770 ) when CO512_dly2 = "101100101" else 
X_RD_DATA( 769 downto 765 ) when CO512_dly2 = "101100110" else 
X_RD_DATA( 764 downto 760 ) when CO512_dly2 = "101100111" else 
X_RD_DATA( 759 downto 755 ) when CO512_dly2 = "101101000" else 
X_RD_DATA( 754 downto 750 ) when CO512_dly2 = "101101001" else 
X_RD_DATA( 749 downto 745 ) when CO512_dly2 = "101101010" else 
X_RD_DATA( 744 downto 740 ) when CO512_dly2 = "101101011" else 
X_RD_DATA( 739 downto 735 ) when CO512_dly2 = "101101100" else 
X_RD_DATA( 734 downto 730 ) when CO512_dly2 = "101101101" else 
X_RD_DATA( 729 downto 725 ) when CO512_dly2 = "101101110" else 
X_RD_DATA( 724 downto 720 ) when CO512_dly2 = "101101111" else 
X_RD_DATA( 719 downto 715 ) when CO512_dly2 = "101110000" else 
X_RD_DATA( 714 downto 710 ) when CO512_dly2 = "101110001" else 
X_RD_DATA( 709 downto 705 ) when CO512_dly2 = "101110010" else 
X_RD_DATA( 704 downto 700 ) when CO512_dly2 = "101110011" else 
X_RD_DATA( 699 downto 695 ) when CO512_dly2 = "101110100" else 
X_RD_DATA( 694 downto 690 ) when CO512_dly2 = "101110101" else 
X_RD_DATA( 689 downto 685 ) when CO512_dly2 = "101110110" else 
X_RD_DATA( 684 downto 680 ) when CO512_dly2 = "101110111" else 
X_RD_DATA( 679 downto 675 ) when CO512_dly2 = "101111000" else 
X_RD_DATA( 674 downto 670 ) when CO512_dly2 = "101111001" else 
X_RD_DATA( 669 downto 665 ) when CO512_dly2 = "101111010" else 
X_RD_DATA( 664 downto 660 ) when CO512_dly2 = "101111011" else 
X_RD_DATA( 659 downto 655 ) when CO512_dly2 = "101111100" else 
X_RD_DATA( 654 downto 650 ) when CO512_dly2 = "101111101" else 
X_RD_DATA( 649 downto 645 ) when CO512_dly2 = "101111110" else 
X_RD_DATA( 644 downto 640 ) when CO512_dly2 = "101111111" else 
X_RD_DATA( 639 downto 635 ) when CO512_dly2 = "110000000" else 
X_RD_DATA( 634 downto 630 ) when CO512_dly2 = "110000001" else 
X_RD_DATA( 629 downto 625 ) when CO512_dly2 = "110000010" else 
X_RD_DATA( 624 downto 620 ) when CO512_dly2 = "110000011" else 
X_RD_DATA( 619 downto 615 ) when CO512_dly2 = "110000100" else 
X_RD_DATA( 614 downto 610 ) when CO512_dly2 = "110000101" else 
X_RD_DATA( 609 downto 605 ) when CO512_dly2 = "110000110" else 
X_RD_DATA( 604 downto 600 ) when CO512_dly2 = "110000111" else 
X_RD_DATA( 599 downto 595 ) when CO512_dly2 = "110001000" else 
X_RD_DATA( 594 downto 590 ) when CO512_dly2 = "110001001" else 
X_RD_DATA( 589 downto 585 ) when CO512_dly2 = "110001010" else 
X_RD_DATA( 584 downto 580 ) when CO512_dly2 = "110001011" else 
X_RD_DATA( 579 downto 575 ) when CO512_dly2 = "110001100" else 
X_RD_DATA( 574 downto 570 ) when CO512_dly2 = "110001101" else 
X_RD_DATA( 569 downto 565 ) when CO512_dly2 = "110001110" else 
X_RD_DATA( 564 downto 560 ) when CO512_dly2 = "110001111" else 
X_RD_DATA( 559 downto 555 ) when CO512_dly2 = "110010000" else 
X_RD_DATA( 554 downto 550 ) when CO512_dly2 = "110010001" else 
X_RD_DATA( 549 downto 545 ) when CO512_dly2 = "110010010" else 
X_RD_DATA( 544 downto 540 ) when CO512_dly2 = "110010011" else 
X_RD_DATA( 539 downto 535 ) when CO512_dly2 = "110010100" else 
X_RD_DATA( 534 downto 530 ) when CO512_dly2 = "110010101" else 
X_RD_DATA( 529 downto 525 ) when CO512_dly2 = "110010110" else 
X_RD_DATA( 524 downto 520 ) when CO512_dly2 = "110010111" else 
X_RD_DATA( 519 downto 515 ) when CO512_dly2 = "110011000" else 
X_RD_DATA( 514 downto 510 ) when CO512_dly2 = "110011001" else 
X_RD_DATA( 509 downto 505 ) when CO512_dly2 = "110011010" else 
X_RD_DATA( 504 downto 500 ) when CO512_dly2 = "110011011" else 
X_RD_DATA( 499 downto 495 ) when CO512_dly2 = "110011100" else 
X_RD_DATA( 494 downto 490 ) when CO512_dly2 = "110011101" else 
X_RD_DATA( 489 downto 485 ) when CO512_dly2 = "110011110" else 
X_RD_DATA( 484 downto 480 ) when CO512_dly2 = "110011111" else 
X_RD_DATA( 479 downto 475 ) when CO512_dly2 = "110100000" else 
X_RD_DATA( 474 downto 470 ) when CO512_dly2 = "110100001" else 
X_RD_DATA( 469 downto 465 ) when CO512_dly2 = "110100010" else 
X_RD_DATA( 464 downto 460 ) when CO512_dly2 = "110100011" else 
X_RD_DATA( 459 downto 455 ) when CO512_dly2 = "110100100" else 
X_RD_DATA( 454 downto 450 ) when CO512_dly2 = "110100101" else 
X_RD_DATA( 449 downto 445 ) when CO512_dly2 = "110100110" else 
X_RD_DATA( 444 downto 440 ) when CO512_dly2 = "110100111" else 
X_RD_DATA( 439 downto 435 ) when CO512_dly2 = "110101000" else 
X_RD_DATA( 434 downto 430 ) when CO512_dly2 = "110101001" else 
X_RD_DATA( 429 downto 425 ) when CO512_dly2 = "110101010" else 
X_RD_DATA( 424 downto 420 ) when CO512_dly2 = "110101011" else 
X_RD_DATA( 419 downto 415 ) when CO512_dly2 = "110101100" else 
X_RD_DATA( 414 downto 410 ) when CO512_dly2 = "110101101" else 
X_RD_DATA( 409 downto 405 ) when CO512_dly2 = "110101110" else 
X_RD_DATA( 404 downto 400 ) when CO512_dly2 = "110101111" else 
X_RD_DATA( 399 downto 395 ) when CO512_dly2 = "110110000" else 
X_RD_DATA( 394 downto 390 ) when CO512_dly2 = "110110001" else 
X_RD_DATA( 389 downto 385 ) when CO512_dly2 = "110110010" else 
X_RD_DATA( 384 downto 380 ) when CO512_dly2 = "110110011" else 
X_RD_DATA( 379 downto 375 ) when CO512_dly2 = "110110100" else 
X_RD_DATA( 374 downto 370 ) when CO512_dly2 = "110110101" else 
X_RD_DATA( 369 downto 365 ) when CO512_dly2 = "110110110" else 
X_RD_DATA( 364 downto 360 ) when CO512_dly2 = "110110111" else 
X_RD_DATA( 359 downto 355 ) when CO512_dly2 = "110111000" else 
X_RD_DATA( 354 downto 350 ) when CO512_dly2 = "110111001" else 
X_RD_DATA( 349 downto 345 ) when CO512_dly2 = "110111010" else 
X_RD_DATA( 344 downto 340 ) when CO512_dly2 = "110111011" else 
X_RD_DATA( 339 downto 335 ) when CO512_dly2 = "110111100" else 
X_RD_DATA( 334 downto 330 ) when CO512_dly2 = "110111101" else 
X_RD_DATA( 329 downto 325 ) when CO512_dly2 = "110111110" else 
X_RD_DATA( 324 downto 320 ) when CO512_dly2 = "110111111" else 
X_RD_DATA( 319 downto 315 ) when CO512_dly2 = "111000000" else 
X_RD_DATA( 314 downto 310 ) when CO512_dly2 = "111000001" else 
X_RD_DATA( 309 downto 305 ) when CO512_dly2 = "111000010" else 
X_RD_DATA( 304 downto 300 ) when CO512_dly2 = "111000011" else 
X_RD_DATA( 299 downto 295 ) when CO512_dly2 = "111000100" else 
X_RD_DATA( 294 downto 290 ) when CO512_dly2 = "111000101" else 
X_RD_DATA( 289 downto 285 ) when CO512_dly2 = "111000110" else 
X_RD_DATA( 284 downto 280 ) when CO512_dly2 = "111000111" else 
X_RD_DATA( 279 downto 275 ) when CO512_dly2 = "111001000" else 
X_RD_DATA( 274 downto 270 ) when CO512_dly2 = "111001001" else 
X_RD_DATA( 269 downto 265 ) when CO512_dly2 = "111001010" else 
X_RD_DATA( 264 downto 260 ) when CO512_dly2 = "111001011" else 
X_RD_DATA( 259 downto 255 ) when CO512_dly2 = "111001100" else 
X_RD_DATA( 254 downto 250 ) when CO512_dly2 = "111001101" else 
X_RD_DATA( 249 downto 245 ) when CO512_dly2 = "111001110" else 
X_RD_DATA( 244 downto 240 ) when CO512_dly2 = "111001111" else 
X_RD_DATA( 239 downto 235 ) when CO512_dly2 = "111010000" else 
X_RD_DATA( 234 downto 230 ) when CO512_dly2 = "111010001" else 
X_RD_DATA( 229 downto 225 ) when CO512_dly2 = "111010010" else 
X_RD_DATA( 224 downto 220 ) when CO512_dly2 = "111010011" else 
X_RD_DATA( 219 downto 215 ) when CO512_dly2 = "111010100" else 
X_RD_DATA( 214 downto 210 ) when CO512_dly2 = "111010101" else 
X_RD_DATA( 209 downto 205 ) when CO512_dly2 = "111010110" else 
X_RD_DATA( 204 downto 200 ) when CO512_dly2 = "111010111" else 
X_RD_DATA( 199 downto 195 ) when CO512_dly2 = "111011000" else 
X_RD_DATA( 194 downto 190 ) when CO512_dly2 = "111011001" else 
X_RD_DATA( 189 downto 185 ) when CO512_dly2 = "111011010" else 
X_RD_DATA( 184 downto 180 ) when CO512_dly2 = "111011011" else 
X_RD_DATA( 179 downto 175 ) when CO512_dly2 = "111011100" else 
X_RD_DATA( 174 downto 170 ) when CO512_dly2 = "111011101" else 
X_RD_DATA( 169 downto 165 ) when CO512_dly2 = "111011110" else 
X_RD_DATA( 164 downto 160 ) when CO512_dly2 = "111011111" else 
X_RD_DATA( 159 downto 155 ) when CO512_dly2 = "111100000" else 
X_RD_DATA( 154 downto 150 ) when CO512_dly2 = "111100001" else 
X_RD_DATA( 149 downto 145 ) when CO512_dly2 = "111100010" else 
X_RD_DATA( 144 downto 140 ) when CO512_dly2 = "111100011" else 
X_RD_DATA( 139 downto 135 ) when CO512_dly2 = "111100100" else 
X_RD_DATA( 134 downto 130 ) when CO512_dly2 = "111100101" else 
X_RD_DATA( 129 downto 125 ) when CO512_dly2 = "111100110" else 
X_RD_DATA( 124 downto 120 ) when CO512_dly2 = "111100111" else 
X_RD_DATA( 119 downto 115 ) when CO512_dly2 = "111101000" else 
X_RD_DATA( 114 downto 110 ) when CO512_dly2 = "111101001" else 
X_RD_DATA( 109 downto 105 ) when CO512_dly2 = "111101010" else 
X_RD_DATA( 104 downto 100 ) when CO512_dly2 = "111101011" else 
X_RD_DATA( 99 downto 95 ) when CO512_dly2 = "111101100" else 
X_RD_DATA( 94 downto 90 ) when CO512_dly2 = "111101101" else 
X_RD_DATA( 89 downto 85 ) when CO512_dly2 = "111101110" else 
X_RD_DATA( 84 downto 80 ) when CO512_dly2 = "111101111" else 
X_RD_DATA( 79 downto 75 ) when CO512_dly2 = "111110000" else 
X_RD_DATA( 74 downto 70 ) when CO512_dly2 = "111110001" else 
X_RD_DATA( 69 downto 65 ) when CO512_dly2 = "111110010" else 
X_RD_DATA( 64 downto 60 ) when CO512_dly2 = "111110011" else 
X_RD_DATA( 59 downto 55 ) when CO512_dly2 = "111110100" else 
X_RD_DATA( 54 downto 50 ) when CO512_dly2 = "111110101" else 
X_RD_DATA( 49 downto 45 ) when CO512_dly2 = "111110110" else 
X_RD_DATA( 44 downto 40 ) when CO512_dly2 = "111110111" else 
X_RD_DATA( 39 downto 35 ) when CO512_dly2 = "111111000" else 
X_RD_DATA( 34 downto 30 ) when CO512_dly2 = "111111001" else 
X_RD_DATA( 29 downto 25 ) when CO512_dly2 = "111111010" else 
X_RD_DATA( 24 downto 20 ) when CO512_dly2 = "111111011" else 
X_RD_DATA( 19 downto 15 ) when CO512_dly2 = "111111100" else 
X_RD_DATA( 14 downto 10 ) when CO512_dly2 = "111111101" else 
X_RD_DATA( 9 downto 5 ) when CO512_dly2 = "111111110" else 
X_RD_DATA( 4 downto 0 ); -- when CO512_dly2 = "111111111" else 


W11Addr_L <= CO9;
W11Addr_H <= ("0" & CO512 & "000") + ("0000" & CO512); -- CO512 x 9
W11Addr_prev <= W11Addr_H + ("00000000" & W11Addr_L);

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		W11Addr <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		if(W11Addr_prev >= -448 and W11Addr_prev <= -1) then -- note that this is Signed
			W11Addr <= (others => '0');
		else
			W11Addr <= W11Addr_prev;
		end if;
	end if;  
end process;

---------------------------------------------------------
enb_a <= '1' when W11Sel = 1 and CO512 = 511 and CO9 = 8 else '0';
enb_b <= '1' when W11Sel = 2 and CO512 = 511 and CO9 = 8 else '0';
enb_c <= '1' when W11Sel = 3 and CO512 = 511 and CO9 = 8 else '0';
enb_d <= '1' when W11Sel = 4 and CO512 = 511 and CO9 = 8 else '0';
enb_e <= '1' when W11Sel = 5 and CO512 = 511 and CO9 = 8 else '0';
enb_f <= '1' when W11Sel = 6 and CO512 = 511 and CO9 = 8 else '0';
enb_g <= '1' when W11Sel = 7 and CO512 = 511 and CO9 = 8 else '0';
enb_h <= '1' when W11Sel = 8 and CO512 = 511 and CO9 = 8 else '0';
enb_i <= '1' when W11Sel = 9 and CO512 = 511 and CO9 = 8 else '0';
enb_j <= '1' when W11Sel = 10 and CO512 = 511 and CO9 = 8 else '0';
enb_k <= '1' when W11Sel = 11 and CO512 = 511 and CO9 = 8 else '0';
enb_l <= '1' when W11Sel = 12 and CO512 = 511 and CO9 = 8 else '0';
enb_m <= '1' when W11Sel = 13 and CO512 = 511 and CO9 = 8 else '0';
enb_n <= '1' when W11Sel = 14 and CO512 = 511 and CO9 = 8 else '0';
enb_o <= '1' when W11Sel = 15 and CO512 = 511 and CO9 = 8 else '0';
enb_p <= '1' when W11Sel = 0 and CO512 = 511 and CO9 = 8 else '0';

-----------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_000_00 <= Relu_000;
		elsif(enb_b = '1') then
			reg_000_01 <= Relu_000;
		elsif(enb_c = '1') then
			reg_000_02 <= Relu_000;
		elsif(enb_d = '1') then
			reg_000_03 <= Relu_000;
		elsif(enb_e = '1') then
			reg_000_04 <= Relu_000;
		elsif(enb_f = '1') then
			reg_000_05 <= Relu_000;
		elsif(enb_g = '1') then
			reg_000_06 <= Relu_000;
		elsif(enb_h = '1') then
			reg_000_07 <= Relu_000;
		elsif(enb_i = '1') then
			reg_000_08 <= Relu_000;
		elsif(enb_j = '1') then
			reg_000_09 <= Relu_000;
		elsif(enb_k = '1') then
			reg_000_10 <= Relu_000;
		elsif(enb_l = '1') then
			reg_000_11 <= Relu_000;
		elsif(enb_m = '1') then
			reg_000_12 <= Relu_000;
		elsif(enb_n = '1') then
			reg_000_13 <= Relu_000;
		elsif(enb_o = '1') then
			reg_000_14 <= Relu_000;
		elsif(enb_p = '1') then
			reg_000_15 <= Relu_000;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_001_00 <= Relu_001;
		elsif(enb_b = '1') then
			reg_001_01 <= Relu_001;
		elsif(enb_c = '1') then
			reg_001_02 <= Relu_001;
		elsif(enb_d = '1') then
			reg_001_03 <= Relu_001;
		elsif(enb_e = '1') then
			reg_001_04 <= Relu_001;
		elsif(enb_f = '1') then
			reg_001_05 <= Relu_001;
		elsif(enb_g = '1') then
			reg_001_06 <= Relu_001;
		elsif(enb_h = '1') then
			reg_001_07 <= Relu_001;
		elsif(enb_i = '1') then
			reg_001_08 <= Relu_001;
		elsif(enb_j = '1') then
			reg_001_09 <= Relu_001;
		elsif(enb_k = '1') then
			reg_001_10 <= Relu_001;
		elsif(enb_l = '1') then
			reg_001_11 <= Relu_001;
		elsif(enb_m = '1') then
			reg_001_12 <= Relu_001;
		elsif(enb_n = '1') then
			reg_001_13 <= Relu_001;
		elsif(enb_o = '1') then
			reg_001_14 <= Relu_001;
		elsif(enb_p = '1') then
			reg_001_15 <= Relu_001;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_002_00 <= Relu_002;
		elsif(enb_b = '1') then
			reg_002_01 <= Relu_002;
		elsif(enb_c = '1') then
			reg_002_02 <= Relu_002;
		elsif(enb_d = '1') then
			reg_002_03 <= Relu_002;
		elsif(enb_e = '1') then
			reg_002_04 <= Relu_002;
		elsif(enb_f = '1') then
			reg_002_05 <= Relu_002;
		elsif(enb_g = '1') then
			reg_002_06 <= Relu_002;
		elsif(enb_h = '1') then
			reg_002_07 <= Relu_002;
		elsif(enb_i = '1') then
			reg_002_08 <= Relu_002;
		elsif(enb_j = '1') then
			reg_002_09 <= Relu_002;
		elsif(enb_k = '1') then
			reg_002_10 <= Relu_002;
		elsif(enb_l = '1') then
			reg_002_11 <= Relu_002;
		elsif(enb_m = '1') then
			reg_002_12 <= Relu_002;
		elsif(enb_n = '1') then
			reg_002_13 <= Relu_002;
		elsif(enb_o = '1') then
			reg_002_14 <= Relu_002;
		elsif(enb_p = '1') then
			reg_002_15 <= Relu_002;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_003_00 <= Relu_003;
		elsif(enb_b = '1') then
			reg_003_01 <= Relu_003;
		elsif(enb_c = '1') then
			reg_003_02 <= Relu_003;
		elsif(enb_d = '1') then
			reg_003_03 <= Relu_003;
		elsif(enb_e = '1') then
			reg_003_04 <= Relu_003;
		elsif(enb_f = '1') then
			reg_003_05 <= Relu_003;
		elsif(enb_g = '1') then
			reg_003_06 <= Relu_003;
		elsif(enb_h = '1') then
			reg_003_07 <= Relu_003;
		elsif(enb_i = '1') then
			reg_003_08 <= Relu_003;
		elsif(enb_j = '1') then
			reg_003_09 <= Relu_003;
		elsif(enb_k = '1') then
			reg_003_10 <= Relu_003;
		elsif(enb_l = '1') then
			reg_003_11 <= Relu_003;
		elsif(enb_m = '1') then
			reg_003_12 <= Relu_003;
		elsif(enb_n = '1') then
			reg_003_13 <= Relu_003;
		elsif(enb_o = '1') then
			reg_003_14 <= Relu_003;
		elsif(enb_p = '1') then
			reg_003_15 <= Relu_003;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_004_00 <= Relu_004;
		elsif(enb_b = '1') then
			reg_004_01 <= Relu_004;
		elsif(enb_c = '1') then
			reg_004_02 <= Relu_004;
		elsif(enb_d = '1') then
			reg_004_03 <= Relu_004;
		elsif(enb_e = '1') then
			reg_004_04 <= Relu_004;
		elsif(enb_f = '1') then
			reg_004_05 <= Relu_004;
		elsif(enb_g = '1') then
			reg_004_06 <= Relu_004;
		elsif(enb_h = '1') then
			reg_004_07 <= Relu_004;
		elsif(enb_i = '1') then
			reg_004_08 <= Relu_004;
		elsif(enb_j = '1') then
			reg_004_09 <= Relu_004;
		elsif(enb_k = '1') then
			reg_004_10 <= Relu_004;
		elsif(enb_l = '1') then
			reg_004_11 <= Relu_004;
		elsif(enb_m = '1') then
			reg_004_12 <= Relu_004;
		elsif(enb_n = '1') then
			reg_004_13 <= Relu_004;
		elsif(enb_o = '1') then
			reg_004_14 <= Relu_004;
		elsif(enb_p = '1') then
			reg_004_15 <= Relu_004;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_005_00 <= Relu_005;
		elsif(enb_b = '1') then
			reg_005_01 <= Relu_005;
		elsif(enb_c = '1') then
			reg_005_02 <= Relu_005;
		elsif(enb_d = '1') then
			reg_005_03 <= Relu_005;
		elsif(enb_e = '1') then
			reg_005_04 <= Relu_005;
		elsif(enb_f = '1') then
			reg_005_05 <= Relu_005;
		elsif(enb_g = '1') then
			reg_005_06 <= Relu_005;
		elsif(enb_h = '1') then
			reg_005_07 <= Relu_005;
		elsif(enb_i = '1') then
			reg_005_08 <= Relu_005;
		elsif(enb_j = '1') then
			reg_005_09 <= Relu_005;
		elsif(enb_k = '1') then
			reg_005_10 <= Relu_005;
		elsif(enb_l = '1') then
			reg_005_11 <= Relu_005;
		elsif(enb_m = '1') then
			reg_005_12 <= Relu_005;
		elsif(enb_n = '1') then
			reg_005_13 <= Relu_005;
		elsif(enb_o = '1') then
			reg_005_14 <= Relu_005;
		elsif(enb_p = '1') then
			reg_005_15 <= Relu_005;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_006_00 <= Relu_006;
		elsif(enb_b = '1') then
			reg_006_01 <= Relu_006;
		elsif(enb_c = '1') then
			reg_006_02 <= Relu_006;
		elsif(enb_d = '1') then
			reg_006_03 <= Relu_006;
		elsif(enb_e = '1') then
			reg_006_04 <= Relu_006;
		elsif(enb_f = '1') then
			reg_006_05 <= Relu_006;
		elsif(enb_g = '1') then
			reg_006_06 <= Relu_006;
		elsif(enb_h = '1') then
			reg_006_07 <= Relu_006;
		elsif(enb_i = '1') then
			reg_006_08 <= Relu_006;
		elsif(enb_j = '1') then
			reg_006_09 <= Relu_006;
		elsif(enb_k = '1') then
			reg_006_10 <= Relu_006;
		elsif(enb_l = '1') then
			reg_006_11 <= Relu_006;
		elsif(enb_m = '1') then
			reg_006_12 <= Relu_006;
		elsif(enb_n = '1') then
			reg_006_13 <= Relu_006;
		elsif(enb_o = '1') then
			reg_006_14 <= Relu_006;
		elsif(enb_p = '1') then
			reg_006_15 <= Relu_006;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_007_00 <= Relu_007;
		elsif(enb_b = '1') then
			reg_007_01 <= Relu_007;
		elsif(enb_c = '1') then
			reg_007_02 <= Relu_007;
		elsif(enb_d = '1') then
			reg_007_03 <= Relu_007;
		elsif(enb_e = '1') then
			reg_007_04 <= Relu_007;
		elsif(enb_f = '1') then
			reg_007_05 <= Relu_007;
		elsif(enb_g = '1') then
			reg_007_06 <= Relu_007;
		elsif(enb_h = '1') then
			reg_007_07 <= Relu_007;
		elsif(enb_i = '1') then
			reg_007_08 <= Relu_007;
		elsif(enb_j = '1') then
			reg_007_09 <= Relu_007;
		elsif(enb_k = '1') then
			reg_007_10 <= Relu_007;
		elsif(enb_l = '1') then
			reg_007_11 <= Relu_007;
		elsif(enb_m = '1') then
			reg_007_12 <= Relu_007;
		elsif(enb_n = '1') then
			reg_007_13 <= Relu_007;
		elsif(enb_o = '1') then
			reg_007_14 <= Relu_007;
		elsif(enb_p = '1') then
			reg_007_15 <= Relu_007;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_008_00 <= Relu_008;
		elsif(enb_b = '1') then
			reg_008_01 <= Relu_008;
		elsif(enb_c = '1') then
			reg_008_02 <= Relu_008;
		elsif(enb_d = '1') then
			reg_008_03 <= Relu_008;
		elsif(enb_e = '1') then
			reg_008_04 <= Relu_008;
		elsif(enb_f = '1') then
			reg_008_05 <= Relu_008;
		elsif(enb_g = '1') then
			reg_008_06 <= Relu_008;
		elsif(enb_h = '1') then
			reg_008_07 <= Relu_008;
		elsif(enb_i = '1') then
			reg_008_08 <= Relu_008;
		elsif(enb_j = '1') then
			reg_008_09 <= Relu_008;
		elsif(enb_k = '1') then
			reg_008_10 <= Relu_008;
		elsif(enb_l = '1') then
			reg_008_11 <= Relu_008;
		elsif(enb_m = '1') then
			reg_008_12 <= Relu_008;
		elsif(enb_n = '1') then
			reg_008_13 <= Relu_008;
		elsif(enb_o = '1') then
			reg_008_14 <= Relu_008;
		elsif(enb_p = '1') then
			reg_008_15 <= Relu_008;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_009_00 <= Relu_009;
		elsif(enb_b = '1') then
			reg_009_01 <= Relu_009;
		elsif(enb_c = '1') then
			reg_009_02 <= Relu_009;
		elsif(enb_d = '1') then
			reg_009_03 <= Relu_009;
		elsif(enb_e = '1') then
			reg_009_04 <= Relu_009;
		elsif(enb_f = '1') then
			reg_009_05 <= Relu_009;
		elsif(enb_g = '1') then
			reg_009_06 <= Relu_009;
		elsif(enb_h = '1') then
			reg_009_07 <= Relu_009;
		elsif(enb_i = '1') then
			reg_009_08 <= Relu_009;
		elsif(enb_j = '1') then
			reg_009_09 <= Relu_009;
		elsif(enb_k = '1') then
			reg_009_10 <= Relu_009;
		elsif(enb_l = '1') then
			reg_009_11 <= Relu_009;
		elsif(enb_m = '1') then
			reg_009_12 <= Relu_009;
		elsif(enb_n = '1') then
			reg_009_13 <= Relu_009;
		elsif(enb_o = '1') then
			reg_009_14 <= Relu_009;
		elsif(enb_p = '1') then
			reg_009_15 <= Relu_009;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_010_00 <= Relu_010;
		elsif(enb_b = '1') then
			reg_010_01 <= Relu_010;
		elsif(enb_c = '1') then
			reg_010_02 <= Relu_010;
		elsif(enb_d = '1') then
			reg_010_03 <= Relu_010;
		elsif(enb_e = '1') then
			reg_010_04 <= Relu_010;
		elsif(enb_f = '1') then
			reg_010_05 <= Relu_010;
		elsif(enb_g = '1') then
			reg_010_06 <= Relu_010;
		elsif(enb_h = '1') then
			reg_010_07 <= Relu_010;
		elsif(enb_i = '1') then
			reg_010_08 <= Relu_010;
		elsif(enb_j = '1') then
			reg_010_09 <= Relu_010;
		elsif(enb_k = '1') then
			reg_010_10 <= Relu_010;
		elsif(enb_l = '1') then
			reg_010_11 <= Relu_010;
		elsif(enb_m = '1') then
			reg_010_12 <= Relu_010;
		elsif(enb_n = '1') then
			reg_010_13 <= Relu_010;
		elsif(enb_o = '1') then
			reg_010_14 <= Relu_010;
		elsif(enb_p = '1') then
			reg_010_15 <= Relu_010;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_011_00 <= Relu_011;
		elsif(enb_b = '1') then
			reg_011_01 <= Relu_011;
		elsif(enb_c = '1') then
			reg_011_02 <= Relu_011;
		elsif(enb_d = '1') then
			reg_011_03 <= Relu_011;
		elsif(enb_e = '1') then
			reg_011_04 <= Relu_011;
		elsif(enb_f = '1') then
			reg_011_05 <= Relu_011;
		elsif(enb_g = '1') then
			reg_011_06 <= Relu_011;
		elsif(enb_h = '1') then
			reg_011_07 <= Relu_011;
		elsif(enb_i = '1') then
			reg_011_08 <= Relu_011;
		elsif(enb_j = '1') then
			reg_011_09 <= Relu_011;
		elsif(enb_k = '1') then
			reg_011_10 <= Relu_011;
		elsif(enb_l = '1') then
			reg_011_11 <= Relu_011;
		elsif(enb_m = '1') then
			reg_011_12 <= Relu_011;
		elsif(enb_n = '1') then
			reg_011_13 <= Relu_011;
		elsif(enb_o = '1') then
			reg_011_14 <= Relu_011;
		elsif(enb_p = '1') then
			reg_011_15 <= Relu_011;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_012_00 <= Relu_012;
		elsif(enb_b = '1') then
			reg_012_01 <= Relu_012;
		elsif(enb_c = '1') then
			reg_012_02 <= Relu_012;
		elsif(enb_d = '1') then
			reg_012_03 <= Relu_012;
		elsif(enb_e = '1') then
			reg_012_04 <= Relu_012;
		elsif(enb_f = '1') then
			reg_012_05 <= Relu_012;
		elsif(enb_g = '1') then
			reg_012_06 <= Relu_012;
		elsif(enb_h = '1') then
			reg_012_07 <= Relu_012;
		elsif(enb_i = '1') then
			reg_012_08 <= Relu_012;
		elsif(enb_j = '1') then
			reg_012_09 <= Relu_012;
		elsif(enb_k = '1') then
			reg_012_10 <= Relu_012;
		elsif(enb_l = '1') then
			reg_012_11 <= Relu_012;
		elsif(enb_m = '1') then
			reg_012_12 <= Relu_012;
		elsif(enb_n = '1') then
			reg_012_13 <= Relu_012;
		elsif(enb_o = '1') then
			reg_012_14 <= Relu_012;
		elsif(enb_p = '1') then
			reg_012_15 <= Relu_012;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_013_00 <= Relu_013;
		elsif(enb_b = '1') then
			reg_013_01 <= Relu_013;
		elsif(enb_c = '1') then
			reg_013_02 <= Relu_013;
		elsif(enb_d = '1') then
			reg_013_03 <= Relu_013;
		elsif(enb_e = '1') then
			reg_013_04 <= Relu_013;
		elsif(enb_f = '1') then
			reg_013_05 <= Relu_013;
		elsif(enb_g = '1') then
			reg_013_06 <= Relu_013;
		elsif(enb_h = '1') then
			reg_013_07 <= Relu_013;
		elsif(enb_i = '1') then
			reg_013_08 <= Relu_013;
		elsif(enb_j = '1') then
			reg_013_09 <= Relu_013;
		elsif(enb_k = '1') then
			reg_013_10 <= Relu_013;
		elsif(enb_l = '1') then
			reg_013_11 <= Relu_013;
		elsif(enb_m = '1') then
			reg_013_12 <= Relu_013;
		elsif(enb_n = '1') then
			reg_013_13 <= Relu_013;
		elsif(enb_o = '1') then
			reg_013_14 <= Relu_013;
		elsif(enb_p = '1') then
			reg_013_15 <= Relu_013;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_014_00 <= Relu_014;
		elsif(enb_b = '1') then
			reg_014_01 <= Relu_014;
		elsif(enb_c = '1') then
			reg_014_02 <= Relu_014;
		elsif(enb_d = '1') then
			reg_014_03 <= Relu_014;
		elsif(enb_e = '1') then
			reg_014_04 <= Relu_014;
		elsif(enb_f = '1') then
			reg_014_05 <= Relu_014;
		elsif(enb_g = '1') then
			reg_014_06 <= Relu_014;
		elsif(enb_h = '1') then
			reg_014_07 <= Relu_014;
		elsif(enb_i = '1') then
			reg_014_08 <= Relu_014;
		elsif(enb_j = '1') then
			reg_014_09 <= Relu_014;
		elsif(enb_k = '1') then
			reg_014_10 <= Relu_014;
		elsif(enb_l = '1') then
			reg_014_11 <= Relu_014;
		elsif(enb_m = '1') then
			reg_014_12 <= Relu_014;
		elsif(enb_n = '1') then
			reg_014_13 <= Relu_014;
		elsif(enb_o = '1') then
			reg_014_14 <= Relu_014;
		elsif(enb_p = '1') then
			reg_014_15 <= Relu_014;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_015_00 <= Relu_015;
		elsif(enb_b = '1') then
			reg_015_01 <= Relu_015;
		elsif(enb_c = '1') then
			reg_015_02 <= Relu_015;
		elsif(enb_d = '1') then
			reg_015_03 <= Relu_015;
		elsif(enb_e = '1') then
			reg_015_04 <= Relu_015;
		elsif(enb_f = '1') then
			reg_015_05 <= Relu_015;
		elsif(enb_g = '1') then
			reg_015_06 <= Relu_015;
		elsif(enb_h = '1') then
			reg_015_07 <= Relu_015;
		elsif(enb_i = '1') then
			reg_015_08 <= Relu_015;
		elsif(enb_j = '1') then
			reg_015_09 <= Relu_015;
		elsif(enb_k = '1') then
			reg_015_10 <= Relu_015;
		elsif(enb_l = '1') then
			reg_015_11 <= Relu_015;
		elsif(enb_m = '1') then
			reg_015_12 <= Relu_015;
		elsif(enb_n = '1') then
			reg_015_13 <= Relu_015;
		elsif(enb_o = '1') then
			reg_015_14 <= Relu_015;
		elsif(enb_p = '1') then
			reg_015_15 <= Relu_015;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_016_00 <= Relu_016;
		elsif(enb_b = '1') then
			reg_016_01 <= Relu_016;
		elsif(enb_c = '1') then
			reg_016_02 <= Relu_016;
		elsif(enb_d = '1') then
			reg_016_03 <= Relu_016;
		elsif(enb_e = '1') then
			reg_016_04 <= Relu_016;
		elsif(enb_f = '1') then
			reg_016_05 <= Relu_016;
		elsif(enb_g = '1') then
			reg_016_06 <= Relu_016;
		elsif(enb_h = '1') then
			reg_016_07 <= Relu_016;
		elsif(enb_i = '1') then
			reg_016_08 <= Relu_016;
		elsif(enb_j = '1') then
			reg_016_09 <= Relu_016;
		elsif(enb_k = '1') then
			reg_016_10 <= Relu_016;
		elsif(enb_l = '1') then
			reg_016_11 <= Relu_016;
		elsif(enb_m = '1') then
			reg_016_12 <= Relu_016;
		elsif(enb_n = '1') then
			reg_016_13 <= Relu_016;
		elsif(enb_o = '1') then
			reg_016_14 <= Relu_016;
		elsif(enb_p = '1') then
			reg_016_15 <= Relu_016;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_017_00 <= Relu_017;
		elsif(enb_b = '1') then
			reg_017_01 <= Relu_017;
		elsif(enb_c = '1') then
			reg_017_02 <= Relu_017;
		elsif(enb_d = '1') then
			reg_017_03 <= Relu_017;
		elsif(enb_e = '1') then
			reg_017_04 <= Relu_017;
		elsif(enb_f = '1') then
			reg_017_05 <= Relu_017;
		elsif(enb_g = '1') then
			reg_017_06 <= Relu_017;
		elsif(enb_h = '1') then
			reg_017_07 <= Relu_017;
		elsif(enb_i = '1') then
			reg_017_08 <= Relu_017;
		elsif(enb_j = '1') then
			reg_017_09 <= Relu_017;
		elsif(enb_k = '1') then
			reg_017_10 <= Relu_017;
		elsif(enb_l = '1') then
			reg_017_11 <= Relu_017;
		elsif(enb_m = '1') then
			reg_017_12 <= Relu_017;
		elsif(enb_n = '1') then
			reg_017_13 <= Relu_017;
		elsif(enb_o = '1') then
			reg_017_14 <= Relu_017;
		elsif(enb_p = '1') then
			reg_017_15 <= Relu_017;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_018_00 <= Relu_018;
		elsif(enb_b = '1') then
			reg_018_01 <= Relu_018;
		elsif(enb_c = '1') then
			reg_018_02 <= Relu_018;
		elsif(enb_d = '1') then
			reg_018_03 <= Relu_018;
		elsif(enb_e = '1') then
			reg_018_04 <= Relu_018;
		elsif(enb_f = '1') then
			reg_018_05 <= Relu_018;
		elsif(enb_g = '1') then
			reg_018_06 <= Relu_018;
		elsif(enb_h = '1') then
			reg_018_07 <= Relu_018;
		elsif(enb_i = '1') then
			reg_018_08 <= Relu_018;
		elsif(enb_j = '1') then
			reg_018_09 <= Relu_018;
		elsif(enb_k = '1') then
			reg_018_10 <= Relu_018;
		elsif(enb_l = '1') then
			reg_018_11 <= Relu_018;
		elsif(enb_m = '1') then
			reg_018_12 <= Relu_018;
		elsif(enb_n = '1') then
			reg_018_13 <= Relu_018;
		elsif(enb_o = '1') then
			reg_018_14 <= Relu_018;
		elsif(enb_p = '1') then
			reg_018_15 <= Relu_018;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_019_00 <= Relu_019;
		elsif(enb_b = '1') then
			reg_019_01 <= Relu_019;
		elsif(enb_c = '1') then
			reg_019_02 <= Relu_019;
		elsif(enb_d = '1') then
			reg_019_03 <= Relu_019;
		elsif(enb_e = '1') then
			reg_019_04 <= Relu_019;
		elsif(enb_f = '1') then
			reg_019_05 <= Relu_019;
		elsif(enb_g = '1') then
			reg_019_06 <= Relu_019;
		elsif(enb_h = '1') then
			reg_019_07 <= Relu_019;
		elsif(enb_i = '1') then
			reg_019_08 <= Relu_019;
		elsif(enb_j = '1') then
			reg_019_09 <= Relu_019;
		elsif(enb_k = '1') then
			reg_019_10 <= Relu_019;
		elsif(enb_l = '1') then
			reg_019_11 <= Relu_019;
		elsif(enb_m = '1') then
			reg_019_12 <= Relu_019;
		elsif(enb_n = '1') then
			reg_019_13 <= Relu_019;
		elsif(enb_o = '1') then
			reg_019_14 <= Relu_019;
		elsif(enb_p = '1') then
			reg_019_15 <= Relu_019;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_020_00 <= Relu_020;
		elsif(enb_b = '1') then
			reg_020_01 <= Relu_020;
		elsif(enb_c = '1') then
			reg_020_02 <= Relu_020;
		elsif(enb_d = '1') then
			reg_020_03 <= Relu_020;
		elsif(enb_e = '1') then
			reg_020_04 <= Relu_020;
		elsif(enb_f = '1') then
			reg_020_05 <= Relu_020;
		elsif(enb_g = '1') then
			reg_020_06 <= Relu_020;
		elsif(enb_h = '1') then
			reg_020_07 <= Relu_020;
		elsif(enb_i = '1') then
			reg_020_08 <= Relu_020;
		elsif(enb_j = '1') then
			reg_020_09 <= Relu_020;
		elsif(enb_k = '1') then
			reg_020_10 <= Relu_020;
		elsif(enb_l = '1') then
			reg_020_11 <= Relu_020;
		elsif(enb_m = '1') then
			reg_020_12 <= Relu_020;
		elsif(enb_n = '1') then
			reg_020_13 <= Relu_020;
		elsif(enb_o = '1') then
			reg_020_14 <= Relu_020;
		elsif(enb_p = '1') then
			reg_020_15 <= Relu_020;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_021_00 <= Relu_021;
		elsif(enb_b = '1') then
			reg_021_01 <= Relu_021;
		elsif(enb_c = '1') then
			reg_021_02 <= Relu_021;
		elsif(enb_d = '1') then
			reg_021_03 <= Relu_021;
		elsif(enb_e = '1') then
			reg_021_04 <= Relu_021;
		elsif(enb_f = '1') then
			reg_021_05 <= Relu_021;
		elsif(enb_g = '1') then
			reg_021_06 <= Relu_021;
		elsif(enb_h = '1') then
			reg_021_07 <= Relu_021;
		elsif(enb_i = '1') then
			reg_021_08 <= Relu_021;
		elsif(enb_j = '1') then
			reg_021_09 <= Relu_021;
		elsif(enb_k = '1') then
			reg_021_10 <= Relu_021;
		elsif(enb_l = '1') then
			reg_021_11 <= Relu_021;
		elsif(enb_m = '1') then
			reg_021_12 <= Relu_021;
		elsif(enb_n = '1') then
			reg_021_13 <= Relu_021;
		elsif(enb_o = '1') then
			reg_021_14 <= Relu_021;
		elsif(enb_p = '1') then
			reg_021_15 <= Relu_021;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_022_00 <= Relu_022;
		elsif(enb_b = '1') then
			reg_022_01 <= Relu_022;
		elsif(enb_c = '1') then
			reg_022_02 <= Relu_022;
		elsif(enb_d = '1') then
			reg_022_03 <= Relu_022;
		elsif(enb_e = '1') then
			reg_022_04 <= Relu_022;
		elsif(enb_f = '1') then
			reg_022_05 <= Relu_022;
		elsif(enb_g = '1') then
			reg_022_06 <= Relu_022;
		elsif(enb_h = '1') then
			reg_022_07 <= Relu_022;
		elsif(enb_i = '1') then
			reg_022_08 <= Relu_022;
		elsif(enb_j = '1') then
			reg_022_09 <= Relu_022;
		elsif(enb_k = '1') then
			reg_022_10 <= Relu_022;
		elsif(enb_l = '1') then
			reg_022_11 <= Relu_022;
		elsif(enb_m = '1') then
			reg_022_12 <= Relu_022;
		elsif(enb_n = '1') then
			reg_022_13 <= Relu_022;
		elsif(enb_o = '1') then
			reg_022_14 <= Relu_022;
		elsif(enb_p = '1') then
			reg_022_15 <= Relu_022;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_023_00 <= Relu_023;
		elsif(enb_b = '1') then
			reg_023_01 <= Relu_023;
		elsif(enb_c = '1') then
			reg_023_02 <= Relu_023;
		elsif(enb_d = '1') then
			reg_023_03 <= Relu_023;
		elsif(enb_e = '1') then
			reg_023_04 <= Relu_023;
		elsif(enb_f = '1') then
			reg_023_05 <= Relu_023;
		elsif(enb_g = '1') then
			reg_023_06 <= Relu_023;
		elsif(enb_h = '1') then
			reg_023_07 <= Relu_023;
		elsif(enb_i = '1') then
			reg_023_08 <= Relu_023;
		elsif(enb_j = '1') then
			reg_023_09 <= Relu_023;
		elsif(enb_k = '1') then
			reg_023_10 <= Relu_023;
		elsif(enb_l = '1') then
			reg_023_11 <= Relu_023;
		elsif(enb_m = '1') then
			reg_023_12 <= Relu_023;
		elsif(enb_n = '1') then
			reg_023_13 <= Relu_023;
		elsif(enb_o = '1') then
			reg_023_14 <= Relu_023;
		elsif(enb_p = '1') then
			reg_023_15 <= Relu_023;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_024_00 <= Relu_024;
		elsif(enb_b = '1') then
			reg_024_01 <= Relu_024;
		elsif(enb_c = '1') then
			reg_024_02 <= Relu_024;
		elsif(enb_d = '1') then
			reg_024_03 <= Relu_024;
		elsif(enb_e = '1') then
			reg_024_04 <= Relu_024;
		elsif(enb_f = '1') then
			reg_024_05 <= Relu_024;
		elsif(enb_g = '1') then
			reg_024_06 <= Relu_024;
		elsif(enb_h = '1') then
			reg_024_07 <= Relu_024;
		elsif(enb_i = '1') then
			reg_024_08 <= Relu_024;
		elsif(enb_j = '1') then
			reg_024_09 <= Relu_024;
		elsif(enb_k = '1') then
			reg_024_10 <= Relu_024;
		elsif(enb_l = '1') then
			reg_024_11 <= Relu_024;
		elsif(enb_m = '1') then
			reg_024_12 <= Relu_024;
		elsif(enb_n = '1') then
			reg_024_13 <= Relu_024;
		elsif(enb_o = '1') then
			reg_024_14 <= Relu_024;
		elsif(enb_p = '1') then
			reg_024_15 <= Relu_024;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_025_00 <= Relu_025;
		elsif(enb_b = '1') then
			reg_025_01 <= Relu_025;
		elsif(enb_c = '1') then
			reg_025_02 <= Relu_025;
		elsif(enb_d = '1') then
			reg_025_03 <= Relu_025;
		elsif(enb_e = '1') then
			reg_025_04 <= Relu_025;
		elsif(enb_f = '1') then
			reg_025_05 <= Relu_025;
		elsif(enb_g = '1') then
			reg_025_06 <= Relu_025;
		elsif(enb_h = '1') then
			reg_025_07 <= Relu_025;
		elsif(enb_i = '1') then
			reg_025_08 <= Relu_025;
		elsif(enb_j = '1') then
			reg_025_09 <= Relu_025;
		elsif(enb_k = '1') then
			reg_025_10 <= Relu_025;
		elsif(enb_l = '1') then
			reg_025_11 <= Relu_025;
		elsif(enb_m = '1') then
			reg_025_12 <= Relu_025;
		elsif(enb_n = '1') then
			reg_025_13 <= Relu_025;
		elsif(enb_o = '1') then
			reg_025_14 <= Relu_025;
		elsif(enb_p = '1') then
			reg_025_15 <= Relu_025;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_026_00 <= Relu_026;
		elsif(enb_b = '1') then
			reg_026_01 <= Relu_026;
		elsif(enb_c = '1') then
			reg_026_02 <= Relu_026;
		elsif(enb_d = '1') then
			reg_026_03 <= Relu_026;
		elsif(enb_e = '1') then
			reg_026_04 <= Relu_026;
		elsif(enb_f = '1') then
			reg_026_05 <= Relu_026;
		elsif(enb_g = '1') then
			reg_026_06 <= Relu_026;
		elsif(enb_h = '1') then
			reg_026_07 <= Relu_026;
		elsif(enb_i = '1') then
			reg_026_08 <= Relu_026;
		elsif(enb_j = '1') then
			reg_026_09 <= Relu_026;
		elsif(enb_k = '1') then
			reg_026_10 <= Relu_026;
		elsif(enb_l = '1') then
			reg_026_11 <= Relu_026;
		elsif(enb_m = '1') then
			reg_026_12 <= Relu_026;
		elsif(enb_n = '1') then
			reg_026_13 <= Relu_026;
		elsif(enb_o = '1') then
			reg_026_14 <= Relu_026;
		elsif(enb_p = '1') then
			reg_026_15 <= Relu_026;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_027_00 <= Relu_027;
		elsif(enb_b = '1') then
			reg_027_01 <= Relu_027;
		elsif(enb_c = '1') then
			reg_027_02 <= Relu_027;
		elsif(enb_d = '1') then
			reg_027_03 <= Relu_027;
		elsif(enb_e = '1') then
			reg_027_04 <= Relu_027;
		elsif(enb_f = '1') then
			reg_027_05 <= Relu_027;
		elsif(enb_g = '1') then
			reg_027_06 <= Relu_027;
		elsif(enb_h = '1') then
			reg_027_07 <= Relu_027;
		elsif(enb_i = '1') then
			reg_027_08 <= Relu_027;
		elsif(enb_j = '1') then
			reg_027_09 <= Relu_027;
		elsif(enb_k = '1') then
			reg_027_10 <= Relu_027;
		elsif(enb_l = '1') then
			reg_027_11 <= Relu_027;
		elsif(enb_m = '1') then
			reg_027_12 <= Relu_027;
		elsif(enb_n = '1') then
			reg_027_13 <= Relu_027;
		elsif(enb_o = '1') then
			reg_027_14 <= Relu_027;
		elsif(enb_p = '1') then
			reg_027_15 <= Relu_027;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_028_00 <= Relu_028;
		elsif(enb_b = '1') then
			reg_028_01 <= Relu_028;
		elsif(enb_c = '1') then
			reg_028_02 <= Relu_028;
		elsif(enb_d = '1') then
			reg_028_03 <= Relu_028;
		elsif(enb_e = '1') then
			reg_028_04 <= Relu_028;
		elsif(enb_f = '1') then
			reg_028_05 <= Relu_028;
		elsif(enb_g = '1') then
			reg_028_06 <= Relu_028;
		elsif(enb_h = '1') then
			reg_028_07 <= Relu_028;
		elsif(enb_i = '1') then
			reg_028_08 <= Relu_028;
		elsif(enb_j = '1') then
			reg_028_09 <= Relu_028;
		elsif(enb_k = '1') then
			reg_028_10 <= Relu_028;
		elsif(enb_l = '1') then
			reg_028_11 <= Relu_028;
		elsif(enb_m = '1') then
			reg_028_12 <= Relu_028;
		elsif(enb_n = '1') then
			reg_028_13 <= Relu_028;
		elsif(enb_o = '1') then
			reg_028_14 <= Relu_028;
		elsif(enb_p = '1') then
			reg_028_15 <= Relu_028;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_029_00 <= Relu_029;
		elsif(enb_b = '1') then
			reg_029_01 <= Relu_029;
		elsif(enb_c = '1') then
			reg_029_02 <= Relu_029;
		elsif(enb_d = '1') then
			reg_029_03 <= Relu_029;
		elsif(enb_e = '1') then
			reg_029_04 <= Relu_029;
		elsif(enb_f = '1') then
			reg_029_05 <= Relu_029;
		elsif(enb_g = '1') then
			reg_029_06 <= Relu_029;
		elsif(enb_h = '1') then
			reg_029_07 <= Relu_029;
		elsif(enb_i = '1') then
			reg_029_08 <= Relu_029;
		elsif(enb_j = '1') then
			reg_029_09 <= Relu_029;
		elsif(enb_k = '1') then
			reg_029_10 <= Relu_029;
		elsif(enb_l = '1') then
			reg_029_11 <= Relu_029;
		elsif(enb_m = '1') then
			reg_029_12 <= Relu_029;
		elsif(enb_n = '1') then
			reg_029_13 <= Relu_029;
		elsif(enb_o = '1') then
			reg_029_14 <= Relu_029;
		elsif(enb_p = '1') then
			reg_029_15 <= Relu_029;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_030_00 <= Relu_030;
		elsif(enb_b = '1') then
			reg_030_01 <= Relu_030;
		elsif(enb_c = '1') then
			reg_030_02 <= Relu_030;
		elsif(enb_d = '1') then
			reg_030_03 <= Relu_030;
		elsif(enb_e = '1') then
			reg_030_04 <= Relu_030;
		elsif(enb_f = '1') then
			reg_030_05 <= Relu_030;
		elsif(enb_g = '1') then
			reg_030_06 <= Relu_030;
		elsif(enb_h = '1') then
			reg_030_07 <= Relu_030;
		elsif(enb_i = '1') then
			reg_030_08 <= Relu_030;
		elsif(enb_j = '1') then
			reg_030_09 <= Relu_030;
		elsif(enb_k = '1') then
			reg_030_10 <= Relu_030;
		elsif(enb_l = '1') then
			reg_030_11 <= Relu_030;
		elsif(enb_m = '1') then
			reg_030_12 <= Relu_030;
		elsif(enb_n = '1') then
			reg_030_13 <= Relu_030;
		elsif(enb_o = '1') then
			reg_030_14 <= Relu_030;
		elsif(enb_p = '1') then
			reg_030_15 <= Relu_030;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_031_00 <= Relu_031;
		elsif(enb_b = '1') then
			reg_031_01 <= Relu_031;
		elsif(enb_c = '1') then
			reg_031_02 <= Relu_031;
		elsif(enb_d = '1') then
			reg_031_03 <= Relu_031;
		elsif(enb_e = '1') then
			reg_031_04 <= Relu_031;
		elsif(enb_f = '1') then
			reg_031_05 <= Relu_031;
		elsif(enb_g = '1') then
			reg_031_06 <= Relu_031;
		elsif(enb_h = '1') then
			reg_031_07 <= Relu_031;
		elsif(enb_i = '1') then
			reg_031_08 <= Relu_031;
		elsif(enb_j = '1') then
			reg_031_09 <= Relu_031;
		elsif(enb_k = '1') then
			reg_031_10 <= Relu_031;
		elsif(enb_l = '1') then
			reg_031_11 <= Relu_031;
		elsif(enb_m = '1') then
			reg_031_12 <= Relu_031;
		elsif(enb_n = '1') then
			reg_031_13 <= Relu_031;
		elsif(enb_o = '1') then
			reg_031_14 <= Relu_031;
		elsif(enb_p = '1') then
			reg_031_15 <= Relu_031;
		end if;
	end if;  
end process;


-------------------------------------------------------
ROM_W11_inst : ROM_W11 port map (
  CLK => CLK,
  ADDR => W11Addr,
  DATA => W11Data );

---------------------------------------------------------
coeff_000 <= W11Data( 1535 downto 1533 ) when W11Sel = "0000" else W11Data( 1532 downto 1530 ) when W11Sel = "0001" else W11Data( 1529 downto 1527 ) when W11Sel = "0010" else W11Data( 1526 downto 1524 ) when W11Sel = "0011" else W11Data( 1523 downto 1521 ) when W11Sel = "0100" else W11Data( 1520 downto 1518 ) when W11Sel = "0101" else W11Data( 1517 downto 1515 ) when W11Sel = "0110" else W11Data( 1514 downto 1512 ) when W11Sel = "0111" else W11Data( 1511 downto 1509 ) when W11Sel = "1000" else W11Data( 1508 downto 1506 ) when W11Sel = "1001" else W11Data( 1505 downto 1503 ) when W11Sel = "1010" else W11Data( 1502 downto 1500 ) when W11Sel = "1011" else W11Data( 1499 downto 1497 ) when W11Sel = "1100" else W11Data( 1496 downto 1494 ) when W11Sel = "1101" else W11Data( 1493 downto 1491 ) when W11Sel = "1110" else W11Data( 1490 downto 1488 );

coeff_001 <= W11Data( 1487 downto 1485 ) when W11Sel = "0000" else W11Data( 1484 downto 1482 ) when W11Sel = "0001" else W11Data( 1481 downto 1479 ) when W11Sel = "0010" else W11Data( 1478 downto 1476 ) when W11Sel = "0011" else W11Data( 1475 downto 1473 ) when W11Sel = "0100" else W11Data( 1472 downto 1470 ) when W11Sel = "0101" else W11Data( 1469 downto 1467 ) when W11Sel = "0110" else W11Data( 1466 downto 1464 ) when W11Sel = "0111" else W11Data( 1463 downto 1461 ) when W11Sel = "1000" else W11Data( 1460 downto 1458 ) when W11Sel = "1001" else W11Data( 1457 downto 1455 ) when W11Sel = "1010" else W11Data( 1454 downto 1452 ) when W11Sel = "1011" else W11Data( 1451 downto 1449 ) when W11Sel = "1100" else W11Data( 1448 downto 1446 ) when W11Sel = "1101" else W11Data( 1445 downto 1443 ) when W11Sel = "1110" else W11Data( 1442 downto 1440 );

coeff_002 <= W11Data( 1439 downto 1437 ) when W11Sel = "0000" else W11Data( 1436 downto 1434 ) when W11Sel = "0001" else W11Data( 1433 downto 1431 ) when W11Sel = "0010" else W11Data( 1430 downto 1428 ) when W11Sel = "0011" else W11Data( 1427 downto 1425 ) when W11Sel = "0100" else W11Data( 1424 downto 1422 ) when W11Sel = "0101" else W11Data( 1421 downto 1419 ) when W11Sel = "0110" else W11Data( 1418 downto 1416 ) when W11Sel = "0111" else W11Data( 1415 downto 1413 ) when W11Sel = "1000" else W11Data( 1412 downto 1410 ) when W11Sel = "1001" else W11Data( 1409 downto 1407 ) when W11Sel = "1010" else W11Data( 1406 downto 1404 ) when W11Sel = "1011" else W11Data( 1403 downto 1401 ) when W11Sel = "1100" else W11Data( 1400 downto 1398 ) when W11Sel = "1101" else W11Data( 1397 downto 1395 ) when W11Sel = "1110" else W11Data( 1394 downto 1392 );

coeff_003 <= W11Data( 1391 downto 1389 ) when W11Sel = "0000" else W11Data( 1388 downto 1386 ) when W11Sel = "0001" else W11Data( 1385 downto 1383 ) when W11Sel = "0010" else W11Data( 1382 downto 1380 ) when W11Sel = "0011" else W11Data( 1379 downto 1377 ) when W11Sel = "0100" else W11Data( 1376 downto 1374 ) when W11Sel = "0101" else W11Data( 1373 downto 1371 ) when W11Sel = "0110" else W11Data( 1370 downto 1368 ) when W11Sel = "0111" else W11Data( 1367 downto 1365 ) when W11Sel = "1000" else W11Data( 1364 downto 1362 ) when W11Sel = "1001" else W11Data( 1361 downto 1359 ) when W11Sel = "1010" else W11Data( 1358 downto 1356 ) when W11Sel = "1011" else W11Data( 1355 downto 1353 ) when W11Sel = "1100" else W11Data( 1352 downto 1350 ) when W11Sel = "1101" else W11Data( 1349 downto 1347 ) when W11Sel = "1110" else W11Data( 1346 downto 1344 );

coeff_004 <= W11Data( 1343 downto 1341 ) when W11Sel = "0000" else W11Data( 1340 downto 1338 ) when W11Sel = "0001" else W11Data( 1337 downto 1335 ) when W11Sel = "0010" else W11Data( 1334 downto 1332 ) when W11Sel = "0011" else W11Data( 1331 downto 1329 ) when W11Sel = "0100" else W11Data( 1328 downto 1326 ) when W11Sel = "0101" else W11Data( 1325 downto 1323 ) when W11Sel = "0110" else W11Data( 1322 downto 1320 ) when W11Sel = "0111" else W11Data( 1319 downto 1317 ) when W11Sel = "1000" else W11Data( 1316 downto 1314 ) when W11Sel = "1001" else W11Data( 1313 downto 1311 ) when W11Sel = "1010" else W11Data( 1310 downto 1308 ) when W11Sel = "1011" else W11Data( 1307 downto 1305 ) when W11Sel = "1100" else W11Data( 1304 downto 1302 ) when W11Sel = "1101" else W11Data( 1301 downto 1299 ) when W11Sel = "1110" else W11Data( 1298 downto 1296 );

coeff_005 <= W11Data( 1295 downto 1293 ) when W11Sel = "0000" else W11Data( 1292 downto 1290 ) when W11Sel = "0001" else W11Data( 1289 downto 1287 ) when W11Sel = "0010" else W11Data( 1286 downto 1284 ) when W11Sel = "0011" else W11Data( 1283 downto 1281 ) when W11Sel = "0100" else W11Data( 1280 downto 1278 ) when W11Sel = "0101" else W11Data( 1277 downto 1275 ) when W11Sel = "0110" else W11Data( 1274 downto 1272 ) when W11Sel = "0111" else W11Data( 1271 downto 1269 ) when W11Sel = "1000" else W11Data( 1268 downto 1266 ) when W11Sel = "1001" else W11Data( 1265 downto 1263 ) when W11Sel = "1010" else W11Data( 1262 downto 1260 ) when W11Sel = "1011" else W11Data( 1259 downto 1257 ) when W11Sel = "1100" else W11Data( 1256 downto 1254 ) when W11Sel = "1101" else W11Data( 1253 downto 1251 ) when W11Sel = "1110" else W11Data( 1250 downto 1248 );

coeff_006 <= W11Data( 1247 downto 1245 ) when W11Sel = "0000" else W11Data( 1244 downto 1242 ) when W11Sel = "0001" else W11Data( 1241 downto 1239 ) when W11Sel = "0010" else W11Data( 1238 downto 1236 ) when W11Sel = "0011" else W11Data( 1235 downto 1233 ) when W11Sel = "0100" else W11Data( 1232 downto 1230 ) when W11Sel = "0101" else W11Data( 1229 downto 1227 ) when W11Sel = "0110" else W11Data( 1226 downto 1224 ) when W11Sel = "0111" else W11Data( 1223 downto 1221 ) when W11Sel = "1000" else W11Data( 1220 downto 1218 ) when W11Sel = "1001" else W11Data( 1217 downto 1215 ) when W11Sel = "1010" else W11Data( 1214 downto 1212 ) when W11Sel = "1011" else W11Data( 1211 downto 1209 ) when W11Sel = "1100" else W11Data( 1208 downto 1206 ) when W11Sel = "1101" else W11Data( 1205 downto 1203 ) when W11Sel = "1110" else W11Data( 1202 downto 1200 );

coeff_007 <= W11Data( 1199 downto 1197 ) when W11Sel = "0000" else W11Data( 1196 downto 1194 ) when W11Sel = "0001" else W11Data( 1193 downto 1191 ) when W11Sel = "0010" else W11Data( 1190 downto 1188 ) when W11Sel = "0011" else W11Data( 1187 downto 1185 ) when W11Sel = "0100" else W11Data( 1184 downto 1182 ) when W11Sel = "0101" else W11Data( 1181 downto 1179 ) when W11Sel = "0110" else W11Data( 1178 downto 1176 ) when W11Sel = "0111" else W11Data( 1175 downto 1173 ) when W11Sel = "1000" else W11Data( 1172 downto 1170 ) when W11Sel = "1001" else W11Data( 1169 downto 1167 ) when W11Sel = "1010" else W11Data( 1166 downto 1164 ) when W11Sel = "1011" else W11Data( 1163 downto 1161 ) when W11Sel = "1100" else W11Data( 1160 downto 1158 ) when W11Sel = "1101" else W11Data( 1157 downto 1155 ) when W11Sel = "1110" else W11Data( 1154 downto 1152 );

coeff_008 <= W11Data( 1151 downto 1149 ) when W11Sel = "0000" else W11Data( 1148 downto 1146 ) when W11Sel = "0001" else W11Data( 1145 downto 1143 ) when W11Sel = "0010" else W11Data( 1142 downto 1140 ) when W11Sel = "0011" else W11Data( 1139 downto 1137 ) when W11Sel = "0100" else W11Data( 1136 downto 1134 ) when W11Sel = "0101" else W11Data( 1133 downto 1131 ) when W11Sel = "0110" else W11Data( 1130 downto 1128 ) when W11Sel = "0111" else W11Data( 1127 downto 1125 ) when W11Sel = "1000" else W11Data( 1124 downto 1122 ) when W11Sel = "1001" else W11Data( 1121 downto 1119 ) when W11Sel = "1010" else W11Data( 1118 downto 1116 ) when W11Sel = "1011" else W11Data( 1115 downto 1113 ) when W11Sel = "1100" else W11Data( 1112 downto 1110 ) when W11Sel = "1101" else W11Data( 1109 downto 1107 ) when W11Sel = "1110" else W11Data( 1106 downto 1104 );

coeff_009 <= W11Data( 1103 downto 1101 ) when W11Sel = "0000" else W11Data( 1100 downto 1098 ) when W11Sel = "0001" else W11Data( 1097 downto 1095 ) when W11Sel = "0010" else W11Data( 1094 downto 1092 ) when W11Sel = "0011" else W11Data( 1091 downto 1089 ) when W11Sel = "0100" else W11Data( 1088 downto 1086 ) when W11Sel = "0101" else W11Data( 1085 downto 1083 ) when W11Sel = "0110" else W11Data( 1082 downto 1080 ) when W11Sel = "0111" else W11Data( 1079 downto 1077 ) when W11Sel = "1000" else W11Data( 1076 downto 1074 ) when W11Sel = "1001" else W11Data( 1073 downto 1071 ) when W11Sel = "1010" else W11Data( 1070 downto 1068 ) when W11Sel = "1011" else W11Data( 1067 downto 1065 ) when W11Sel = "1100" else W11Data( 1064 downto 1062 ) when W11Sel = "1101" else W11Data( 1061 downto 1059 ) when W11Sel = "1110" else W11Data( 1058 downto 1056 );

coeff_010 <= W11Data( 1055 downto 1053 ) when W11Sel = "0000" else W11Data( 1052 downto 1050 ) when W11Sel = "0001" else W11Data( 1049 downto 1047 ) when W11Sel = "0010" else W11Data( 1046 downto 1044 ) when W11Sel = "0011" else W11Data( 1043 downto 1041 ) when W11Sel = "0100" else W11Data( 1040 downto 1038 ) when W11Sel = "0101" else W11Data( 1037 downto 1035 ) when W11Sel = "0110" else W11Data( 1034 downto 1032 ) when W11Sel = "0111" else W11Data( 1031 downto 1029 ) when W11Sel = "1000" else W11Data( 1028 downto 1026 ) when W11Sel = "1001" else W11Data( 1025 downto 1023 ) when W11Sel = "1010" else W11Data( 1022 downto 1020 ) when W11Sel = "1011" else W11Data( 1019 downto 1017 ) when W11Sel = "1100" else W11Data( 1016 downto 1014 ) when W11Sel = "1101" else W11Data( 1013 downto 1011 ) when W11Sel = "1110" else W11Data( 1010 downto 1008 );

coeff_011 <= W11Data( 1007 downto 1005 ) when W11Sel = "0000" else W11Data( 1004 downto 1002 ) when W11Sel = "0001" else W11Data( 1001 downto 999 ) when W11Sel = "0010" else W11Data( 998 downto 996 ) when W11Sel = "0011" else W11Data( 995 downto 993 ) when W11Sel = "0100" else W11Data( 992 downto 990 ) when W11Sel = "0101" else W11Data( 989 downto 987 ) when W11Sel = "0110" else W11Data( 986 downto 984 ) when W11Sel = "0111" else W11Data( 983 downto 981 ) when W11Sel = "1000" else W11Data( 980 downto 978 ) when W11Sel = "1001" else W11Data( 977 downto 975 ) when W11Sel = "1010" else W11Data( 974 downto 972 ) when W11Sel = "1011" else W11Data( 971 downto 969 ) when W11Sel = "1100" else W11Data( 968 downto 966 ) when W11Sel = "1101" else W11Data( 965 downto 963 ) when W11Sel = "1110" else W11Data( 962 downto 960 );

coeff_012 <= W11Data( 959 downto 957 ) when W11Sel = "0000" else W11Data( 956 downto 954 ) when W11Sel = "0001" else W11Data( 953 downto 951 ) when W11Sel = "0010" else W11Data( 950 downto 948 ) when W11Sel = "0011" else W11Data( 947 downto 945 ) when W11Sel = "0100" else W11Data( 944 downto 942 ) when W11Sel = "0101" else W11Data( 941 downto 939 ) when W11Sel = "0110" else W11Data( 938 downto 936 ) when W11Sel = "0111" else W11Data( 935 downto 933 ) when W11Sel = "1000" else W11Data( 932 downto 930 ) when W11Sel = "1001" else W11Data( 929 downto 927 ) when W11Sel = "1010" else W11Data( 926 downto 924 ) when W11Sel = "1011" else W11Data( 923 downto 921 ) when W11Sel = "1100" else W11Data( 920 downto 918 ) when W11Sel = "1101" else W11Data( 917 downto 915 ) when W11Sel = "1110" else W11Data( 914 downto 912 );

coeff_013 <= W11Data( 911 downto 909 ) when W11Sel = "0000" else W11Data( 908 downto 906 ) when W11Sel = "0001" else W11Data( 905 downto 903 ) when W11Sel = "0010" else W11Data( 902 downto 900 ) when W11Sel = "0011" else W11Data( 899 downto 897 ) when W11Sel = "0100" else W11Data( 896 downto 894 ) when W11Sel = "0101" else W11Data( 893 downto 891 ) when W11Sel = "0110" else W11Data( 890 downto 888 ) when W11Sel = "0111" else W11Data( 887 downto 885 ) when W11Sel = "1000" else W11Data( 884 downto 882 ) when W11Sel = "1001" else W11Data( 881 downto 879 ) when W11Sel = "1010" else W11Data( 878 downto 876 ) when W11Sel = "1011" else W11Data( 875 downto 873 ) when W11Sel = "1100" else W11Data( 872 downto 870 ) when W11Sel = "1101" else W11Data( 869 downto 867 ) when W11Sel = "1110" else W11Data( 866 downto 864 );

coeff_014 <= W11Data( 863 downto 861 ) when W11Sel = "0000" else W11Data( 860 downto 858 ) when W11Sel = "0001" else W11Data( 857 downto 855 ) when W11Sel = "0010" else W11Data( 854 downto 852 ) when W11Sel = "0011" else W11Data( 851 downto 849 ) when W11Sel = "0100" else W11Data( 848 downto 846 ) when W11Sel = "0101" else W11Data( 845 downto 843 ) when W11Sel = "0110" else W11Data( 842 downto 840 ) when W11Sel = "0111" else W11Data( 839 downto 837 ) when W11Sel = "1000" else W11Data( 836 downto 834 ) when W11Sel = "1001" else W11Data( 833 downto 831 ) when W11Sel = "1010" else W11Data( 830 downto 828 ) when W11Sel = "1011" else W11Data( 827 downto 825 ) when W11Sel = "1100" else W11Data( 824 downto 822 ) when W11Sel = "1101" else W11Data( 821 downto 819 ) when W11Sel = "1110" else W11Data( 818 downto 816 );

coeff_015 <= W11Data( 815 downto 813 ) when W11Sel = "0000" else W11Data( 812 downto 810 ) when W11Sel = "0001" else W11Data( 809 downto 807 ) when W11Sel = "0010" else W11Data( 806 downto 804 ) when W11Sel = "0011" else W11Data( 803 downto 801 ) when W11Sel = "0100" else W11Data( 800 downto 798 ) when W11Sel = "0101" else W11Data( 797 downto 795 ) when W11Sel = "0110" else W11Data( 794 downto 792 ) when W11Sel = "0111" else W11Data( 791 downto 789 ) when W11Sel = "1000" else W11Data( 788 downto 786 ) when W11Sel = "1001" else W11Data( 785 downto 783 ) when W11Sel = "1010" else W11Data( 782 downto 780 ) when W11Sel = "1011" else W11Data( 779 downto 777 ) when W11Sel = "1100" else W11Data( 776 downto 774 ) when W11Sel = "1101" else W11Data( 773 downto 771 ) when W11Sel = "1110" else W11Data( 770 downto 768 );

coeff_016 <= W11Data( 767 downto 765 ) when W11Sel = "0000" else W11Data( 764 downto 762 ) when W11Sel = "0001" else W11Data( 761 downto 759 ) when W11Sel = "0010" else W11Data( 758 downto 756 ) when W11Sel = "0011" else W11Data( 755 downto 753 ) when W11Sel = "0100" else W11Data( 752 downto 750 ) when W11Sel = "0101" else W11Data( 749 downto 747 ) when W11Sel = "0110" else W11Data( 746 downto 744 ) when W11Sel = "0111" else W11Data( 743 downto 741 ) when W11Sel = "1000" else W11Data( 740 downto 738 ) when W11Sel = "1001" else W11Data( 737 downto 735 ) when W11Sel = "1010" else W11Data( 734 downto 732 ) when W11Sel = "1011" else W11Data( 731 downto 729 ) when W11Sel = "1100" else W11Data( 728 downto 726 ) when W11Sel = "1101" else W11Data( 725 downto 723 ) when W11Sel = "1110" else W11Data( 722 downto 720 );

coeff_017 <= W11Data( 719 downto 717 ) when W11Sel = "0000" else W11Data( 716 downto 714 ) when W11Sel = "0001" else W11Data( 713 downto 711 ) when W11Sel = "0010" else W11Data( 710 downto 708 ) when W11Sel = "0011" else W11Data( 707 downto 705 ) when W11Sel = "0100" else W11Data( 704 downto 702 ) when W11Sel = "0101" else W11Data( 701 downto 699 ) when W11Sel = "0110" else W11Data( 698 downto 696 ) when W11Sel = "0111" else W11Data( 695 downto 693 ) when W11Sel = "1000" else W11Data( 692 downto 690 ) when W11Sel = "1001" else W11Data( 689 downto 687 ) when W11Sel = "1010" else W11Data( 686 downto 684 ) when W11Sel = "1011" else W11Data( 683 downto 681 ) when W11Sel = "1100" else W11Data( 680 downto 678 ) when W11Sel = "1101" else W11Data( 677 downto 675 ) when W11Sel = "1110" else W11Data( 674 downto 672 );

coeff_018 <= W11Data( 671 downto 669 ) when W11Sel = "0000" else W11Data( 668 downto 666 ) when W11Sel = "0001" else W11Data( 665 downto 663 ) when W11Sel = "0010" else W11Data( 662 downto 660 ) when W11Sel = "0011" else W11Data( 659 downto 657 ) when W11Sel = "0100" else W11Data( 656 downto 654 ) when W11Sel = "0101" else W11Data( 653 downto 651 ) when W11Sel = "0110" else W11Data( 650 downto 648 ) when W11Sel = "0111" else W11Data( 647 downto 645 ) when W11Sel = "1000" else W11Data( 644 downto 642 ) when W11Sel = "1001" else W11Data( 641 downto 639 ) when W11Sel = "1010" else W11Data( 638 downto 636 ) when W11Sel = "1011" else W11Data( 635 downto 633 ) when W11Sel = "1100" else W11Data( 632 downto 630 ) when W11Sel = "1101" else W11Data( 629 downto 627 ) when W11Sel = "1110" else W11Data( 626 downto 624 );

coeff_019 <= W11Data( 623 downto 621 ) when W11Sel = "0000" else W11Data( 620 downto 618 ) when W11Sel = "0001" else W11Data( 617 downto 615 ) when W11Sel = "0010" else W11Data( 614 downto 612 ) when W11Sel = "0011" else W11Data( 611 downto 609 ) when W11Sel = "0100" else W11Data( 608 downto 606 ) when W11Sel = "0101" else W11Data( 605 downto 603 ) when W11Sel = "0110" else W11Data( 602 downto 600 ) when W11Sel = "0111" else W11Data( 599 downto 597 ) when W11Sel = "1000" else W11Data( 596 downto 594 ) when W11Sel = "1001" else W11Data( 593 downto 591 ) when W11Sel = "1010" else W11Data( 590 downto 588 ) when W11Sel = "1011" else W11Data( 587 downto 585 ) when W11Sel = "1100" else W11Data( 584 downto 582 ) when W11Sel = "1101" else W11Data( 581 downto 579 ) when W11Sel = "1110" else W11Data( 578 downto 576 );

coeff_020 <= W11Data( 575 downto 573 ) when W11Sel = "0000" else W11Data( 572 downto 570 ) when W11Sel = "0001" else W11Data( 569 downto 567 ) when W11Sel = "0010" else W11Data( 566 downto 564 ) when W11Sel = "0011" else W11Data( 563 downto 561 ) when W11Sel = "0100" else W11Data( 560 downto 558 ) when W11Sel = "0101" else W11Data( 557 downto 555 ) when W11Sel = "0110" else W11Data( 554 downto 552 ) when W11Sel = "0111" else W11Data( 551 downto 549 ) when W11Sel = "1000" else W11Data( 548 downto 546 ) when W11Sel = "1001" else W11Data( 545 downto 543 ) when W11Sel = "1010" else W11Data( 542 downto 540 ) when W11Sel = "1011" else W11Data( 539 downto 537 ) when W11Sel = "1100" else W11Data( 536 downto 534 ) when W11Sel = "1101" else W11Data( 533 downto 531 ) when W11Sel = "1110" else W11Data( 530 downto 528 );

coeff_021 <= W11Data( 527 downto 525 ) when W11Sel = "0000" else W11Data( 524 downto 522 ) when W11Sel = "0001" else W11Data( 521 downto 519 ) when W11Sel = "0010" else W11Data( 518 downto 516 ) when W11Sel = "0011" else W11Data( 515 downto 513 ) when W11Sel = "0100" else W11Data( 512 downto 510 ) when W11Sel = "0101" else W11Data( 509 downto 507 ) when W11Sel = "0110" else W11Data( 506 downto 504 ) when W11Sel = "0111" else W11Data( 503 downto 501 ) when W11Sel = "1000" else W11Data( 500 downto 498 ) when W11Sel = "1001" else W11Data( 497 downto 495 ) when W11Sel = "1010" else W11Data( 494 downto 492 ) when W11Sel = "1011" else W11Data( 491 downto 489 ) when W11Sel = "1100" else W11Data( 488 downto 486 ) when W11Sel = "1101" else W11Data( 485 downto 483 ) when W11Sel = "1110" else W11Data( 482 downto 480 );

coeff_022 <= W11Data( 479 downto 477 ) when W11Sel = "0000" else W11Data( 476 downto 474 ) when W11Sel = "0001" else W11Data( 473 downto 471 ) when W11Sel = "0010" else W11Data( 470 downto 468 ) when W11Sel = "0011" else W11Data( 467 downto 465 ) when W11Sel = "0100" else W11Data( 464 downto 462 ) when W11Sel = "0101" else W11Data( 461 downto 459 ) when W11Sel = "0110" else W11Data( 458 downto 456 ) when W11Sel = "0111" else W11Data( 455 downto 453 ) when W11Sel = "1000" else W11Data( 452 downto 450 ) when W11Sel = "1001" else W11Data( 449 downto 447 ) when W11Sel = "1010" else W11Data( 446 downto 444 ) when W11Sel = "1011" else W11Data( 443 downto 441 ) when W11Sel = "1100" else W11Data( 440 downto 438 ) when W11Sel = "1101" else W11Data( 437 downto 435 ) when W11Sel = "1110" else W11Data( 434 downto 432 );

coeff_023 <= W11Data( 431 downto 429 ) when W11Sel = "0000" else W11Data( 428 downto 426 ) when W11Sel = "0001" else W11Data( 425 downto 423 ) when W11Sel = "0010" else W11Data( 422 downto 420 ) when W11Sel = "0011" else W11Data( 419 downto 417 ) when W11Sel = "0100" else W11Data( 416 downto 414 ) when W11Sel = "0101" else W11Data( 413 downto 411 ) when W11Sel = "0110" else W11Data( 410 downto 408 ) when W11Sel = "0111" else W11Data( 407 downto 405 ) when W11Sel = "1000" else W11Data( 404 downto 402 ) when W11Sel = "1001" else W11Data( 401 downto 399 ) when W11Sel = "1010" else W11Data( 398 downto 396 ) when W11Sel = "1011" else W11Data( 395 downto 393 ) when W11Sel = "1100" else W11Data( 392 downto 390 ) when W11Sel = "1101" else W11Data( 389 downto 387 ) when W11Sel = "1110" else W11Data( 386 downto 384 );

coeff_024 <= W11Data( 383 downto 381 ) when W11Sel = "0000" else W11Data( 380 downto 378 ) when W11Sel = "0001" else W11Data( 377 downto 375 ) when W11Sel = "0010" else W11Data( 374 downto 372 ) when W11Sel = "0011" else W11Data( 371 downto 369 ) when W11Sel = "0100" else W11Data( 368 downto 366 ) when W11Sel = "0101" else W11Data( 365 downto 363 ) when W11Sel = "0110" else W11Data( 362 downto 360 ) when W11Sel = "0111" else W11Data( 359 downto 357 ) when W11Sel = "1000" else W11Data( 356 downto 354 ) when W11Sel = "1001" else W11Data( 353 downto 351 ) when W11Sel = "1010" else W11Data( 350 downto 348 ) when W11Sel = "1011" else W11Data( 347 downto 345 ) when W11Sel = "1100" else W11Data( 344 downto 342 ) when W11Sel = "1101" else W11Data( 341 downto 339 ) when W11Sel = "1110" else W11Data( 338 downto 336 );

coeff_025 <= W11Data( 335 downto 333 ) when W11Sel = "0000" else W11Data( 332 downto 330 ) when W11Sel = "0001" else W11Data( 329 downto 327 ) when W11Sel = "0010" else W11Data( 326 downto 324 ) when W11Sel = "0011" else W11Data( 323 downto 321 ) when W11Sel = "0100" else W11Data( 320 downto 318 ) when W11Sel = "0101" else W11Data( 317 downto 315 ) when W11Sel = "0110" else W11Data( 314 downto 312 ) when W11Sel = "0111" else W11Data( 311 downto 309 ) when W11Sel = "1000" else W11Data( 308 downto 306 ) when W11Sel = "1001" else W11Data( 305 downto 303 ) when W11Sel = "1010" else W11Data( 302 downto 300 ) when W11Sel = "1011" else W11Data( 299 downto 297 ) when W11Sel = "1100" else W11Data( 296 downto 294 ) when W11Sel = "1101" else W11Data( 293 downto 291 ) when W11Sel = "1110" else W11Data( 290 downto 288 );

coeff_026 <= W11Data( 287 downto 285 ) when W11Sel = "0000" else W11Data( 284 downto 282 ) when W11Sel = "0001" else W11Data( 281 downto 279 ) when W11Sel = "0010" else W11Data( 278 downto 276 ) when W11Sel = "0011" else W11Data( 275 downto 273 ) when W11Sel = "0100" else W11Data( 272 downto 270 ) when W11Sel = "0101" else W11Data( 269 downto 267 ) when W11Sel = "0110" else W11Data( 266 downto 264 ) when W11Sel = "0111" else W11Data( 263 downto 261 ) when W11Sel = "1000" else W11Data( 260 downto 258 ) when W11Sel = "1001" else W11Data( 257 downto 255 ) when W11Sel = "1010" else W11Data( 254 downto 252 ) when W11Sel = "1011" else W11Data( 251 downto 249 ) when W11Sel = "1100" else W11Data( 248 downto 246 ) when W11Sel = "1101" else W11Data( 245 downto 243 ) when W11Sel = "1110" else W11Data( 242 downto 240 );

coeff_027 <= W11Data( 239 downto 237 ) when W11Sel = "0000" else W11Data( 236 downto 234 ) when W11Sel = "0001" else W11Data( 233 downto 231 ) when W11Sel = "0010" else W11Data( 230 downto 228 ) when W11Sel = "0011" else W11Data( 227 downto 225 ) when W11Sel = "0100" else W11Data( 224 downto 222 ) when W11Sel = "0101" else W11Data( 221 downto 219 ) when W11Sel = "0110" else W11Data( 218 downto 216 ) when W11Sel = "0111" else W11Data( 215 downto 213 ) when W11Sel = "1000" else W11Data( 212 downto 210 ) when W11Sel = "1001" else W11Data( 209 downto 207 ) when W11Sel = "1010" else W11Data( 206 downto 204 ) when W11Sel = "1011" else W11Data( 203 downto 201 ) when W11Sel = "1100" else W11Data( 200 downto 198 ) when W11Sel = "1101" else W11Data( 197 downto 195 ) when W11Sel = "1110" else W11Data( 194 downto 192 );

coeff_028 <= W11Data( 191 downto 189 ) when W11Sel = "0000" else W11Data( 188 downto 186 ) when W11Sel = "0001" else W11Data( 185 downto 183 ) when W11Sel = "0010" else W11Data( 182 downto 180 ) when W11Sel = "0011" else W11Data( 179 downto 177 ) when W11Sel = "0100" else W11Data( 176 downto 174 ) when W11Sel = "0101" else W11Data( 173 downto 171 ) when W11Sel = "0110" else W11Data( 170 downto 168 ) when W11Sel = "0111" else W11Data( 167 downto 165 ) when W11Sel = "1000" else W11Data( 164 downto 162 ) when W11Sel = "1001" else W11Data( 161 downto 159 ) when W11Sel = "1010" else W11Data( 158 downto 156 ) when W11Sel = "1011" else W11Data( 155 downto 153 ) when W11Sel = "1100" else W11Data( 152 downto 150 ) when W11Sel = "1101" else W11Data( 149 downto 147 ) when W11Sel = "1110" else W11Data( 146 downto 144 );

coeff_029 <= W11Data( 143 downto 141 ) when W11Sel = "0000" else W11Data( 140 downto 138 ) when W11Sel = "0001" else W11Data( 137 downto 135 ) when W11Sel = "0010" else W11Data( 134 downto 132 ) when W11Sel = "0011" else W11Data( 131 downto 129 ) when W11Sel = "0100" else W11Data( 128 downto 126 ) when W11Sel = "0101" else W11Data( 125 downto 123 ) when W11Sel = "0110" else W11Data( 122 downto 120 ) when W11Sel = "0111" else W11Data( 119 downto 117 ) when W11Sel = "1000" else W11Data( 116 downto 114 ) when W11Sel = "1001" else W11Data( 113 downto 111 ) when W11Sel = "1010" else W11Data( 110 downto 108 ) when W11Sel = "1011" else W11Data( 107 downto 105 ) when W11Sel = "1100" else W11Data( 104 downto 102 ) when W11Sel = "1101" else W11Data( 101 downto 99 ) when W11Sel = "1110" else W11Data( 98 downto 96 );

coeff_030 <= W11Data( 95 downto 93 ) when W11Sel = "0000" else W11Data( 92 downto 90 ) when W11Sel = "0001" else W11Data( 89 downto 87 ) when W11Sel = "0010" else W11Data( 86 downto 84 ) when W11Sel = "0011" else W11Data( 83 downto 81 ) when W11Sel = "0100" else W11Data( 80 downto 78 ) when W11Sel = "0101" else W11Data( 77 downto 75 ) when W11Sel = "0110" else W11Data( 74 downto 72 ) when W11Sel = "0111" else W11Data( 71 downto 69 ) when W11Sel = "1000" else W11Data( 68 downto 66 ) when W11Sel = "1001" else W11Data( 65 downto 63 ) when W11Sel = "1010" else W11Data( 62 downto 60 ) when W11Sel = "1011" else W11Data( 59 downto 57 ) when W11Sel = "1100" else W11Data( 56 downto 54 ) when W11Sel = "1101" else W11Data( 53 downto 51 ) when W11Sel = "1110" else W11Data( 50 downto 48 );

coeff_031 <= W11Data( 47 downto 45 ) when W11Sel = "0000" else W11Data( 44 downto 42 ) when W11Sel = "0001" else W11Data( 41 downto 39 ) when W11Sel = "0010" else W11Data( 38 downto 36 ) when W11Sel = "0011" else W11Data( 35 downto 33 ) when W11Sel = "0100" else W11Data( 32 downto 30 ) when W11Sel = "0101" else W11Data( 29 downto 27 ) when W11Sel = "0110" else W11Data( 26 downto 24 ) when W11Sel = "0111" else W11Data( 23 downto 21 ) when W11Sel = "1000" else W11Data( 20 downto 18 ) when W11Sel = "1001" else W11Data( 17 downto 15 ) when W11Sel = "1010" else W11Data( 14 downto 12 ) when W11Sel = "1011" else W11Data( 11 downto 9 ) when W11Sel = "1100" else W11Data( 8 downto 6 ) when W11Sel = "1101" else W11Data( 5 downto 3 ) when W11Sel = "1110" else W11Data( 2 downto 0 );

---------------------------------------------------------
dotpro_w11_000 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_000,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_000 );

dotpro_w11_001 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_001,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_001 );

dotpro_w11_002 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_002,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_002 );

dotpro_w11_003 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_003,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_003 );

dotpro_w11_004 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_004,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_004 );

dotpro_w11_005 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_005,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_005 );

dotpro_w11_006 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_006,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_006 );

dotpro_w11_007 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_007,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_007 );

dotpro_w11_008 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_008,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_008 );

dotpro_w11_009 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_009,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_009 );

dotpro_w11_010 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_010,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_010 );

dotpro_w11_011 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_011,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_011 );

dotpro_w11_012 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_012,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_012 );

dotpro_w11_013 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_013,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_013 );

dotpro_w11_014 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_014,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_014 );

dotpro_w11_015 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_015,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_015 );

dotpro_w11_016 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_016,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_016 );

dotpro_w11_017 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_017,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_017 );

dotpro_w11_018 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_018,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_018 );

dotpro_w11_019 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_019,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_019 );

dotpro_w11_020 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_020,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_020 );

dotpro_w11_021 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_021,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_021 );

dotpro_w11_022 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_022,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_022 );

dotpro_w11_023 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_023,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_023 );

dotpro_w11_024 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_024,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_024 );

dotpro_w11_025 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_025,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_025 );

dotpro_w11_026 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_026,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_026 );

dotpro_w11_027 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_027,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_027 );

dotpro_w11_028 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_028,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_028 );

dotpro_w11_029 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_029,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_029 );

dotpro_w11_030 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_030,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_030 );

dotpro_w11_031 : dotpro_w11 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_031,
  CO9is1 => CO9is1,
  CO512is0 => CO512is0,
  MATMULOUT => mac_031 );

-------------------------------------------------------
  truncate_000 <= mac_000(13 downto 7);
  truncate_001 <= mac_001(13 downto 7);
  truncate_002 <= mac_002(13 downto 7);
  truncate_003 <= mac_003(13 downto 7);
  truncate_004 <= mac_004(13 downto 7);
  truncate_005 <= mac_005(13 downto 7);
  truncate_006 <= mac_006(13 downto 7);
  truncate_007 <= mac_007(13 downto 7);
  truncate_008 <= mac_008(13 downto 7);
  truncate_009 <= mac_009(13 downto 7);
  truncate_010 <= mac_010(13 downto 7);
  truncate_011 <= mac_011(13 downto 7);
  truncate_012 <= mac_012(13 downto 7);
  truncate_013 <= mac_013(13 downto 7);
  truncate_014 <= mac_014(13 downto 7);
  truncate_015 <= mac_015(13 downto 7);
  truncate_016 <= mac_016(13 downto 7);
  truncate_017 <= mac_017(13 downto 7);
  truncate_018 <= mac_018(13 downto 7);
  truncate_019 <= mac_019(13 downto 7);
  truncate_020 <= mac_020(13 downto 7);
  truncate_021 <= mac_021(13 downto 7);
  truncate_022 <= mac_022(13 downto 7);
  truncate_023 <= mac_023(13 downto 7);
  truncate_024 <= mac_024(13 downto 7);
  truncate_025 <= mac_025(13 downto 7);
  truncate_026 <= mac_026(13 downto 7);
  truncate_027 <= mac_027(13 downto 7);
  truncate_028 <= mac_028(13 downto 7);
  truncate_029 <= mac_029(13 downto 7);
  truncate_030 <= mac_030(13 downto 7);
  truncate_031 <= mac_031(13 downto 7);

---------------------------------------------------------
  Relu_000 <= (others => '0') when truncate_000 < 0 else
             (others => '1') when truncate_000 > 31 else
             truncate_000(4 downto 0);

  Relu_001 <= (others => '0') when truncate_001 < 0 else
             (others => '1') when truncate_001 > 31 else
             truncate_001(4 downto 0);

  Relu_002 <= (others => '0') when truncate_002 < 0 else
             (others => '1') when truncate_002 > 31 else
             truncate_002(4 downto 0);

  Relu_003 <= (others => '0') when truncate_003 < 0 else
             (others => '1') when truncate_003 > 31 else
             truncate_003(4 downto 0);

  Relu_004 <= (others => '0') when truncate_004 < 0 else
             (others => '1') when truncate_004 > 31 else
             truncate_004(4 downto 0);

  Relu_005 <= (others => '0') when truncate_005 < 0 else
             (others => '1') when truncate_005 > 31 else
             truncate_005(4 downto 0);

  Relu_006 <= (others => '0') when truncate_006 < 0 else
             (others => '1') when truncate_006 > 31 else
             truncate_006(4 downto 0);

  Relu_007 <= (others => '0') when truncate_007 < 0 else
             (others => '1') when truncate_007 > 31 else
             truncate_007(4 downto 0);

  Relu_008 <= (others => '0') when truncate_008 < 0 else
             (others => '1') when truncate_008 > 31 else
             truncate_008(4 downto 0);

  Relu_009 <= (others => '0') when truncate_009 < 0 else
             (others => '1') when truncate_009 > 31 else
             truncate_009(4 downto 0);

  Relu_010 <= (others => '0') when truncate_010 < 0 else
             (others => '1') when truncate_010 > 31 else
             truncate_010(4 downto 0);

  Relu_011 <= (others => '0') when truncate_011 < 0 else
             (others => '1') when truncate_011 > 31 else
             truncate_011(4 downto 0);

  Relu_012 <= (others => '0') when truncate_012 < 0 else
             (others => '1') when truncate_012 > 31 else
             truncate_012(4 downto 0);

  Relu_013 <= (others => '0') when truncate_013 < 0 else
             (others => '1') when truncate_013 > 31 else
             truncate_013(4 downto 0);

  Relu_014 <= (others => '0') when truncate_014 < 0 else
             (others => '1') when truncate_014 > 31 else
             truncate_014(4 downto 0);

  Relu_015 <= (others => '0') when truncate_015 < 0 else
             (others => '1') when truncate_015 > 31 else
             truncate_015(4 downto 0);

  Relu_016 <= (others => '0') when truncate_016 < 0 else
             (others => '1') when truncate_016 > 31 else
             truncate_016(4 downto 0);

  Relu_017 <= (others => '0') when truncate_017 < 0 else
             (others => '1') when truncate_017 > 31 else
             truncate_017(4 downto 0);

  Relu_018 <= (others => '0') when truncate_018 < 0 else
             (others => '1') when truncate_018 > 31 else
             truncate_018(4 downto 0);

  Relu_019 <= (others => '0') when truncate_019 < 0 else
             (others => '1') when truncate_019 > 31 else
             truncate_019(4 downto 0);

  Relu_020 <= (others => '0') when truncate_020 < 0 else
             (others => '1') when truncate_020 > 31 else
             truncate_020(4 downto 0);

  Relu_021 <= (others => '0') when truncate_021 < 0 else
             (others => '1') when truncate_021 > 31 else
             truncate_021(4 downto 0);

  Relu_022 <= (others => '0') when truncate_022 < 0 else
             (others => '1') when truncate_022 > 31 else
             truncate_022(4 downto 0);

  Relu_023 <= (others => '0') when truncate_023 < 0 else
             (others => '1') when truncate_023 > 31 else
             truncate_023(4 downto 0);

  Relu_024 <= (others => '0') when truncate_024 < 0 else
             (others => '1') when truncate_024 > 31 else
             truncate_024(4 downto 0);

  Relu_025 <= (others => '0') when truncate_025 < 0 else
             (others => '1') when truncate_025 > 31 else
             truncate_025(4 downto 0);

  Relu_026 <= (others => '0') when truncate_026 < 0 else
             (others => '1') when truncate_026 > 31 else
             truncate_026(4 downto 0);

  Relu_027 <= (others => '0') when truncate_027 < 0 else
             (others => '1') when truncate_027 > 31 else
             truncate_027(4 downto 0);

  Relu_028 <= (others => '0') when truncate_028 < 0 else
             (others => '1') when truncate_028 > 31 else
             truncate_028(4 downto 0);

  Relu_029 <= (others => '0') when truncate_029 < 0 else
             (others => '1') when truncate_029 > 31 else
             truncate_029(4 downto 0);

  Relu_030 <= (others => '0') when truncate_030 < 0 else
             (others => '1') when truncate_030 > 31 else
             truncate_030(4 downto 0);

  Relu_031 <= (others => '0') when truncate_031 < 0 else
             (others => '1') when truncate_031 > 31 else
             truncate_031(4 downto 0);


--ReluSum <= ("0000000" & Relu_00) + ("0000000" & Relu_01)
--		+ ("0000000" & Relu_02) + ("0000000" & Relu_03)
--		+ ("0000000" & Relu_04) + ("0000000" & Relu_05)
--		+ ("0000000" & Relu_06) + ("0000000" & Relu_07)
--		+ ("0000000" & Relu_08) + ("0000000" & Relu_09)
--
--		+ ("0000000" & Relu_10) + ("0000000" & Relu_11)
--		+ ("0000000" & Relu_12) + ("0000000" & Relu_13)
--		+ ("0000000" & Relu_14) + ("0000000" & Relu_15)
--		+ ("0000000" & Relu_16) + ("0000000" & Relu_17)
--		+ ("0000000" & Relu_18) + ("0000000" & Relu_19)
--
--		+ ("0000000" & Relu_20) + ("0000000" & Relu_21)
--		+ ("0000000" & Relu_22) + ("0000000" & Relu_23)
--		+ ("0000000" & Relu_24) + ("0000000" & Relu_25)
--		+ ("0000000" & Relu_26) + ("0000000" & Relu_27)
--		+ ("0000000" & Relu_28) + ("0000000" & Relu_29)
--
--		+ ("0000000" & Relu_30) + ("0000000" & Relu_31);

ReluAll <= 
	reg_000_00 & reg_000_01 & reg_000_02 & reg_000_03 & 
	reg_000_04 & reg_000_05 & reg_000_06 & reg_000_07 & 
	reg_000_08 & reg_000_09 & reg_000_10 & reg_000_11 & 
	reg_000_12 & reg_000_13 & reg_000_14 & reg_000_15 & 

	reg_001_00 & reg_001_01 & reg_001_02 & reg_001_03 & 
	reg_001_04 & reg_001_05 & reg_001_06 & reg_001_07 & 
	reg_001_08 & reg_001_09 & reg_001_10 & reg_001_11 & 
	reg_001_12 & reg_001_13 & reg_001_14 & reg_001_15 & 

	reg_002_00 & reg_002_01 & reg_002_02 & reg_002_03 & 
	reg_002_04 & reg_002_05 & reg_002_06 & reg_002_07 & 
	reg_002_08 & reg_002_09 & reg_002_10 & reg_002_11 & 
	reg_002_12 & reg_002_13 & reg_002_14 & reg_002_15 & 

	reg_003_00 & reg_003_01 & reg_003_02 & reg_003_03 & 
	reg_003_04 & reg_003_05 & reg_003_06 & reg_003_07 & 
	reg_003_08 & reg_003_09 & reg_003_10 & reg_003_11 & 
	reg_003_12 & reg_003_13 & reg_003_14 & reg_003_15 & 

	reg_004_00 & reg_004_01 & reg_004_02 & reg_004_03 & 
	reg_004_04 & reg_004_05 & reg_004_06 & reg_004_07 & 
	reg_004_08 & reg_004_09 & reg_004_10 & reg_004_11 & 
	reg_004_12 & reg_004_13 & reg_004_14 & reg_004_15 & 

	reg_005_00 & reg_005_01 & reg_005_02 & reg_005_03 & 
	reg_005_04 & reg_005_05 & reg_005_06 & reg_005_07 & 
	reg_005_08 & reg_005_09 & reg_005_10 & reg_005_11 & 
	reg_005_12 & reg_005_13 & reg_005_14 & reg_005_15 & 

	reg_006_00 & reg_006_01 & reg_006_02 & reg_006_03 & 
	reg_006_04 & reg_006_05 & reg_006_06 & reg_006_07 & 
	reg_006_08 & reg_006_09 & reg_006_10 & reg_006_11 & 
	reg_006_12 & reg_006_13 & reg_006_14 & reg_006_15 & 

	reg_007_00 & reg_007_01 & reg_007_02 & reg_007_03 & 
	reg_007_04 & reg_007_05 & reg_007_06 & reg_007_07 & 
	reg_007_08 & reg_007_09 & reg_007_10 & reg_007_11 & 
	reg_007_12 & reg_007_13 & reg_007_14 & reg_007_15 & 

	reg_008_00 & reg_008_01 & reg_008_02 & reg_008_03 & 
	reg_008_04 & reg_008_05 & reg_008_06 & reg_008_07 & 
	reg_008_08 & reg_008_09 & reg_008_10 & reg_008_11 & 
	reg_008_12 & reg_008_13 & reg_008_14 & reg_008_15 & 

	reg_009_00 & reg_009_01 & reg_009_02 & reg_009_03 & 
	reg_009_04 & reg_009_05 & reg_009_06 & reg_009_07 & 
	reg_009_08 & reg_009_09 & reg_009_10 & reg_009_11 & 
	reg_009_12 & reg_009_13 & reg_009_14 & reg_009_15 & 

	reg_010_00 & reg_010_01 & reg_010_02 & reg_010_03 & 
	reg_010_04 & reg_010_05 & reg_010_06 & reg_010_07 & 
	reg_010_08 & reg_010_09 & reg_010_10 & reg_010_11 & 
	reg_010_12 & reg_010_13 & reg_010_14 & reg_010_15 & 

	reg_011_00 & reg_011_01 & reg_011_02 & reg_011_03 & 
	reg_011_04 & reg_011_05 & reg_011_06 & reg_011_07 & 
	reg_011_08 & reg_011_09 & reg_011_10 & reg_011_11 & 
	reg_011_12 & reg_011_13 & reg_011_14 & reg_011_15 & 

	reg_012_00 & reg_012_01 & reg_012_02 & reg_012_03 & 
	reg_012_04 & reg_012_05 & reg_012_06 & reg_012_07 & 
	reg_012_08 & reg_012_09 & reg_012_10 & reg_012_11 & 
	reg_012_12 & reg_012_13 & reg_012_14 & reg_012_15 & 

	reg_013_00 & reg_013_01 & reg_013_02 & reg_013_03 & 
	reg_013_04 & reg_013_05 & reg_013_06 & reg_013_07 & 
	reg_013_08 & reg_013_09 & reg_013_10 & reg_013_11 & 
	reg_013_12 & reg_013_13 & reg_013_14 & reg_013_15 & 

	reg_014_00 & reg_014_01 & reg_014_02 & reg_014_03 & 
	reg_014_04 & reg_014_05 & reg_014_06 & reg_014_07 & 
	reg_014_08 & reg_014_09 & reg_014_10 & reg_014_11 & 
	reg_014_12 & reg_014_13 & reg_014_14 & reg_014_15 & 

	reg_015_00 & reg_015_01 & reg_015_02 & reg_015_03 & 
	reg_015_04 & reg_015_05 & reg_015_06 & reg_015_07 & 
	reg_015_08 & reg_015_09 & reg_015_10 & reg_015_11 & 
	reg_015_12 & reg_015_13 & reg_015_14 & reg_015_15 & 

	reg_016_00 & reg_016_01 & reg_016_02 & reg_016_03 & 
	reg_016_04 & reg_016_05 & reg_016_06 & reg_016_07 & 
	reg_016_08 & reg_016_09 & reg_016_10 & reg_016_11 & 
	reg_016_12 & reg_016_13 & reg_016_14 & reg_016_15 & 

	reg_017_00 & reg_017_01 & reg_017_02 & reg_017_03 & 
	reg_017_04 & reg_017_05 & reg_017_06 & reg_017_07 & 
	reg_017_08 & reg_017_09 & reg_017_10 & reg_017_11 & 
	reg_017_12 & reg_017_13 & reg_017_14 & reg_017_15 & 

	reg_018_00 & reg_018_01 & reg_018_02 & reg_018_03 & 
	reg_018_04 & reg_018_05 & reg_018_06 & reg_018_07 & 
	reg_018_08 & reg_018_09 & reg_018_10 & reg_018_11 & 
	reg_018_12 & reg_018_13 & reg_018_14 & reg_018_15 & 

	reg_019_00 & reg_019_01 & reg_019_02 & reg_019_03 & 
	reg_019_04 & reg_019_05 & reg_019_06 & reg_019_07 & 
	reg_019_08 & reg_019_09 & reg_019_10 & reg_019_11 & 
	reg_019_12 & reg_019_13 & reg_019_14 & reg_019_15 & 

	reg_020_00 & reg_020_01 & reg_020_02 & reg_020_03 & 
	reg_020_04 & reg_020_05 & reg_020_06 & reg_020_07 & 
	reg_020_08 & reg_020_09 & reg_020_10 & reg_020_11 & 
	reg_020_12 & reg_020_13 & reg_020_14 & reg_020_15 & 

	reg_021_00 & reg_021_01 & reg_021_02 & reg_021_03 & 
	reg_021_04 & reg_021_05 & reg_021_06 & reg_021_07 & 
	reg_021_08 & reg_021_09 & reg_021_10 & reg_021_11 & 
	reg_021_12 & reg_021_13 & reg_021_14 & reg_021_15 & 

	reg_022_00 & reg_022_01 & reg_022_02 & reg_022_03 & 
	reg_022_04 & reg_022_05 & reg_022_06 & reg_022_07 & 
	reg_022_08 & reg_022_09 & reg_022_10 & reg_022_11 & 
	reg_022_12 & reg_022_13 & reg_022_14 & reg_022_15 & 

	reg_023_00 & reg_023_01 & reg_023_02 & reg_023_03 & 
	reg_023_04 & reg_023_05 & reg_023_06 & reg_023_07 & 
	reg_023_08 & reg_023_09 & reg_023_10 & reg_023_11 & 
	reg_023_12 & reg_023_13 & reg_023_14 & reg_023_15 & 

	reg_024_00 & reg_024_01 & reg_024_02 & reg_024_03 & 
	reg_024_04 & reg_024_05 & reg_024_06 & reg_024_07 & 
	reg_024_08 & reg_024_09 & reg_024_10 & reg_024_11 & 
	reg_024_12 & reg_024_13 & reg_024_14 & reg_024_15 & 

	reg_025_00 & reg_025_01 & reg_025_02 & reg_025_03 & 
	reg_025_04 & reg_025_05 & reg_025_06 & reg_025_07 & 
	reg_025_08 & reg_025_09 & reg_025_10 & reg_025_11 & 
	reg_025_12 & reg_025_13 & reg_025_14 & reg_025_15 & 

	reg_026_00 & reg_026_01 & reg_026_02 & reg_026_03 & 
	reg_026_04 & reg_026_05 & reg_026_06 & reg_026_07 & 
	reg_026_08 & reg_026_09 & reg_026_10 & reg_026_11 & 
	reg_026_12 & reg_026_13 & reg_026_14 & reg_026_15 & 

	reg_027_00 & reg_027_01 & reg_027_02 & reg_027_03 & 
	reg_027_04 & reg_027_05 & reg_027_06 & reg_027_07 & 
	reg_027_08 & reg_027_09 & reg_027_10 & reg_027_11 & 
	reg_027_12 & reg_027_13 & reg_027_14 & reg_027_15 & 

	reg_028_00 & reg_028_01 & reg_028_02 & reg_028_03 & 
	reg_028_04 & reg_028_05 & reg_028_06 & reg_028_07 & 
	reg_028_08 & reg_028_09 & reg_028_10 & reg_028_11 & 
	reg_028_12 & reg_028_13 & reg_028_14 & reg_028_15 & 

	reg_029_00 & reg_029_01 & reg_029_02 & reg_029_03 & 
	reg_029_04 & reg_029_05 & reg_029_06 & reg_029_07 & 
	reg_029_08 & reg_029_09 & reg_029_10 & reg_029_11 & 
	reg_029_12 & reg_029_13 & reg_029_14 & reg_029_15 & 

	reg_030_00 & reg_030_01 & reg_030_02 & reg_030_03 & 
	reg_030_04 & reg_030_05 & reg_030_06 & reg_030_07 & 
	reg_030_08 & reg_030_09 & reg_030_10 & reg_030_11 & 
	reg_030_12 & reg_030_13 & reg_030_14 & reg_030_15 & 

	reg_031_00 & reg_031_01 & reg_031_02 & reg_031_03 & 
	reg_031_04 & reg_031_05 & reg_031_06 & reg_031_07 & 
	reg_031_08 & reg_031_09 & reg_031_10 & reg_031_11 & 
	reg_031_12 & reg_031_13 & reg_031_14 & reg_031_15 ; 


ReluAllWrN <= not enb_a;

end RTL;

