-- A module of Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2019/02/13

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity dotpro_w11 is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  PixRomData : in std_logic_vector(4 downto 0); 
  MatRomData : in std_logic_vector(2 downto 0);
  CO9is1 : in std_logic;
  CO512is0 : in std_logic;
  MATMULOUT : Out std_logic_vector(13 downto 0) );  
end dotpro_w11;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of dotpro_w11 is

component SHIFTMUL_3B2
   Port (
  COEFF : in std_logic_vector(2 downto 0); 
  PIXDATA : in std_logic_vector(4 downto 0);
  MULOUT : Out std_logic_vector(9 downto 0) );  
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal mulout : std_logic_vector(9 downto 0);
signal mulout_dly : std_logic_vector(13 downto 0);
signal addout_dly_clr : std_logic;
signal addout, addout_dly : std_logic_vector(13 downto 0);

begin

---------------------------------------------------------
--  mulout <= PixRomData * MatRomData;
SHIFTMUL_3B2_inst : SHIFTMUL_3B2 port map (
  COEFF => MatRomData,
  PIXDATA => PixRomData,
  MULOUT => mulout );

---------------------------------------------------------
--process( RST_N, CLK )  
--begin 
--	if( RST_N = '0' ) then
--		mulout_dly <= (others => '0');
--	elsif( CLK'event and CLK = '1' ) then
		mulout_dly <= mulout(9) & mulout(9) & mulout(9) &  mulout(9) & mulout;
--	end if;  
--end process;

---------------------------------------------------------
  addout <= mulout_dly + addout_dly;

  addout_dly_clr <= '1' when (CO9is1 = '1' and CO512is0 = '1') else '0';

process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		addout_dly <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		if( addout_dly_clr = '1' ) then
			addout_dly <= (others => '0');
		else 
			addout_dly <= addout;
		end if;
	end if;  
end process;

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		MATMULOUT <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		if( addout_dly_clr = '1' ) then
			MATMULOUT <= addout;
		end if;
	end if;  
end process;

end RTL;

