--------------------------------------------------
-- env.vhd
-- by Toshio Iwata at digitalfilter.com 2004/09/09

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.STD_LOGIC_UNSIGNED.ALL;
  use IEEE.std_logic_arith.all;
  use STD.TEXTIO.all;
  use ieee.std_logic_textio.all;

entity env is
   Port (
          CLK : out std_logic; 
          CLK100 : out std_logic; 
          RST_N : out std_logic;
          DISPJUDGE : out std_logic;
          PCLK : out std_logic;
  ReluWrN : in std_logic;
  ReluWrData : in std_logic_vector(2559 downto 0);
  ReluWrAddr : in std_logic_vector(4 downto 0);
  ReluWrSel : in std_logic_vector(1 downto 0);
  LayNextWrN : out std_logic;
  LayNextWrData : out std_logic_vector(2559 downto 0);
  LayNextWrAddr : out std_logic_vector(4 downto 0);
  LayNextWrSel : out std_logic_vector(1 downto 0);
          CamHsync : out std_logic;
          CamVsync : out std_logic;
          CamData : Out std_logic_vector(7 downto 0) );
end env;

architecture beh of env is

  constant sv_clk_period : time := 20 ns; -- 50MHz -- 2012/06/28
  constant sv_clk_value : real := 20.0e-9; -- 50MHz

  constant sv_pclk_period : time := 40 ns; -- 25MHz -- 2012/06/28
  constant sv_pclk_value : real := 40.0e-9; -- 25MHz

  constant sv_chsync_period : time := 128000 ns; -- 80ns * 1600 (128us, 7.8khz)
  constant sv_chsync_value : real := 128000.0e-9; -- 2018/07/13

  constant sv_cvsync_period : time := 64000000 ns; -- 80ns * 1600 * 500 (64ms, 15.6hz)
  constant sv_cvsync_value : real := 64000000.0e-9; -- 2018/07/13 

signal yin_sig : std_logic_vector(7 downto 0);
signal lineco	: integer := 0;
signal CamHsync_sig	: std_logic;
signal DISPJUDGE_sig	: std_logic;
signal ReluWrN_sig	: std_logic;

begin

process
begin
  CLK <= '0';
  wait for sv_clk_period / 2.0;
  CLK <= '1';
  wait for sv_clk_period - sv_clk_period / 2.0;
end process;

process
variable first_time : boolean := true;
begin
  if(first_time) then
    wait for 1 ns;
    first_time := false;
  end if;
  CLK100 <= '0';
  wait for sv_clk_period / 4.0;
  CLK100 <= '1';
  wait for sv_clk_period / 2.0 - sv_clk_period / 4.0;
end process;

process
variable first_time : boolean := true;
begin
  if( first_time ) then
    wait for sv_pclk_period/4;
    first_time := false;
  end if;
  PCLK <= '1';
  wait for sv_pclk_period / 2.0;
  PCLK <= '0';
  wait for sv_pclk_period - sv_pclk_period / 2.0;
end process;

process
begin
  if(lineco = 499) then
    lineco <= 0;
  else
    lineco <= lineco + 1;
  end if;
  CamHsync_sig <= '0';
  wait for sv_pclk_period * 1920;
  CamHsync_sig <= '1';
  wait for sv_chsync_period - sv_pclk_period * 1920;
end process;

  CamHsync <= CamHsync_sig when lineco >= 10 and lineco < 490 else '0'; -- from 10 to 5 to speed up 2020/01/02

process
begin
--  lineco <= 0;
  CamVsync <= '1';
  wait for sv_chsync_period * 2;
  CamVsync <= '0';
  wait for sv_cvsync_period - sv_chsync_period * 2;
end process;

process
variable first_time : boolean := true;
variable pi : real := 3.14159;
variable t : real := 0.0;
variable idata : integer := 0;
begin
  if( first_time ) then
    wait for sv_pclk_period/2;
    first_time := false;
  end if;
  if(idata = 255) then
    idata := 0;
  else 
    idata := idata + 1;
  end if;
  yin_sig <= conv_std_logic_vector(idata, 8);
  wait for sv_pclk_period;
  t := t + sv_pclk_value;
end process;

  CamData <= yin_sig;

process
begin
   RST_N <= '0';
   wait for 1 us;
   RST_N <= '1';
   wait;
end process ;

process
begin
   DISPJUDGE_sig <= '0';
   wait for 330 us;
   DISPJUDGE_sig <= '1';
   wait;
end process ;

DISPJUDGE <= DISPJUDGE_sig;

ReluWrN_sig <= ReluWrN after 1 ns; -- to catch reluwrdata

process
variable Lo : line;
file Outv : TEXT is out "relu.out";
begin
  wait until ReluWrN_sig = '0';
  if (now > 100 ns) then
    write(Lo, NOW, RIGHT, 15);
    write(Lo, ReluWrSel, RIGHT, 5);
    write(Lo, ReluWrAddr, RIGHT, 15);
    write(Lo, ReluWrData, RIGHT, 2565);
    writeline(Outv, Lo);
  end if;
  wait until ReluWrN_sig = '1';
end process ;

process
variable Li : line;
file Inv : TEXT is in "lay11.in";
variable tm : Time;
variable WrN : std_logic;
variable WrData : std_logic_vector(2559 downto 0);
variable WrAddr : std_logic_vector(5 downto 0);
variable WrSel : std_logic_vector(1 downto 0);
variable first_time : boolean := true;
variable co19 : integer := 0;
begin
  if(first_time) then
    --wait until DISPJUDGE_sig = '1';
    LayNextWrSel <= (others => '0');
    LayNextWrAddr <= (others => '0');
    LayNextWrData <= (others => '0');
    LayNextWrN <= '1';
    wait for 28293000 ns;
    first_time := false;
  end if;
--  wait until ReluWrN = '0';

  wait for 368630 ns;
  WrN := '0';
  LayNextWrN <= WrN;

  readline(inv, Li);
  read(Li, tm);
  read(Li, WrSel);
  read(Li, WrAddr);
  read(Li, WrData);

  LayNextWrSel <= WrSel;
  LayNextWrAddr <= WrAddr(4 downto 0);
  LayNextWrData <= WrData;

  wait for 10 ns;
  WrN := '1';
  LayNextWrN <= WrN;
  if(co19 = 18) then
    wait for 20643.84 us; -- 368.64us x 56(75-19)
    co19 := 0;
  else
    co19 := co19 + 1;
  end if;

end process ;

end beh;
