-- Test Bench
-- Designed by T.Iwata at DIGITALFILTER.COM 2007/09/18

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;

entity tb_VideoProc is
end tb_VideoProc ;

architecture beh of tb_VideoProc is

component VideoProcCore
	PORT	(
	CLK	: IN std_logic;
	CLK100M	: IN std_logic;
	RST_N	: IN std_logic;
	DISPJUDGE	: IN std_logic;
	mdet_enb	: IN std_logic;
  ViewMode : in std_logic_vector(3 downto 0);
	CamHsync	: IN std_logic;
	CamVsync	: IN std_logic;
	PCLK	: IN std_logic;
  CamData	: in std_logic_vector(7 downto 0);
	MDET_L : out std_logic;
	  PIXSTB : in std_logic;
  PIXRD : in std_logic;
  PIXWR : in std_logic;
  PIX2NIOS : out std_logic_vector(15 downto 0);
  PIX2LOGIC : in std_logic_vector(15 downto 0);
  LBRDY : out std_logic;
  CurrentFrame : out std_logic_vector(2 downto 0);
  NeededFrame : in std_logic_vector(2 downto 0);
	SDRAM_DQENB_N : out std_logic;
  SDRAM_DQMH	: out std_logic;
  SDRAM_DQML	: out std_logic;
  SDRAM_CS_N	: out std_logic;
  SDRAM_WE_N	: out std_logic;
  SDRAM_RAS_N	: out std_logic;
  SDRAM_CAS_N	: out std_logic;
  SDRAM_CKE	: out std_logic;
  SDRAM_DQI : in std_logic_vector(15 downto 0);
  SDRAM_DQO : out std_logic_vector(15 downto 0);
  SDRAM_A : out std_logic_vector(11 downto 0);
  SDRAM_BA : out std_logic_vector(1 downto 0);
  SDRAM_CLK : out std_logic;
  ReluWrN : out std_logic;
  ReluWrData : out std_logic_vector(2559 downto 0);
  ReluWrAddr : out std_logic_vector(4 downto 0);
  ReluWrSel : out std_logic_vector(1 downto 0);
  LayNextWrN : in std_logic;
  LayNextWrData : in std_logic_vector(2559 downto 0);
  LayNextWrAddr : in std_logic_vector(4 downto 0);
  LayNextWrSel : in std_logic_vector(1 downto 0);
	VgaVsync : OUT std_logic;
	VgaHsync : OUT std_logic;
   VgaDataR	: OUT std_logic_vector(7 downto 0);
   VgaDataG	: OUT std_logic_vector(7 downto 0);
   VgaDataB	: OUT std_logic_vector(7 downto 0) );
end component;

component env
	Port (
          CLK : out std_logic; 
          CLK100 : out std_logic; 
          RST_N : out std_logic;
          DISPJUDGE: out std_logic;
          PCLK : out std_logic;
  ReluWrN : in std_logic;
  ReluWrData : in std_logic_vector(2559 downto 0);
  ReluWrAddr : in std_logic_vector(4 downto 0);
  ReluWrSel : in std_logic_vector(1 downto 0);
  LayNextWrN : out std_logic;
  LayNextWrData : out std_logic_vector(2559 downto 0);
  LayNextWrAddr : out std_logic_vector(4 downto 0);
  LayNextWrSel : out std_logic_vector(1 downto 0);
          CamHsync : out std_logic;
          CamVsync : out std_logic;
          CamData : Out std_logic_vector(7 downto 0) );
end component;

signal rst_n_sig, DISPJUDGE_sig : std_logic;
signal mclk_sig, mclk100_sig : std_logic;
signal fsclk_sig : std_logic;
signal packet_sig : std_logic;
signal strobe_sig : std_logic;
signal yin_sig : std_logic_vector(7 downto 0);
signal dummy_high : std_logic;
signal sdata_sig : std_logic;
signal latch0_sig : std_logic;
signal trigger_sig : std_logic;
signal CamHsync_sig, CamVsync_sig, pclk_sig : std_logic;

signal  SDRAM_DQENB_N : std_logic;
signal   SDRAM_DQMH	: std_logic;
signal   SDRAM_DQML	: std_logic;
signal   SDRAM_CS_N	: std_logic;
signal   SDRAM_WE_N	: std_logic;
signal   SDRAM_RAS_N	: std_logic;
signal   SDRAM_CAS_N	: std_logic;
signal   SDRAM_CKE	: std_logic;
signal   SDRAM_CLK : std_logic;
signal   SDRAM_DQI, SDRAM_DQO : std_logic_vector(15 downto 0);
signal   SDRAM_A : std_logic_vector(11 downto 0);
signal   SDRAM_BA : std_logic_vector(1 downto 0);
signal  SDRAM_DQ_ENB : std_logic;
signal dq : std_logic_vector(15 downto 0);
signal wen : std_logic;
signal rasn : std_logic;
signal csn : std_logic;
signal casn : std_logic;
signal daddr : std_logic_vector(11 downto 0);
signal dq_out_dly : std_logic_vector(15 downto 0);
signal pix2nios_sig, pix2logic_sig : std_logic_vector(15 downto 0);
signal pixstb_sig, pixrd_sig, pixwr_sig : std_logic;
signal dvi_data0_p, dvi_data0_n		: std_logic;		-- GPIO1_D17
signal dvi_data1_p, dvi_data1_n		: std_logic;		-- GPIO1_D21
signal dvi_data2_p, dvi_data2_n		: std_logic;		-- GPIO1_D12
signal dvi_clock_p, dvi_clock_n		: std_logic;		-- GPIO1_D9
signal DCLK, SCLK : std_logic;
signal data0p_h, data0p_l, data0n_h, data0n_l : std_logic;	
signal data1p_h, data1p_l, data1n_h, data1n_l : std_logic;
signal data2p_h, data2p_l, data2n_h, data2n_l : std_logic;
signal clockp_h, clockp_l, clockn_h, clockn_l : std_logic;
signal ViewMode : std_logic_vector(3 downto 0);
signal ReluWrN : std_logic;
signal ReluWrData : std_logic_vector(2559 downto 0);
signal ReluWrAddr : std_logic_vector(4 downto 0);
signal ReluWrSel : std_logic_vector(1 downto 0);
signal LayNextWrN : std_logic;
signal LayNextWrData : std_logic_vector(2559 downto 0);
signal LayNextWrAddr : std_logic_vector(4 downto 0);
signal LayNextWrSel : std_logic_vector(1 downto 0);

begin
  
  dummy_high <= '1';
  
VideoProcCore_1 : VideoProcCore port map (
		CLK	=> mclk_sig,
		CLK100M	=> mclk100_sig,
		RST_N	=> rst_n_sig,
     DISPJUDGE => DISPJUDGE_sig,
		mdet_enb => dummy_high,
		ViewMode => "0000",--ViewMode,
	CamHsync	=> CamHsync_sig,
	CamVsync	=> CamVsync_sig,
	PCLK	=> pclk_sig,
  CamData	=> yin_sig,
  MDET_L => open, 
  PIXSTB => '0',--pixstb_sig,
  PIXRD => '0',--pixrd_sig,
  PIXWR => '0',--pixwr_sig,
  PIX2NIOS => pix2nios_sig,
  PIX2LOGIC => PIX2LOGIC_sig,
LBRDY => open,
  CurrentFrame => open, --CurrentFrame,
  NeededFrame => "001", --NeededFrame,
     SDRAM_DQENB_N => SDRAM_DQENB_N,
  SDRAM_DQMH	=> SDRAM_DQMH,
  SDRAM_DQML	=> SDRAM_DQML,
  SDRAM_CS_N	=> SDRAM_CS_N,
  SDRAM_WE_N	=> SDRAM_WE_N,
  SDRAM_RAS_N	=> SDRAM_RAS_N,
  SDRAM_CAS_N	=> SDRAM_CAS_N,
  SDRAM_CKE	=> SDRAM_CKE,
  SDRAM_DQI => SDRAM_DQI,
  SDRAM_DQO => SDRAM_DQO,
  SDRAM_A => SDRAM_A,
  SDRAM_BA => SDRAM_BA,
  SDRAM_CLK	=> SDRAM_CLK,
  ReluWrN => ReluWrN, 
  ReluWrData => ReluWrData, 
  ReluWrAddr => ReluWrAddr, 
  ReluWrSel => ReluWrSel, 
  LayNextWrN => LayNextWrN, 
  LayNextWrData => LayNextWrData, 
  LayNextWrAddr => LayNextWrAddr, 
  LayNextWrSel => LayNextWrSel, 
      VgaVsync => open,
      VgaHsync => open,
      VgaDataR => open, 
		VgaDataG => open, 
		VgaDataB => open );


env_1 : env port map (
     CLK => mclk_sig, 
     CLK100 => mclk100_sig, 
     RST_N => rst_n_sig,
     DISPJUDGE => DISPJUDGE_sig,
          PCLK => pclk_sig,
  ReluWrN => ReluWrN, 
  ReluWrData => ReluWrData, 
  ReluWrAddr => ReluWrAddr, 
  ReluWrSel => ReluWrSel, 
  LayNextWrN => LayNextWrN, 
  LayNextWrData => LayNextWrData, 
  LayNextWrAddr => LayNextWrAddr, 
  LayNextWrSel => LayNextWrSel, 
          CamHsync => CamHsync_sig,
          CamVsync => CamVsync_sig,
     CamData => yin_sig );

MAIN_P: process

begin -- process MAIN_P

	wait;

end process ;  -- MAIN_P

end beh ;
